// Alarm Daemon for KOrganizer
// (c) 1998 Preston Brown
// 	$Id: alarmdaemon.cpp,v 1.12 1998/08/18 19:23:09 pbrown Exp $	

#include <qtimer.h>
#include <qdatetm.h>
#include <qstring.h>
#include <qmsgbox.h>
#include <kapp.h>

#include "alarmdaemon.h"
#include "alarmdaemon.moc"

AlarmDaemon::AlarmDaemon(const char *fn, 
			 QObject *parent, const char *name) 
  : QObject(parent, name)
{
  calendar = new CalObject;
  fileName = fn;

  calendar->load(fileName);
  
  // set up the alarm timer
  QTimer *alarmTimer = new QTimer(this);
  
  connect(alarmTimer, SIGNAL(timeout()),
	  calendar, SLOT(checkAlarms()));
  connect(calendar, SIGNAL(alarmSignal(QList<KDPEvent> &)),
    this, SLOT(showAlarms(QList<KDPEvent> &)));
  
  alarmTimer->start(1000*60); // every minute
}

AlarmDaemon::~AlarmDaemon()
{
  calendar->close();
  delete calendar;
}

void AlarmDaemon::reloadCal()
{
  calendar->close();
  calendar->load(fileName);
}

void AlarmDaemon::showAlarms(QList<KDPEvent> &alarmEvents)
{
  KDPEvent *anEvent;
  QString messageStr, titleStr;
  QDateTime tmpDT;

  tmpDT = alarmEvents.first()->getAlarmTime();

  titleStr.sprintf(i18n("KOrganizer Alarm: %s\n"),tmpDT.toString().data());
  messageStr += i18n("The following events triggered alarms:\n\n");

  for (anEvent = alarmEvents.first(); anEvent;
       anEvent = alarmEvents.next())
    messageStr += anEvent->getSummary() + "\n";
  
  qApp->beep();
  QMessageBox::information(0, titleStr.data(), 
			   messageStr.data(),
			   QMessageBox::Ok | QMessageBox::Default);
}

