/*

   	 $Id: calendarlatexprinter.cpp,v 1.6 1998/08/05 19:48:23 pbrown Exp $	

	 Requires the Qt and KDE widget libraries, available at no cost at
	 http://www.troll.no and http://www.kde.org respectively
	 
	 Copyright (C) 1997, 1998 
	 Preston Brown (preston.brown@yale.edu)
	 Fester Zigter, man (F.J.F.ZigtermanRustenburg@student.utwente.nl)
	 Ian Dawes (iadawes@globalserve.net)
	 Neil Hart (nhart@magma.ca)
	 Laszlo Boloni (boloni@cs.purdue.edu)
	 
	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.
	 
	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
	 GNU General Public License for more details.
	 
	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <kapp.h>
#include <qmsgbox.h>
#include <qfile.h>

#include "calendarlatexprinter.h"
#include "calendarlatexprinter.moc"

// Main application
extern KApplication *korganizer;


CalendarLatexPrinter *CalendarLatexPrinter::_instance = 0;


CalendarLatexPrinter* CalendarLatexPrinter::getInstance(CalObject *cal) {    
    if (!cal) {
	delete _instance;
	return NULL;
	}
    if (_instance == 0) {
	_instance = new CalendarLatexPrinter(cal);
    } else {
	_instance->calendar = cal;
    }
    return _instance;
}


CalendarLatexPrinter::CalendarLatexPrinter(CalObject *cal) {
    config = kapp->getConfig();
    print_completed_todos = TRUE;
    dvi_preview = FALSE;
    use_ulem = TRUE;
    //
    calendar = cal;
    // temporary filename
    strcpy(filename,"korganizer");    

}

// CalendarLatexPrinter is a singleton so this will be called only once.
CalendarLatexPrinter::~CalendarLatexPrinter() {
    // delete the temporary files
    QString str;
    debug("Here I delete the temporary files!");
    str.sprintf("%s.tex", filename);
    unlink(str.data());
    str.sprintf("%s.log", filename);
    unlink(str.data());
    str.sprintf("%s.aux", filename);
    unlink(str.data());
    str.sprintf("%s.dvi", filename);
    unlink(str.data());
    str.sprintf("%s.ps", filename);
    unlink(str.data());
}

void CalendarLatexPrinter::setDates(QDate from, QDate to) {
    // some thing to verify here...
    start = from;
    end = to;
};


/**
 *
 *  Generate a temporary latex file
 *
 */
void CalendarLatexPrinter::generate() {
    char buffer[1000];
    sprintf(buffer,"%s.tex",filename);
    if (!(file = fopen(buffer,"w"))) {
	debug("Can't open file korganizer.tex");
	return;
    }
    fprintf(file, "\\documentclass{letter}\n");
    if (!use_ulem)
	fprintf(file, "\\usepackage{color}\n");
    else {
	fprintf(file, "\\newlength{\\wrdwd}\n\\newlength{\\wrdht}\n"
		"\\newcommand{\\strikeout}[1]\n"
		"{\\settowidth{\\wrdwd}{#1}\\settoheight{\\wrdht}{#1}\n"
		"#1\\hspace{-\\wrdwd}\\rule[.5\\wrdht]{\\wrdwd}{.01cm}}\n");		
    }
    fprintf(file, "\\begin{document}\n");
    if(!use_ulem)
        fprintf(file, "\\definecolor{grey}{gray}{.5}");
    fprintf(file,"\\begin{center}\n\\Large\nTasks from %s to %s"
                 "\\end{center}\n", 
	    //calendar->getOwner().data(),
	    start.toString().data(),
	    end.toString().data());
    
    QList<KDPEvent> &todoList = calendar->getTodoList();
    //    todoList = calendar->getTodoList();
    KDPEvent *aTodo;
    bool hasTodos = FALSE;
    for (aTodo = todoList.first(); aTodo; aTodo = todoList.next()) {
        if (!aTodo->getStatusStr().contains("COMPLETED")) {
	    if (!hasTodos) {
		hasTodos = TRUE;
		fprintf(file,"\\begin{center}\n\\Large\n%s\n\\end{center}\n", i18n("Todos"));
		fprintf(file,"\\begin{tabular}{p{0.5cm} p{8cm}}\n\n");
	    }
	    fprintf(file, "%d & %s \\\\\n", 
		    aTodo->getPriority(), 
		    aTodo->getSummary().data());
	}
    } 
    if (print_completed_todos) {
	for (aTodo = todoList.first(); aTodo; aTodo = todoList.next()) {
	    if (aTodo->getStatusStr().contains("COMPLETED")) {
		if (!hasTodos) {
		    hasTodos = TRUE;
		    fprintf(file,"\\begin{center}\n\\Large\n%s\n\\end{center}\n", i18n("Todos"));
		    fprintf(file,"\\begin{tabular}{p{0.5cm} p{8cm}}\n\n");
		}
		if (use_ulem) { // use overstriked characters
		    fprintf(file, "%d & \\strikeout{%s} \\\\\n", 
			    aTodo->getPriority(), 
			    aTodo->getSummary().data());		    
		} else {
		    fprintf(file, "%d & \\textcolor{gray}{%s} \\\\\n", 
			    aTodo->getPriority(), 
			    aTodo->getSummary().data());
		}
	    }
	} 
    }
    if (hasTodos) {
	fprintf(file,"\\end{tabular}\n\n");    
    } 

    //    fprintf(file,"\\begin{itemize}\n\n");
    // iterate for the upcoming days
    fprintf(file,"\\begin{center}\n\\Large\n%s\n\\end{center}\n\n", i18n("Appointments"));

    QDate currentDate=start;
    bool eventThatDay = FALSE;
    while (currentDate.daysTo(end) >= 0) {
	eventThatDay = FALSE;
	// iterate for events on that day
	QList<KDPEvent> events(calendar->getEventsForDate(currentDate));
	
	for (aTodo = events.first(); aTodo; aTodo = events.next()) {
	    if (!eventThatDay) {
		eventThatDay = TRUE;
		fprintf(file, "\\begin{center}\n\\large\n%s\n\\end{center}\n\n",
			currentDate.toString().data());
		fprintf(file,"\\begin{tabular}{p{3cm} p{9cm}}\n\\hline\n");		
	    }
	    fprintf(file, "{\\bf %s - %s} & %s ", 
		    aTodo->getDtStartTimeStr().data(),
		    aTodo->getDtEndTimeStr().data(),
		    aTodo->getSummary().data());
	    // cathegories
	    QStrList categories = aTodo->getCategories();
	    QString category;
	    if (categories.count() > 0) {
		bool first=TRUE;
		fprintf(file, "\\newline {\\bf Categories:} ");
		for (category = categories.first(); category; category = categories.next()) {
		    if (!first) {
			first = FALSE; 
			fprintf(file,", ");
		    }
		    printf("Categ %s \n", category.data());
		    fprintf(file, category.data());
		}
		fprintf(file, "\n\n");
	    }
	    // attendees;
	    fprintf(file, " \\\\\n");
	} // next event 
	if (eventThatDay) {
		fprintf(file,"\\hline\n\\end{tabular}\n\n");			    
	}
	currentDate = currentDate.addDays(1);
    }
    // this is the end
    fprintf(file, "\\end{document}\n");    
    fclose(file);
}

/**
 *
 *  Printing!!!
 *
 */
void CalendarLatexPrinter::print() {
    QString buffer;
    KProcess *latex = new KProcess;
    config->setGroup("Printer");
    *latex <<  config->readEntry("latex_executable");;
    buffer.sprintf("%s.tex",filename);
    *latex << buffer;
    connect(latex,  SIGNAL(processExited(KProcess *)), SLOT(print2(KProcess*)));
    latex->start();
    return;
}

void CalendarLatexPrinter::preview() {
    QString buffer;
    KProcess *latex = new KProcess;
    config->setGroup("Printer");
    //    char *test = config->readEntry("latex_executable").data();
    *latex << config->readEntry("latex_executable");
    buffer.sprintf("%s.tex",filename);
    *latex << buffer;
    connect(latex,  SIGNAL(processExited(KProcess *)), SLOT(pspreview2(KProcess*)));
    latex->start();
    return;
}

void CalendarLatexPrinter::pspreview2(KProcess *proc) {      
    char buffer[1000];
    if (!checkTermination(proc, "latex")) return;
    KProcess *dvips = new KProcess;
    *dvips << config->readEntry("dvips_executable");
    sprintf(buffer,"%s.dvi",filename);
    *dvips << buffer;
    sprintf(buffer,"%s.ps",filename);
    *dvips << "-o" << buffer;
    connect(dvips,  SIGNAL(processExited(KProcess *)), SLOT(pspreview3(KProcess*)));
    dvips->start();
}

void  CalendarLatexPrinter::pspreview3(KProcess *proc) {      
    QString buffer;
    if (!checkTermination(proc, "dvips")) return;    
    kghostview.clearArguments();
    kghostview << config->readEntry("ps_viewer_executable");
    buffer.sprintf("%s.ps",filename);
    kghostview << buffer;
    kghostview.start(KProcess::DontCare);
}

void  CalendarLatexPrinter::print2(KProcess *proc) {      
    debug("print2");
    char buffer[1000];
    if (!checkTermination(proc, "latex")) return;    
    dvipr.clearArguments();
    dvipr << config->readEntry("dvipr_executable");
    sprintf(buffer,"%s.dvi",filename);
    dvipr << buffer;
    dvipr.start(KProcess::DontCare);
}

bool CalendarLatexPrinter::checkTermination(KProcess *proc, char *execname) {
    QString message;
    message.sprintf(i18n("Cannot run the %s executable.\n" 
			 "Check the path under Options/Printer"), execname);
    if (proc->normalExit()) {
	if (proc->exitStatus()==255) {
	    QMessageBox::warning(0, i18n("Error"), message, i18n("&Ok"));
	    return FALSE;
	}
    } else {
	debug("Funny exit from KProcess!");
	return FALSE;
    }
    return TRUE;
}
