/*   $Id: editeventwin.cpp,v 1.77 1998/08/25 03:00:03 pbrown Exp $

     Requires the Qt and KDE widget libraries, available at no cost at
     http://www.troll.no and http://www.kde.org respectively

     Copyright (C) 1997, 1998 Preston Brown
     preston.brown@yale.edu

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

     -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-

     This file implements a class for displaying a dialog box for
     adding or editing appointments/events.

*/

#include <qframe.h>
#include <qpixmap.h>
#include <kdatepik.h>
#include <kiconloader.h>
#include <kapp.h>
#include <stdio.h>

#include "qdatelist.h"
#include "misc.h"
#include "editeventwin.moc"

EditEventWin::EditEventWin(CalObject *cal, bool todo )
  : KTopLevelWidget("eventedit")
{
  addFlag = TRUE;
  calendar = cal;
  isTodo = todo;

  setCaption(i18n("New Appointment - KOrganizer"));
  initConstructor();

  fillInDefaults(QDateTime(QDate::currentDate(), QTime(12,0,0)),
		 QDateTime(QDate::currentDate(), QTime(13,0,0)), FALSE);
  prevNextUsed = FALSE;
}

void EditEventWin::editEvent( KDPEvent *event, QDate qd )
{
  isTodo = event->getTodoStatus();
  // check to see if save & close should be enabled or not
  if (event->getOrganizer() != calendar->getEmail()) {
    saveExit->setEnabled(FALSE);
    if (isTodo)
      setCaption(i18n("Edit Todo - KOrganizer (Readonly)"));
    else
      setCaption(i18n("Edit Appointment - KOrganizer (Readonly)"));
  } else {
    if (isTodo)
      setCaption(i18n("Edit Todo - KOrganizer"));
    else
      setCaption(i18n("Edit Appointment - KOrganizer"));
  }

  currEvent = event;
  addFlag = FALSE;
  fillInFields(qd);

}

void EditEventWin::newEvent( QDateTime from, QDateTime to, bool allDay )
{
  currEvent = 0L;
  fillInDefaults( from, to, allDay );
}

void EditEventWin::initConstructor()
{
  initToolBar();

  m_frame = new QFrame(this);
  tabCtl = new KTabCtl(m_frame);
  QString tmpStr;

  CHECK_PTR(tabCtl);
  General = new EventWinGeneral(m_frame, "General", isTodo);
  CHECK_PTR(General);
  Details = new EventWinDetails(m_frame, "Details", isTodo);
  CHECK_PTR(Details);
  
  if (!isTodo) {
      Recurrence = new EventWinRecurrence(m_frame, "Recurrence", isTodo);
      CHECK_PTR(Recurrence);
  }
  setFrameBorderWidth(0);

  tabCtl->addTab(General, i18n("General"));
  tabCtl->addTab(Details, i18n("Details"));
  if (!isTodo)
      tabCtl->addTab(Recurrence, i18n("Recurrence"));

  setView(m_frame);
  // how do you resize the view to fit everything by default??
  resize(605, 455);
  
  // this is temporary until we can have the thing resize properly.
  setFixedWidth(605);
  setFixedHeight(455);

  // signal/slot stuff

  // category views on General and Details tab are synchronized
  catDlg = new CategoryDialog();
  connect(General->categoriesButton, SIGNAL(clicked()), 
	  catDlg, SLOT(show()));
  connect(Details->categoriesButton, SIGNAL(clicked()),
	  catDlg, SLOT(show()));
  connect(catDlg, SIGNAL(okClicked(QString)), 
	  this, SLOT(updateCatEdit(QString)));
  connect(General->categoriesEdit, SIGNAL(textChanged(const char *)),
	  Details->categoriesEdit, SLOT(setText(const char *)));
  connect(Details->categoriesEdit, SIGNAL(textChanged(const char *)),
	  General->categoriesEdit, SLOT(setText(const char *)));

  if (isTodo) {
      //debug("No connect, I'm leaving");
      return; // hangs up
  }

  // time widgets on General and Recurrence tab are synchronized
  connect(General->startTimeEdit, SIGNAL(timeChanged(QTime, int)),
	  this, SLOT(startTimeChanged(QTime, int)));
  connect(General->endTimeEdit, SIGNAL(timeChanged(QTime, int)),
	  this, SLOT(endTimeChanged(QTime, int)));
  connect(Recurrence->startTimeEdit, SIGNAL(timeChanged(QTime, int)),
	  this, SLOT(startTimeChanged(QTime, int)));
  connect(Recurrence->endTimeEdit, SIGNAL(timeChanged(QTime, int)),
	  this, SLOT(endTimeChanged(QTime, int)));

  // date widgets on General and Recurrence tab are synchronized
  connect(General->startDateEdit, SIGNAL(dateChanged(QDate)),
	  this, SLOT(startDateChanged(QDate)));
  connect(Recurrence->startDateEdit, SIGNAL(dateChanged(QDate)),
	  this, SLOT(startDateChanged(QDate)));
  connect(General->endDateEdit, SIGNAL(dateChanged(QDate)),
	  this, SLOT(endDateChanged(QDate)));

  // attach the enable/disabling of recurrence tab stuff
  connect(General->recursButton, SIGNAL(toggled(bool)),
	  this, SLOT(recurStuffEnable(bool)));
  connect(General->noTimeButton, SIGNAL(toggled(bool)),
	  Recurrence, SLOT(timeStuffDisable(bool)));

}

EditEventWin::~EditEventWin()
{
  // delete NON-CHILDREN widgets
  delete catDlg;
  delete m_frame;
}

/*************************************************************************/
/********************** PROTECTED METHODS ********************************/
/*************************************************************************/
void EditEventWin::initMenus()
{

}

void EditEventWin::initToolBar()
{
  QPixmap pixmap;

  toolBar = new KToolBar(this);
  addToolBar(toolBar);

  saveExit = new KPTButton( this );
  saveExit->setText(i18n("Save and Close"));
  saveExit->setPixmap(Icon("filefloppy.xpm"));
  connect( saveExit, SIGNAL(clicked()), SLOT(saveAndClose()) );
  toolBar->insertWidget(0, saveExit->sizeHint().width(), saveExit);

  pixmap = Icon("exit.xpm");
  toolBar->insertButton(pixmap, 0,
			SIGNAL(clicked()), this,
			SLOT(cancel()), TRUE,
			i18n("Cancel"));

  QFrame *sepFrame = new QFrame(toolBar);
  sepFrame->setFrameStyle(QFrame::VLine|QFrame::Raised);
  toolBar->insertWidget(0, 10, sepFrame);

  pixmap = Icon("up.xpm");
  toolBar->insertButton(pixmap, 0,
			SIGNAL(clicked()), this, 
			SLOT(prevEvent()), TRUE, 
			i18n("Previous Event"));

  pixmap = Icon("down.xpm");
  toolBar->insertButton(pixmap, 0,
			SIGNAL(clicked()), this,
			SLOT(nextEvent()), TRUE, 
			i18n("Next Event"));
  
  sepFrame = new QFrame(toolBar);
  sepFrame->setFrameStyle(QFrame::VLine|QFrame::Raised);
  toolBar->insertWidget(0, 10, sepFrame);

  pixmap = Icon("delete.xpm");
  toolBar->insertButton(pixmap, 0, 
			SIGNAL(clicked()), this,
			SLOT(deleteEvent()), TRUE,
			i18n("Delete Event"));
}

void EditEventWin::fillInTodoFields() {
    if (currEvent->getStatusStr() == "X-ACTION")
      General->completedButton->setChecked(FALSE);
    else
      General->completedButton->setChecked(TRUE);

    General->priorityCombo->setCurrentItem(currEvent->getPriority()-1);
}

void EditEventWin::fillInFields(QDate qd)
{
  QString tmpStr;
  QDateTime tmpDT;
  QBitArray rDays;
  QList<KDPEvent::rMonthPos> rmp;
  QList<int> rmd;
  int i;
  bool ok;


  /******************** GENERAL *******************************/
  General->summaryEdit->setText(currEvent->getSummary().data());
  General->descriptionEdit->setText(currEvent->getDescription().data());

  // organizer information
  General->ownerLabel->setText(tmpStr.sprintf(i18n("Owner: %s"),
					      currEvent->getOrganizer().data()).data());

  if (isTodo) {
      fillInTodoFields();
      return;
  }
  
  // the rest is for the events only
  General->noTimeButton->setChecked(currEvent->doesFloat());
  General->timeStuffDisable(currEvent->doesFloat());
  General->alarmStuffDisable(currEvent->doesFloat());

  currStartDateTime = currEvent->getDtStart();
  currEndDateTime = currEvent->getDtEnd();
  setDuration();

  General->startDateEdit->setDate(currStartDateTime.date());
  General->startTimeEdit->setTime(currStartDateTime.time());
  
  General->endDateEdit->setDate(currEndDateTime.date());
  General->endTimeEdit->setTime(currEndDateTime.time());

  General->recursButton->setChecked(currEvent->doesRecur());

  General->privateButton->setChecked((currEvent->getSecrecy() > 0) 
				     ? TRUE : FALSE);

  General->alarmButton->setChecked(currEvent->getAlarmRepeatCount());
  if (General->alarmButton->isChecked()) {
    General->alarmStuffEnable(TRUE);
    tmpDT = currEvent->getAlarmTime();
    i = tmpDT.secsTo(currStartDateTime);
    i = i / 60; // make minutes
    tmpStr.sprintf("%d",i);
    General->alarmTimeEdit->setText(tmpStr.data());
  } else
    General->alarmStuffEnable(FALSE);

  if (currEvent->getTransparency() > 0)
    General->freeTimeCombo->setCurrentItem(1);
  // else it is implicitly 0 (i.e. busy)

  // categories
  //char *catStr = currEvent->getCategoriesStr();
  //General->categoriesEdit->setText(catStr);
  //  delete catStr; catStr = 0L;
  General->categoriesEdit->setText( currEvent->getCategoriesStr() );

  /******************** DETAILS *******************************/
  // attendee information
  QList<Attendee> al = currEvent->getAttendeeList();
  QListIterator<Attendee> ai(al);
  for (; ai.current(); ++ai) {
    tmpStr.sprintf("%s\n \n \n%s",
		   ai.current()->getName().data(),
		   ai.current()->RSVP() ? "Y" : "N");
    printf("tmpStr: %s\n",tmpStr.data());
    Details->attendeeListBox->appendItem(tmpStr.data());
  }

  Details->attachListBox->insertItem(i18n("Not implemented yet."));
  
  // set the status combobox
  Details->statusCombo->setCurrentItem(currEvent->getStatus());
  

  /******************** RECURRENCE *******************************/
  Recurrence->startTimeEdit->setTime(General->startTimeEdit->getTime(ok));
  Recurrence->endTimeEdit->setTime(General->endTimeEdit->getTime(ok));
  Recurrence->timeStuffDisable(currEvent->doesFloat());

  recurStuffEnable(currEvent->doesRecur());
  General->recursButton->setChecked(currEvent->doesRecur());

  // unset everything
  Recurrence->dailyButton->setChecked(FALSE);
  Recurrence->weeklyButton->setChecked(FALSE);
  Recurrence->monthlyButton->setChecked(FALSE);
  Recurrence->yearlyButton->setChecked(FALSE);
  Recurrence->onNthDay->setChecked(FALSE);
  Recurrence->onNthTypeOfDay->setChecked(FALSE);
  Recurrence->yearMonthButton->setChecked(FALSE);
  Recurrence->yearDayButton->setChecked(FALSE);
  
  Recurrence->mondayBox->setChecked(FALSE);
  Recurrence->tuesdayBox->setChecked(FALSE);
  Recurrence->wednesdayBox->setChecked(FALSE);
  Recurrence->thursdayBox->setChecked(FALSE);
  Recurrence->fridayBox->setChecked(FALSE);
  Recurrence->saturdayBox->setChecked(FALSE);
  Recurrence->sundayBox->setChecked(FALSE);
  Recurrence->endDateButton->setChecked(FALSE);
  Recurrence->noEndDateButton->setChecked(FALSE);
  Recurrence->endDurationButton->setChecked(FALSE);
  switch (currEvent->doesRecur()) {
  case KDPEvent::rNone:
    break;
  case KDPEvent::rDaily:
    Recurrence->dailyButton->setChecked(TRUE);
    tmpStr.sprintf("%d",currEvent->getRecursFrequency());
    Recurrence->nDaysEntry->setText(tmpStr.data());
    break;
  case KDPEvent::rWeekly:
    Recurrence->weeklyButton->setChecked(TRUE);
    tmpStr.sprintf("%d", currEvent->getRecursFrequency());
    Recurrence->nWeeksEntry->setText(tmpStr.data());
    
    rDays = currEvent->getRecursDays();
    if (rDays.testBit(0))
      Recurrence->mondayBox->setChecked(TRUE);
    if (rDays.testBit(1))
      Recurrence->tuesdayBox->setChecked(TRUE);
    if (rDays.testBit(2))
      Recurrence->wednesdayBox->setChecked(TRUE);
    if (rDays.testBit(3))
      Recurrence->thursdayBox->setChecked(TRUE);
    if (rDays.testBit(4))
      Recurrence->fridayBox->setChecked(TRUE);
    if (rDays.testBit(5))
      Recurrence->saturdayBox->setChecked(TRUE);
    if (rDays.testBit(6))
      Recurrence->sundayBox->setChecked(TRUE);
    break;
  case KDPEvent::rMonthlyPos:
    // we only handle one possibility in the list right now,
    // so I have hardcoded calls with first().  If we make the GUI
    // more extended, this can be changed.
    Recurrence->monthlyButton->setChecked(TRUE);
    Recurrence->onNthTypeOfDay->setChecked(TRUE);
    rmp = currEvent->getRecursMonthPositions();
    if (rmp.first()->negative)
      i = 5 - rmp.first()->rPos - 1;
    else
      i = rmp.first()->rPos - 1;
    Recurrence->nthNumberEntry->setCurrentItem(i);
    i = 0;
    while (!rmp.first()->rDays.testBit(i))
      ++i;
    Recurrence->nthTypeOfDayEntry->setCurrentItem(i);
    tmpStr.sprintf("%d",currEvent->getRecursFrequency());
    Recurrence->nMonthsEntry->setText(tmpStr.data());
    break;
  case KDPEvent::rMonthlyDay:
    Recurrence->monthlyButton->setChecked(TRUE);
    Recurrence->onNthDay->setChecked(TRUE);
    rmd = currEvent->getRecursMonthDays();
    i = *rmd.first() - 1;
    Recurrence->nthDayEntry->setCurrentItem(i);
    tmpStr.sprintf("%d",currEvent->getRecursFrequency());
    Recurrence->nMonthsEntry->setText(tmpStr.data());
    break;
  case KDPEvent::rYearlyMonth:
    Recurrence->yearlyButton->setChecked(TRUE);
    Recurrence->yearMonthButton->setChecked(TRUE);
    rmd = currEvent->getRecursYearNums();
    Recurrence->yearMonthComboBox->setCurrentItem(*rmd.first() - 1);
    tmpStr.sprintf("%d",currEvent->getRecursFrequency());
    Recurrence->nYearsEntry->setText(tmpStr.data());
    break;
  case KDPEvent::rYearlyDay:
    Recurrence->yearlyButton->setChecked(TRUE);
    Recurrence->yearDayButton->setChecked(TRUE);
    tmpStr.sprintf("%d",currEvent->getRecursFrequency());
    Recurrence->nYearsEntry->setText(tmpStr.data());
    break;
  default:
    break;
  }

  if (currEvent->doesRecur()) {
    // get range information
    if (currEvent->getRecursDuration() == -1)
      Recurrence->noEndDateButton->setChecked(TRUE);
    else if (currEvent->getRecursDuration() == 0) {
      Recurrence->endDateButton->setChecked(TRUE);
      Recurrence->endDateEdit->setDate(currEvent->getRecursEndDate());
    } else {
      Recurrence->endDurationButton->setChecked(TRUE);
      tmpStr.sprintf("%d",currEvent->getRecursDuration());
      Recurrence->endDurationEdit->setText(tmpStr.data());
    }
  } else {
    // the event doesn't recur, but we should provide some logical
    // defaults in case they go and make it recur.
    Recurrence->noEndDateButton->setChecked(TRUE);
    Recurrence->weeklyButton->setChecked(TRUE);
    Recurrence->nDaysEntry->setText("1");
    Recurrence->nWeeksEntry->setText("1");
    switch (currStartDateTime.date().dayOfWeek()) {
    case 1:
      Recurrence->mondayBox->setChecked(TRUE);
      break;
    case 2:
      Recurrence->tuesdayBox->setChecked(TRUE);
      break;
    case 3:
      Recurrence->wednesdayBox->setChecked(TRUE);
      break;
    case 4:
      Recurrence->thursdayBox->setChecked(TRUE);
      break;
    case 5:
      Recurrence->fridayBox->setChecked(TRUE);
      break;
    case 6:
      Recurrence->saturdayBox->setChecked(TRUE);
      break;
    case 7:
      Recurrence->sundayBox->setChecked(TRUE);
      break;
    }
    Recurrence->onNthDay->setChecked(TRUE);
    Recurrence->nthDayEntry->setCurrentItem(currStartDateTime.date().day()-1);
    Recurrence->nMonthsEntry->setText("1");
    Recurrence->yearDayButton->setChecked(TRUE);
    tmpStr.sprintf("%i",currStartDateTime.date().dayOfYear());
    Recurrence->nYearsEntry->setText("1");
  }

  QDateList exDates(FALSE);
  exDates = currEvent->getExDates();
  QDate *curDate;
  Recurrence->exceptionDateEdit->setDate(qd);
  for (curDate = exDates.first(); curDate;
       curDate = exDates.next())
    Recurrence->exceptionList->insertItem(curDate->toString().data());
}

void EditEventWin::fillInDefaults(QDateTime from, 
				  QDateTime to, bool allDay)
{
  QString tmpStr;

  currStartDateTime = from;
  currEndDateTime = to;
  setDuration();

  // default owner is calendar owner.
  General->ownerLabel->setText(tmpStr.sprintf(i18n("Owner: %s"),
					      calendar->getEmail().data()).data());

  if (isTodo) return;

  /********************************* GENERAL *******************************/

  General->noTimeButton->setChecked(allDay);
  General->timeStuffDisable(allDay);
  General->alarmStuffDisable(allDay);
  General->startDateEdit->setDate(from.date());
  General->endDateEdit->setDate(to.date());
  General->startTimeEdit->setTime(from.time());
  General->endTimeEdit->setTime(to.time());

  recurStuffEnable(FALSE);
  General->recursButton->setChecked(FALSE);
  General->alarmTimeEdit->setText("15");
  General->alarmStuffEnable(FALSE);
  /******************************** DETAILS ********************************/

  /******************************* RECURRENCE ******************************/
  Recurrence->timeStuffDisable(allDay);
  Recurrence->noEndDateButton->setChecked(TRUE);
  Recurrence->weeklyButton->setChecked(TRUE);
  Recurrence->nDaysEntry->setText("1");
  Recurrence->nWeeksEntry->setText("1");
  // unset everything
  Recurrence->mondayBox->setChecked(FALSE);
  Recurrence->tuesdayBox->setChecked(FALSE);
  Recurrence->wednesdayBox->setChecked(FALSE);
  Recurrence->thursdayBox->setChecked(FALSE);
  Recurrence->fridayBox->setChecked(FALSE);
  Recurrence->saturdayBox->setChecked(FALSE);
  Recurrence->sundayBox->setChecked(FALSE);
  
  // now get the right one for the start date
  switch (from.date().dayOfWeek()) {
  case 1:
    Recurrence->mondayBox->setChecked(TRUE);
    break;
  case 2:
    Recurrence->tuesdayBox->setChecked(TRUE);
    break;
  case 3:
    Recurrence->wednesdayBox->setChecked(TRUE);
    break;
  case 4:
    Recurrence->thursdayBox->setChecked(TRUE);
    break;
  case 5:
    Recurrence->fridayBox->setChecked(TRUE);
    break;
  case 6:
    Recurrence->saturdayBox->setChecked(TRUE);
    break;
  case 7:
    Recurrence->sundayBox->setChecked(TRUE);
    break;
  }
  Recurrence->onNthDay->setChecked(TRUE);
  Recurrence->nthDayEntry->setCurrentItem(from.date().day()-1);
  Recurrence->nMonthsEntry->setText("1");
  Recurrence->yearDayButton->setChecked(TRUE);
  tmpStr.sprintf("%i",from.date().dayOfYear());
  Recurrence->nYearsEntry->setText("1");
}

void EditEventWin::closeEvent(QCloseEvent *)
{
  // we clean up after ourselves...
  emit closed(this);
}

/*************************************************************************/
/********************** PROTECTED SLOTS **********************************/
/*************************************************************************/

// used to make a new event / update an old event

void EditEventWin::makeEvent()
{
  KDPEvent *anEvent;
  QDate tmpDate;
  QTime tmpTime;
  QDateTime tmpDT;

  // temp. until something better happens.
  QString tmpStr;
  bool ok;
  uint i;
  int j;

  if (addFlag) {
    anEvent = new KDPEvent;
  } else {
    anEvent = currEvent;
  }

  // turn off signal emission when all these changes are going on to
  // avoid a signal storm
  anEvent->blockSignals(TRUE);

  /******************************* GENERAL WIN ***************************/
  // set summary
  anEvent->setSummary(General->summaryEdit->text());

  // set description
  anEvent->setDescription(General->descriptionEdit->text());

  // set categories
  anEvent->setCategories(QString(General->categoriesEdit->text()));

  // set secrecy value
  anEvent->setSecrecy(General->privateButton->isChecked() ? 1 : 0);
  /******************************* DETAILS WIN ***************************/
  // set attendee list

  // this might not be such a good idea because of the modified flag?
  anEvent->clearAttendees();

  for (i = 0; i < Details->attendeeListBox->count(); i++) {
    bool rsvp = (strcmp(Details->attendeeListBox->text(i,3),"Y") == 0);
    Attendee *a = new Attendee(Details->attendeeListBox->text(i, 0), rsvp);
    anEvent->addAttendee(a);
  }
  
  
  if (isTodo) {   
      currEvent->setPriority(General->priorityCombo->currentItem()+1);
      if (General->completedButton->isChecked()) {
	  anEvent->setStatus(QString("COMPLETED"));
      } else {
	  anEvent->setStatus(QString("NEEDS ACTION"));
      }

      emit eventChanged(anEvent, EVENTEDITED);
      close();
      return;
  }
  // the rest is for appointments only

  if (General->noTimeButton->isChecked()) {
    anEvent->setFloats(TRUE);
    // need to change this.
    tmpDate = General->startDateEdit->getDate();
    tmpTime.setHMS(0,0,0);
    tmpDT.setDate(tmpDate);
    tmpDT.setTime(tmpTime);
    anEvent->setDtStart(tmpDT);

    tmpDate = General->endDateEdit->getDate();
    tmpTime.setHMS(0,0,0);
    tmpDT.setDate(tmpDate);
    tmpDT.setTime(tmpTime);
    anEvent->setDtEnd(tmpDT);
  } else {
    anEvent->setFloats(FALSE);
    
    // set date/time start
    tmpDate = General->startDateEdit->getDate();
    tmpTime = General->startTimeEdit->getTime(ok);
    tmpDT.setDate(tmpDate);
    tmpDT.setTime(tmpTime);
    anEvent->setDtStart(tmpDT);
    
    // set date/time end
    tmpDate = General->endDateEdit->getDate();
    tmpTime = General->endTimeEdit->getTime(ok);
    tmpDT.setDate(tmpDate);
    tmpDT.setTime(tmpTime);
    anEvent->setDtEnd(tmpDT);
  } // check for float


  // alarm stuff
  if (General->alarmButton->isChecked()) {
    anEvent->setAlarmRepeatCount(1);
    tmpStr = General->alarmTimeEdit->text();
    j = tmpStr.toInt() * -60;
    tmpDT = anEvent->getDtStart();
    tmpDT = tmpDT.addSecs(j);
    anEvent->setAlarmTime(tmpDT);
  } else
    anEvent->setAlarmRepeatCount(0);
  
  // note, that if on the details tab the "Transparency" option is implemented,
  // we will have to change this to suit.
  anEvent->setTransparency(General->freeTimeCombo->currentItem());
  


  /******************************* RECURRENCE WIN ************************/
  // get recurrence information
  // need a check to see if recurrence is enabled...
  if (General->recursButton->isChecked()) {

    int rDuration;
    QDate rEndDate;
    
    // clear out any old settings;
    anEvent->unsetRecurs();

    // first get range information.  It is common to all types
    // of recurring events.
    if (Recurrence->noEndDateButton->isChecked()) {
      rDuration = -1;
    } else if (Recurrence->endDurationButton->isChecked()) {
      tmpStr = Recurrence->endDurationEdit->text();
      rDuration = tmpStr.toInt();
    } else {
      rDuration = 0;
      rEndDate = Recurrence->endDateEdit->getDate();
    }
    
    // check for daily recurrence
    if (Recurrence->dailyButton->isChecked()) {
      int rFreq;
      
      tmpStr = Recurrence->nDaysEntry->text();
      rFreq = tmpStr.toInt();
      if (rDuration != 0)
	anEvent->setRecursDaily(rFreq, rDuration);
      else
	anEvent->setRecursDaily(rFreq, rEndDate);
      // check for weekly recurrence
    } else if (Recurrence->weeklyButton->isChecked()) {
      int rFreq;
      QBitArray rDays(7);
      
      tmpStr = Recurrence->nWeeksEntry->text();
      rFreq = tmpStr.toInt();
      rDays.fill(FALSE);
      if (Recurrence->mondayBox->isChecked())
	rDays.setBit(0, 1);
      if (Recurrence->tuesdayBox->isChecked())
	rDays.setBit(1, 1);
      if (Recurrence->wednesdayBox->isChecked())
	rDays.setBit(2, 1);
      if (Recurrence->thursdayBox->isChecked())
	rDays.setBit(3, 1);
      if (Recurrence->fridayBox->isChecked())
	rDays.setBit(4, 1);
      if (Recurrence->saturdayBox->isChecked())
	rDays.setBit(5, 1);
      if (Recurrence->sundayBox->isChecked())
	rDays.setBit(6, 1);
      if (rDuration != 0)
	anEvent->setRecursWeekly(rFreq, rDays, rDuration);
      else
	anEvent->setRecursWeekly(rFreq, rDays, rEndDate);
    } else if (Recurrence->monthlyButton->isChecked()) {
      if (Recurrence->onNthTypeOfDay->isChecked()) {
	// it's by position
	int rFreq, rPos;
	QBitArray rDays(7);
	
	tmpStr = Recurrence->nMonthsEntry->text();
	rFreq = tmpStr.toInt();
	rDays.fill(FALSE);
	rPos = Recurrence->nthNumberEntry->currentItem() + 1;
	rDays.setBit(Recurrence->nthTypeOfDayEntry->currentItem());
	if (rDuration != 0)
	  anEvent->setRecursMonthly(KDPEvent::rMonthlyPos, rFreq, rDuration);
	else
	  anEvent->setRecursMonthly(KDPEvent::rMonthlyPos, rFreq, rEndDate);
	anEvent->addRecursMonthlyPos(rPos, rDays);
      } else {
	// it's by day
	int rFreq;
	short rDay;
	
	tmpStr = Recurrence->nMonthsEntry->text();
	rFreq = tmpStr.toInt();
	
	rDay = Recurrence->nthDayEntry->currentItem() + 1;
	
	if (rDuration != 0)
	  anEvent->setRecursMonthly(KDPEvent::rMonthlyDay, rFreq, rDuration);
	else
	  anEvent->setRecursMonthly(KDPEvent::rMonthlyDay, rFreq, rEndDate);
	anEvent->addRecursMonthlyDay(rDay);
      }
    } else if (Recurrence->yearlyButton->isChecked()) {
      if (Recurrence->yearMonthButton->isChecked()) {
	int rFreq, rMonth;

	tmpStr = Recurrence->nYearsEntry->text();
	rFreq = tmpStr.toInt();
	rMonth = Recurrence->yearMonthComboBox->currentItem() + 1;
	if (rDuration != 0)
	  anEvent->setRecursYearly(KDPEvent::rYearlyMonth, rFreq, rDuration);
	else
	  anEvent->setRecursYearly(KDPEvent::rYearlyMonth, rFreq, rEndDate);
	anEvent->addRecursYearlyNum(rMonth);
      } else {
	// it's by day
	int rFreq;
	int rDay;

	tmpStr = Recurrence->nYearsEntry->text();
	rFreq = tmpStr.toInt();
	
	//tmpStr = Recurrence->yearDayLineEdit->text();
	rDay = anEvent->getDtStart().date().dayOfYear();

	if (rDuration != 0)
	  anEvent->setRecursYearly(KDPEvent::rYearlyDay, rFreq, rDuration);
	else
	  anEvent->setRecursYearly(KDPEvent::rYearlyDay, rFreq, rEndDate);
	anEvent->addRecursYearlyNum(rDay);
      }
    } // yearly
  } else
    anEvent->unsetRecurs();

  QDateList exDates;
  for (i = 0; i < Recurrence->exceptionList->count(); i++)
    exDates.inSort(dateFromText(Recurrence->exceptionList->text(i)));

  anEvent->setExDates(exDates);
  exDates.clear();
  // we should remove this.
  anEvent->setStatus(Details->statusCombo->currentItem());

  /******************************* ADD THE EVENT ***************************/

  // makeEvent_final:
  
  anEvent->setLastModified(QDateTime::currentDateTime());

  // allow signals again so other stuff sees this update
  anEvent->blockSignals(FALSE);

  if (addFlag) {
    calendar->addEvent(anEvent);
    emit eventChanged(anEvent, EVENTADDED);
    currEvent = anEvent;
    addFlag = FALSE;
  } else {
    anEvent->setRevisionNum(anEvent->getRevisionNum()+1);
    emit eventChanged(anEvent, EVENTEDITED);
  }
}

QDate *EditEventWin::dateFromText(const char *text)
{
  QString tmpStr = text;
  tmpStr.remove(0,4);
  QString name = tmpStr.left(3);

  int y, m, d;

  name = name.upper();

  y = tmpStr.right(4).toInt();
  d = tmpStr.mid(4,2).toInt();
  if (name == "JAN")
    m = 1;
  else if (name == "FEB")
    m = 2;
  else if (name == "MAR")
    m = 3;
  else if (name == "APR")
    m = 4;
  else if (name == "MAY")
    m = 5;
  else if (name == "JUN")
    m = 6;
  else if (name == "JUL")
    m = 7;
  else if (name == "AUG")
    m = 8;
  else if (name == "SEP")
    m = 9;
  else if (name == "OCT")
    m = 10;
  else if (name == "NOV")
    m = 11;
  else if (name == "DEC")
    m = 12;
  else
    // should never get here!
    m = 0;  
  
  return new QDate(y,m,d);
}

void EditEventWin::cancel()
{
  close();
}

void EditEventWin::updateCatEdit(QString _str)
{
  General->categoriesEdit->setText(_str.data());
  // Details categoriesEdit is automatically changed by 
  // SIGNAL/SLOT connection.
}

void EditEventWin::recurStuffEnable(bool enable)
{
  if (enable) {
    //General->endLabel->hide();
    General->startDateEdit->hide();
    General->endDateEdit->hide();
  } else {
    //General->endLabel->show();
    General->startDateEdit->show();
    General->endDateEdit->show();
  }
  tabCtl->setTabEnabled("Recurrence", enable);
}

void EditEventWin::startTimeChanged(QTime newtime, int wrapval)
{
  int secsep;

  secsep = currStartDateTime.secsTo(currEndDateTime);
  
  currStartDateTime = currStartDateTime.addDays(wrapval);
  currStartDateTime.setTime(newtime);

  currEndDateTime = currStartDateTime.addSecs(secsep);
  
  Recurrence->startDateEdit->setDate(currStartDateTime.date());
  Recurrence->startTimeEdit->setTime(currStartDateTime.time());

  Recurrence->endTimeEdit->setTime(currEndDateTime.time());

  General->startDateEdit->setDate(currStartDateTime.date());
  General->endDateEdit->setDate(currEndDateTime.date());

  General->startTimeEdit->setTime(currStartDateTime.time());
  General->endTimeEdit->setTime(currEndDateTime.time());

  setDuration();
}

void EditEventWin::endTimeChanged(QTime newtime, int wrapval)
{
  QDateTime newdt(currEndDateTime.addDays(wrapval).date(), newtime);

  if(newdt < currStartDateTime) {
    // oops, can't let that happen.
    newdt = currStartDateTime;
  }
  currEndDateTime = newdt;
  
  General->endDateEdit->setDate(currEndDateTime.date());
  Recurrence->endTimeEdit->setTime(currEndDateTime.time());
  General->endTimeEdit->setTime(currEndDateTime.time());

  setDuration();
}

void EditEventWin::startDateChanged(QDate newdate)
{
  int daysep;
  daysep = currStartDateTime.daysTo(currEndDateTime);
  
  currStartDateTime.setDate(newdate);
  currEndDateTime.setDate(currStartDateTime.date().addDays(daysep));

  Recurrence->startDateEdit->setDate(currStartDateTime.date());
  General->startDateEdit->setDate(currStartDateTime.date());
  General->endDateEdit->setDate(currEndDateTime.date());

  setDuration();
}

void EditEventWin::endDateChanged(QDate newdate)
{
  QDateTime newdt(newdate, currEndDateTime.time());

  if(newdt < currStartDateTime) {
    // oops, we can't let that happen.
    newdt = currStartDateTime;
    General->endDateEdit->setDate(newdt.date());
    Recurrence->endTimeEdit->setTime(newdt.time());
    General->endTimeEdit->setTime(newdt.time());
  }
  currEndDateTime = newdt;

  setDuration();
}

void EditEventWin::setDuration()
{
  QString tmpStr;
  int hourdiff, minutediff;

  if (isTodo)
    return;
  if (General->noTimeButton->isChecked()) {
    tmpStr.sprintf(i18n("Duration: all day"));
  } else {
    hourdiff = currStartDateTime.date().daysTo(currEndDateTime.date()) * 24;
    hourdiff += currEndDateTime.time().hour() - 
      currStartDateTime.time().hour();
    minutediff = currEndDateTime.time().minute() -
      currStartDateTime.time().minute();
    if (hourdiff && minutediff)
      tmpStr.sprintf(i18n("Duration: %i hour(s), %i minute(s)"), hourdiff, minutediff);
    else if (hourdiff)
      tmpStr.sprintf(i18n("Duration: %i hour(s)"), hourdiff);
    else if (minutediff)
      tmpStr.sprintf(i18n("Duration: %i minute(s)"), minutediff);
    else
      tmpStr = "";
  }
  Recurrence->durationLabel->setText(tmpStr.data());
  Recurrence->durationLabel->adjustSize();

}

/*****************************************************************************/
void EditEventWin::prevEvent()
{
  if (!prevNextUsed) {
    calendar->prev();
    prevNextUsed = TRUE;
  }
  KDPEvent *anEvent(calendar->prev());
  editEvent(anEvent, anEvent->getDtStart().date());
}

void EditEventWin::nextEvent()
{
  if (!prevNextUsed) {
    calendar->next();
    prevNextUsed = TRUE;
  }
  KDPEvent *anEvent(calendar->next());
  editEvent(anEvent, anEvent->getDtStart().date());
}

void EditEventWin::saveAndClose()
{
  makeEvent();
  close();
}

void EditEventWin::deleteEvent()
{
  if (!addFlag) {
    calendar->deleteEvent(currEvent->getDtStart().date(),
			  currEvent->getEventId());
  }
  emit eventChanged(currEvent, EVENTDELETED);
  close();
}
