// eventwidget.cpp
// (C) 1998 F.Zigterman, Preston Brown

#include <stdio.h>
#include <stdlib.h>

#include <qpainter.h>
#include <kapp.h>
#include <kiconloader.h>

#include "eventwidget.h"
#include "eventwidget.moc"

#define DRAGTHRESHOLD 10
#define DRAGBORDER 5

EventWidget::EventWidget( KDPEvent *thisevent, QDate thisdate, QWidget *parent, char *name )
	: QLabel( parent, name )
{
	// TextBox contains summary and any pixmaps
	textB = new TextBox( this );

	alarmPxmp = Icon("bell.xpm");
	recurPxmp = Icon("recur.xpm");

	// set flags
//	FrameStyle = Box | Plain;
	FrameStyle = WinPanel|Raised;
	Flags = NoFlags;
	ResizeStatus = 0;
	widgetDate = thisdate;
	widgetEvent = thisevent;
	widgetSelected = FALSE;
	HeaderSize = fontMetrics().height();
	BodySize = textB->fontMetrics().height();

	// printf("ew:%s\n",widgetEvent->getDtStart().toString().data() );

	textB->setEnabled(TRUE);
	textB->installEventFilter( this );
	//installEventFilter(this);

	// set widget to 1 pixel box frame with minimum 1 line of text
	setFrameStyle( FrameStyle );
	setLineWidth(1);

	// track mouse for cursor shape change
	setMouseTracking(TRUE);

	setMinimumWidth( 20 );
	setMinimumHeight( BodySize );
//	setBackgroundMode( PaletteBackground );
	setAlignment( AlignTop | AlignLeft );
	setEnabled(TRUE);
	setFocusPolicy( NoFocus );

	// set header & body
	updateHeader();
	setBodyText( widgetEvent->getSummary() );


	// new palette for the widget and the TextBox
	EWPalette = new QPalette( palette().copy() );
	HiliteCG = new QColorGroup( EWPalette->normal().foreground(), EWPalette->normal().light(),
				EWPalette->normal().light(), EWPalette->normal().dark(), 
				EWPalette->normal().mid(), EWPalette->normal().text(), EWPalette->normal().light() );
	NormalCG = new QColorGroup( EWPalette->normal().foreground(), EWPalette->normal().mid(), 
				EWPalette->normal().light(), EWPalette->normal().dark(), 
				EWPalette->normal().mid(), EWPalette->normal().text(), EWPalette->normal().mid() );
	
	EWPalette->setNormal( *NormalCG );
	setPalette( *EWPalette );
	textB->setPalette( *EWPalette );
}

EventWidget::~EventWidget()
{
  delete textB;
}

void EventWidget::drawContents(QPainter *p)
{
  int x = frameWidth();
  int fw = frameWidth();
  if(widgetEvent->doesRecur()) {
    p->drawPixmap(x, fw, recurPxmp);
    x += recurPxmp.width()+2;
  }
  if(widgetEvent->getAlarmRepeatCount() > 0) {
    p->drawPixmap(x, fw, alarmPxmp);
    x += alarmPxmp.width()+2;
  }
  QFontMetrics fm = p->fontMetrics();
  int yPos;
  int pmheight = QMAX(recurPxmp.height(), alarmPxmp.height());
  if(pmheight < fm.height()) 
    yPos = fm.ascent() + fm.leading()/2;
  else
    yPos = pmheight/2 - fm.height()/2  + fm.ascent() + 1;
  p->drawText(x, yPos, text());
}
/*
int EventWidget::height() const
{
  return QMAX(recurPxmp.height(), 
	      QMAX(alarmPxmp.height(), fontMetrics().lineSpacing()+1));
}

int EventWidget::width() const
{
  int x = 3;
  if(widgetEvent->doesRecur()) {
    x += recurPxmp.width()+2;
  }
  if(widgetEvent->getAlarmRepeatCount() > 0) {
    x += alarmPxmp.width()+2;
  }
  
  return(x + fontMetrics().width(text())+1);
}
*/

QDate EventWidget::date()
{
  return widgetDate;
}

int EventWidget::dragPosX()
{
  return DragX;
}

int EventWidget::dragPosY()
{
  return DragY;
}

void EventWidget::updateHeader()
{
  setHeaderText( widgetEvent->getDtStartTimeStr().data() + QString("-") +
		 widgetEvent->getDtEndTimeStr().data() );
}

void EventWidget::startEdit()
{
  Flags = Edit;
  widgetSelected = TRUE;
  textB->update();
  textB->setFocus();
}

bool EventWidget::eventFilter( QObject *ob, QEvent *ev )
{
	QPoint mousepos, mousepos2;
//	printf("flags:%d ResizeStatus:%d\n",Flags, ResizeStatus );
	switch( ev->type() )
	{
	case Event_MouseButtonPress:
	  DragCenterX = textB->mapToParent( ((QMouseEvent*)ev)->pos() ).x();
	  DragCenterY = textB->mapToParent( ((QMouseEvent*)ev)->pos() ).y();
	  if( Flags == Edit ) // if editing let event though
	    return FALSE;
	  else // not editing, stop event
	    {
	      Flags = NoFlags;
	      if( widgetSelected == FALSE ) { // do a select
		//printf("select\n");
		setFocus();
		emit selected( this );
		setFrameStyle(FrameStyle);
		Flags = Select; 
	      } else if( ! hasFocus() ) {
		Flags = Select;
		setFocus();
	      }
	      return TRUE;
	    }
	case Event_MouseButtonRelease:
	  // BL: setMouseTracking(FALSE);
	  if(((QMouseEvent *) ev)->button() == RightButton )
	    {	// rmb popup , no flags, cancel edit
	      setFocus();
	      //	emit selected( this );
	      emit popupSignal( widgetEvent );
	      Flags = Select;
	      return TRUE;
	    }
	  if( Flags == Edit )
	    {	printf("Still Editing!\n");
	    return FALSE;
	    }
	  if( widgetSelected && Flags == NoFlags )  
	    // going to edit, set flags, and framestyle
	    {
	      Flags = Edit;
	      textB->setFocus();
	      return TRUE;
	    }
	  if( Flags & Drag) {
	    Flags = NoFlags;
	    emit eventChanged( this, MOVE );
	  }
	  // what does this mean? -- PGB
	  //printf("no fit\n");
	  Flags = NoFlags;
	  return FALSE;
	  
	case Event_MouseMove: {
	  mousepos = ((QMouseEvent*)ev)->pos();
	  mousepos2= textB->mapToParent(mousepos);
	  
	  if( Flags == Edit )
	    {
	      //debug("It was an edit: return");
	      Flags = Select;
		    return TRUE;
	    }
		int buttonState =((QMouseEvent*)ev)->state();
		if (! ( (buttonState & LeftButton) || (buttonState & MidButton)) )
		  return TRUE;
		if (Flags & Resize)
		  return TRUE;
		// we only allow "normal" events to be dragged.
		if (Flags == Drag || 
		    ((abs(mousepos2.x() - DragCenterX) > DRAGTHRESHOLD ||
		      abs( mousepos2.y() - DragCenterY) > DRAGTHRESHOLD )) &&
		    !(widgetEvent->doesRecur() || 
		      widgetEvent->isMultiDay())){
		  Flags = Drag;
		  // calculate new drag coords.
		  DragX = mapToParent( textB->mapToParent( mousepos )).x();
		  DragY = mapToParent( textB->mapToParent( mousepos )).y() - DragCenterY;
		  emit eventChanged( this, MOVE );
		  return TRUE;
		}
		return TRUE;
	}
	case Event_FocusIn:
		if( Flags == Edit )
		// going to edit, setup frame 
		{
			startEdit();
			return FALSE;
		} else {
			setFocus();
			return TRUE;
		}

	case Event_FocusOut:
		if( Flags == Edit ) { // cancel edit, frame, and flags
		    // BL: in my interpretation, this is not a cancel, but a "save"
		    widgetEvent->setSummary( textB->text().simplifyWhiteSpace() );
		    Flags = NoFlags;
		    setFocus();
		}
		return FALSE;

	case Event_MouseButtonDblClick:
	    if(ob == textB) { // avoid opening two windows
		Flags = NoFlags;
		setFocus();
		emit editEventSignal( widgetEvent );
		return TRUE;
	    } else {
	      //debug("Doubleclick coming from somewhere else...");
		return TRUE;
	    }
	case Event_KeyPress:
	    {
		unsigned int Kcode = ((QKeyEvent*)ev)->key();
		if( Kcode == Key_Return ) {
		    textB->home(FALSE);
		    setFocus();
		    widgetEvent->setSummary( textB->text().simplifyWhiteSpace() );
		    fitText();
		    return TRUE;
		}
		else if( Kcode < Key_Home || Kcode > Key_F24 && Kcode != Key_Space ) {
		    //	fitText();
		}
	    }
	    return FALSE;
	default:
	    return FALSE;
	}
}


void EventWidget::mousePressEvent( QMouseEvent *ev)
{

	// TODO: find out here if we want to resize
	// NB: moving is now done in the eventfilter !

	emit selected(this);

	Flags = Resize;
	if( ev->x() < DRAGBORDER )
		ResizeStatus = RESIZELEFT;
	else if( ev->x() > width()- DRAGBORDER )
		ResizeStatus = RESIZERIGHT;
	else if( ev->y() < DRAGBORDER )
		ResizeStatus = RESIZETOP;
	else if( ev->y() > (height() - DRAGBORDER) )
		ResizeStatus = RESIZEBOTTOM;
	else
	{
//		printf("Resizing NO status\n");
		 ResizeStatus = 0;
	}
	//printf("mousetracking ON\n"); // well it is not on BL:
	DragX = ev->x();
	DragY = ev->y();
	
	if( ev->button() == RightButton )
		emit popupSignal( widgetEvent );
  
	if( ev->type() == Event_MouseButtonDblClick )
		emit editEventSignal( widgetEvent );
	// BL: better not eat it here
	if (ResizeStatus == 0)
	    eventFilter(this, ev);
}

void EventWidget::mouseReleaseEvent(QMouseEvent *ev)
{
	Flags = 0;
	ResizeStatus=0;
//	setMouseTracking( FALSE);
//	printf("mousetracking OFF %d\n", ResizeStatus);
	setCursor( ArrowCursor );
	
	//  dragging_enabled = FALSE;
	//  moving_enabled = FALSE;
	// BL: better not eat it here
	eventFilter(this, ev);
}

void EventWidget::mouseMoveEvent(QMouseEvent *mev)
{
//	printf("mousemove %d\n", ResizeStatus);
	// ONLY do resizing here !
	if( Flags != Resize )
	{
	  //printf("pointer change\n");
	  if( mev->x() < DRAGBORDER )
	    setCursor( SizeHorCursor );
	  else if( mev->x() > width()- DRAGBORDER )
	    setCursor( SizeHorCursor );
	  else if( mev->y() < DRAGBORDER )
	    setCursor( SizeVerCursor );
	  else if( mev->y() > height() - DRAGBORDER )
	    setCursor( SizeVerCursor );
	  else setCursor( ArrowCursor );
	}

	DragX = mev->x();
	DragY = mev->y();

	// if Resizing, send signal. 
	if( ResizeStatus != 0 ) 
		emit eventChanged( this, ResizeStatus );

	// BL: better not eat it here
	eventFilter(this, mev);
}

void EventWidget::focusInEvent( QFocusEvent *)
{
	//Flags = Select;
	//emit selected( this );
	//setFrameStyle(FrameStyle);
}

void EventWidget::focusOutEvent( QFocusEvent *)
{}

void EventWidget::keyPressEvent( QKeyEvent *kev )
{
	printf("keypressevent %x\n", kev->key() );
	switch( kev->key() )
	{
	case Key_Delete:
		emit deleteEventSignal( widgetEvent );
		break;
	case Key_Return:
		Flags = Edit;
		textB->setFocus();
		break;
	case Key_Plus:
	  //		widgetEvent->setDtStart( widgetEvent->getDtStart().addSecs( 5*60 ) );
		widgetEvent->setDtEnd( widgetEvent->getDtEnd().addSecs( 5*60 ) );
		//printf("Datestart++\n");
		emit eventChanged( this, DTCHANGE );
		updateHeader();
		break;
	case Key_Minus:
	  //		widgetEvent->setDtStart( widgetEvent->getDtStart().addSecs(-60 ) );
		widgetEvent->setDtEnd( widgetEvent->getDtEnd().addSecs( -5*60 ) );
		//printf("Datestart--\n");
		emit eventChanged( this, DTCHANGE );
		updateHeader();
		break;
	}
}

void EventWidget::resizeEvent( QResizeEvent * )
{
	int MARGIN = 3;
//	printf("resizing ew:%X\n", this );

	HeaderSize = fontMetrics().height();
	BodySize =  textB->fontMetrics().height();
	if (( height() > HeaderSize + BodySize + MARGIN ) &&
	    ( widgetEvent->doesFloat() == FALSE ) &&
	    ( widgetEvent->isMultiDay() == FALSE))
	  textB->setGeometry( MARGIN, 
			      HeaderSize + MARGIN, 
			      width() - MARGIN*2,
			      height() - HeaderSize - MARGIN*2 );
	else
	  textB->setGeometry( MARGIN, MARGIN, width()-MARGIN*2, height()-MARGIN*2 );
	
	textB->raise();

	fitText();
}

void EventWidget::fitText()
{
	int startindex, endindex, countindex, testlength, line, column;

	// Map the text to the width of the widget

	textB->cursorPosition( &line, &column );
	countindex =-1;
	QString testText = textB->text().simplifyWhiteSpace() + " ";
	//	printf("text:-%s-\n",testText.data() );
	startindex =0;
	testlength = testText.length();
	countindex = testText.find( " ", 0 );
	while( (endindex = testText.find( " ", countindex+1) ) >-1  )
	{
		QString middle;
		int len;
		len = endindex - startindex;
		//	printf("test:%d %d %d %d\n", startindex, countindex, endindex, len );
		middle = testText.mid( startindex, len );
		//	printf("->%d,%d  middle:%s\n", startindex, endindex, middle.data() );
		if( textB->textWidth( &middle ) > width() )
		{
			// printf("replace:%d,%d,%d\n",startindex,countindex, endindex );
			testText.replace( countindex, 1, "\n" );
			startindex = countindex;
			// printf("replace done.\n");
		}
		countindex = endindex;
	}
	textB->setText( testText.stripWhiteSpace() );
	textB->setCursorPosition( line, column );
	// BL:
	setToolTip(textB->text().simplifyWhiteSpace());
}


void EventWidget::setSelected( bool selected )
{
	if( widgetSelected = selected )
	{
	  raise();
	  //setMouseTracking(TRUE);
	  if (!widgetEvent->doesFloat())
	    setLineWidth(1);
	  EWPalette->setNormal( *HiliteCG );
	  EWPalette->setActive( *HiliteCG );
	  textB->setPalette( *EWPalette );
	  setPalette( *EWPalette );


	}
	else
	{
	  //setMouseTracking(FALSE);
	  //setFrameStyle( FrameStyle );
	  setLineWidth(1);

	  EWPalette->setNormal( *NormalCG );
	  EWPalette->setActive( *NormalCG );
	  textB->setPalette( *EWPalette );
	  setPalette( *EWPalette );
	}
	update();
}

void EventWidget::setToolTip( QString tiptext )
{
	ToolTipText = tiptext;
	QToolTip::add( this, tiptext );
	QToolTip::add( textB, tiptext );
}

QString EventWidget::toolTip()
{
	return ToolTipText;
}

void EventWidget::setBodyText( QString Text ) 
{
	textB->setText( Text );
	textB->raise();
}

QString EventWidget::bodyText()
{
	return textB->text();
}

void EventWidget::setHeaderText( QString Text ) 
{
	setText( Text );
}

QString EventWidget::headerText()
{
	return text();
}

void EventWidget::setHeaderFont( QFont headerfont )
{
	setFont( headerfont );
	HeaderSize = fontMetrics().height();
}

QFont EventWidget::headerFont()
{
	return font();
}

void EventWidget::setBodyFont( QFont bodyfont )
{
	textB->setFont( bodyfont );
}

QFont EventWidget::bodyFont()
{
	return textB->font();
}

KDPEvent *EventWidget::getEvent()
{
	return widgetEvent;
}

TextBox::TextBox( QWidget *parent, char *name )
	: QMultiLineEdit( parent, name )
{
	clearTableFlags( Tbl_autoScrollBars );
	setFrameStyle( NoFrame );
}

int TextBox::textWidth( QString *str )
{
	return QMultiLineEdit::textWidth( str );
}
void TextBox::home(bool mark)
{
	QMultiLineEdit::home(mark);
}
