// eventwidget.h
// (C) F.Zigterman, Preston Brown 1998

#ifndef EVENTWIDGET_H
#define EVENTWIDGET_H

#include <qlabel.h>
#include <qstring.h>
#include <qmlined.h>
#include <qtooltip.h>
#include <qkeycode.h>
#include <qpixmap.h>

#include "kdpevent.h"

/** this widget offers an edit box without scrollbars */
class TextBox : public QMultiLineEdit
{
	Q_OBJECT
public:
	TextBox( QWidget *parent = 0, char *name = "" );
	int textWidth( QString * );
	void home(bool mark );
};

/** this widget displays the time and summary of events */
class EventWidget : public QLabel
{
	Q_OBJECT
public:
	/** Constructor	*/
	EventWidget( KDPEvent *, QDate, QWidget *parent = 0, char *name = 0 );
	
	/** Destructor */
	virtual ~EventWidget();

	/** overloaded functions to deal with pixmaps + textbox */
	void drawContents(QPainter *);
	/*	int width() const;
	int height() const;*/

	/** Return date for event display */
	QDate date();
	
	/** if the information in the header has changed, call this */
	void updateHeader();
		
	/** Set the Text in the Header */
	void setHeaderText( QString );
	QString headerText();

	/* Set the text in the body (textbox) */
	void setBodyText( QString );
	QString bodyText();

	/**	Set font for header and calculate height*/
	void setHeaderFont( QFont );
	QFont headerFont();

	/**	Set font for body (textbox)*/
	void setBodyFont( QFont );
	QFont bodyFont();

	/**	Set the tooltip for the entire widget*/
	void setToolTip( QString );
	QString toolTip();

	/**	return event */
	KDPEvent *getEvent();

	/**	return if widget is selected */
	bool selected();

	/** If the widget is dragged, contains the new desired X position */
	int dragPosX();

	/** If the widget is dragged, contains the new desired Y position */
	int dragPosY();

	enum { MOVE,RESIZETOP,RESIZEBOTTOM,RESIZELEFT,RESIZERIGHT,DTCHANGE,CONTENTS };
public slots:
	/**	select this widget if TRUE unselect if FALSE */
	void setSelected( bool );
        /** select the textbox and start editing this event's summary */
	void startEdit();

signals:
	void selected( EventWidget * );
	void moved(EventWidget *, int x, int y, int xPtrOff, int yPtroff);
	void resized(EventWidget *, QRect newSize);
	void editEventSignal( KDPEvent *);
	void deleteEventSignal( KDPEvent *);
	void popupSignal( KDPEvent *);
	void eventChanged( EventWidget *, int Action );
	
protected:
	TextBox *textB;
	QPixmap alarmPxmp;
	QPixmap recurPxmp;
	
	KDPEvent *widgetEvent;
	QDate widgetDate;
	bool eventFilter(QObject *, QEvent *);
	void mousePressEvent( QMouseEvent *);
	void mouseReleaseEvent(QMouseEvent *);
	void mouseMoveEvent(QMouseEvent *);
	void keyPressEvent( QKeyEvent *);
	void resizeEvent( QResizeEvent * );	
	void focusInEvent( QFocusEvent *);
	void focusOutEvent( QFocusEvent *);
	void fitText();
	int HeaderSize, BodySize, FrameStyle;
	bool widgetSelected;
	QString ToolTipText;
	int Flags; // State of Widget: DRAG, EDIT
	int Threshold;	// counting value for drag enable
	enum { Edit=1, Select=2, Mouse = 4, Drag = 8, Resize=16, NoFlags=0 };
	int DragX, DragY; 	// new desired coords.
	int DragCenterX, DragCenterY; 	// drag center point
	int dragging_enabled;
	int moving_enabled;
	int ResizeStatus;
	// Miserable color groups
	QColorGroup *NormalCG, *HiliteCG;
	QPalette *EWPalette;

};

#endif
