// 	$Id: kagenda.cpp,v 1.106 1998/08/18 19:23:15 pbrown Exp $	

#include "kagenda.h"
#include "kagenda.moc"

#define DAYMARGIN 1
#define DATELABELSIZE 24
#define CELLWIDTH 40
#define TIMESIZE 18
#define TIMEWIDTH 3
#define TIMESCALE 6

KAgenda::KAgenda( CalObject *cal, QWidget *parent, const char *name )
		: QFrame( parent, name )
{
	// global Calendar object
	AgendaCal = cal;
	config = kapp->getConfig();

	// increment size for dragging/resizing events.
	timeGridSize = 15;
	
	// we want to delete dynamically allocated dates when we clear...
	selectedDates.setAutoDelete(TRUE);
	selectedDates.append(new QDate(QDate::currentDate()));

	// setup header widgets at top and left, and main view
	dayFrame = new QLabel( this,"dayFrame" );
	allDayFrame = new QFrame( this,"allDayFrame" );
	sheetFrame = new QLabel( this,"sheetFrame" );
	agendaSheet = new KAgendaSheet( KAgendaSheet::Time_24, CELLWIDTH, 
					1, sheetFrame );
	scrollBar = new QScrollBar( QScrollBar::Vertical, this,
				    "eventScrollBar" );

	QPixmap px;
	px = loader.loadIcon("1leftarrow.xpm" );
	prevdatbt = new KPButton( px, dayFrame );
	px = loader.loadIcon("dayicon.xpm" );
	viewbt = new KPButton( px, dayFrame );
	px = loader.loadIcon("1rightarrow.xpm" );
	nextdatbt = new KPButton( px, dayFrame );
	connect( prevdatbt, SIGNAL( clicked() ), SLOT( slotPrevDates() ) );
	connect( nextdatbt, SIGNAL( clicked() ), SLOT( slotNextDates() ) );
	connect( viewbt, SIGNAL( clicked() ), SLOT( slotViewChange() ) );

	dayFrame->setFixedHeight( DATELABELSIZE );
	allDayFrame->setFixedHeight( 1 );
	//	allDayFrame->setFrameStyle(Panel|Raised);
	//	allDayFrame->setBackgroundMode(PaletteBase);
	scrollBar->setFixedWidth( 16 );
	connect( scrollBar, SIGNAL( valueChanged(int) ), this, SLOT( slotScrollBar(int) ) );
	
	// the layout managers for the view
	/*Layout = new QHBoxLayout( this, 2, 1 );
	subLayout = new QVBoxLayout( 1 );
	dayLayout = new QHBoxLayout( dayFrame, 1 );
	Layout->addLayout( subLayout );
	Layout->addWidget( scrollBar );
	subLayout->addWidget( dayFrame );
	subLayout->addWidget( allDayFrame );
	subLayout->addWidget( sheetFrame );*/
	subLayout = 0;
	Layout = 0;
	// Agenda init
	WidgetList.setAutoDelete(TRUE);
	dateLabels.setAutoDelete(TRUE);
	setFrameStyle( Box | Sunken );
	setMinimumSize( 100,100 );
	
	// sheet settings
//	agendaSheet->setFocusPolicy( ClickFocus );
	agendaSheet->setBackgroundMode( PaletteMid );

	QFont f;
	f = agendaSheet->font();
	f.setFamily("Lucida");
	f.setPointSize(18);
	agendaSheet->setFont( f );

	// create menu and connect to handler
	QPixmap pm;
	pm = loader.loadIcon("delete.xpm");

	agendaPopup = new QPopupMenu();
	agendaPopup->insertItem(i18n("&Edit"));
	agendaPopup->insertItem(pm, i18n("&Delete"));
	pm = loader.loadIcon("bell.xpm");
	agendaPopup->insertItem(pm, i18n("Toggle Alarm"));
	connect( agendaPopup, SIGNAL( activated(int) ), this, SLOT( agendaPopupSlot( int ) ) );
	connect( agendaSheet, SIGNAL( popupActivated(int) ), this, SLOT( sheetPopupSlot( int ) ) );

	connect(agendaSheet, SIGNAL(addNewEventSignal(int, QTime)),
				    this, SLOT(addNewEvent(int, QTime)));

	// read settings from KConfig
	updateConfig();

	// set view type to a 5 day view starting on monday
	AgendaStartDate = QDate::currentDate();
	selectedWidget = 0L;
	ViewType = DAY;
	slotViewChange();
	
	resizeEvent(0);
	// this should really be made a configuration variable.
	int eightoclock = scrollBar->minValue()+
	    (scrollBar->maxValue() - scrollBar->minValue() )/ 3;
	scrollBar->setValue(eightoclock);	
}


void KAgenda::fillAgenda()
{
//	Fill Agenda View day with events
  //  int scrollStart = 0;
  const char *weekday[] = { i18n("Mon"),i18n("Tue"),i18n("Wed"),
		      i18n("Thu"),i18n("Fri"),i18n("Sat"),i18n("Sun") };
  KDPEvent *currentevent = 0L;
  // kill obsolete eventwidgets
  setSelected( 0 );
  agendaSheet->setFocus();
  WidgetList.clear();
  floatEvents.clear();
  floatCount = 0;

  if( selectedDates.count() ) {
    QLabel *lb;
    QString datetext;
    if ( ! dateLabels.count() ) {
      dateLabels.append( lb = new QLabel( dayFrame ));
    }
    dateLabels.first();		
    // Go through the list making date labels and new event widgets		
    for( selectedDates.first(); selectedDates.current(); selectedDates.next() ) {
      if( (lb = dateLabels.current()) ) {
	dateLabels.next();
      } else {
	lb = new QLabel( dayFrame );
	dateLabels.append( lb );
	dateLabels.next();
      }
      lb->setFrameStyle( Raised | Panel );
      lb->setAlignment( AlignTop | AlignHCenter );
      lb->setText( datetext.sprintf("%s %d",
				    weekday[ selectedDates.current()->dayOfWeek() -1 ],
				    selectedDates.current()->day() ) );
    }		
    // Get events for this date and put em in
    for( selectedDates.first(); selectedDates.current(); selectedDates.next() ) {
      QList<KDPEvent> dateevents(AgendaCal->getEventsForDate( *selectedDates.current() ));
      // make a widget for each event
      for( currentevent = dateevents.first(); currentevent; 
	   currentevent = dateevents.next() )
	newEventWidget(currentevent, *selectedDates.current());      
    }
  }
  // set grid dimensions
  agendaSheet->setColumns( selectedDates.count() );
  // and layout the widgets
  doLayout();
}

EventWidget *KAgenda::newEventWidget(KDPEvent *currEvent, QDate displayDate)
{
  EventWidget *ew;

  if (currEvent->isMultiDay()) {
    if (floatEvents.findRef(currEvent) == -1) {
      ++floatCount;
      floatEvents.append(currEvent);
    } else {
      return 0;
    }
  }

  // also need to add to floatcount all those events w/no time associated.
  if (currEvent->doesFloat() && (floatEvents.findRef(currEvent) == -1))
    ++floatCount;

  if (currEvent->doesFloat() || currEvent->isMultiDay()) {
    ew = new EventWidget(currEvent,displayDate,allDayFrame);
  } else {
    ew = new EventWidget(currEvent,displayDate,agendaSheet);
  }
  
  connect( ew, SIGNAL( selected(EventWidget*) ), 
	     this, SLOT(setSelected(EventWidget*) ) );

  connect( ew, SIGNAL( editEventSignal( KDPEvent *) ), 
	   this, SIGNAL( editEventSignal( KDPEvent *) ) );
  connect( ew, SIGNAL( deleteEventSignal( KDPEvent *) ), 
	   this, SIGNAL(deleteEventSignal( KDPEvent *) ) );
  connect( ew, SIGNAL( popupSignal( KDPEvent *) ), 
	   this, SLOT( RMBSlot() ) );
  connect( ew, SIGNAL( eventChanged( EventWidget *, int ) ),
		this, SLOT( updateEventWidget( EventWidget *, int ) ) );

  ew->setHeaderFont( font() );
  ew->setBodyFont( font() );

  if (!currEvent->doesFloat())
    ew->setToolTip( ew->text() );
  else
    ew->setToolTip(i18n("All Day Event"));
  
  WidgetList.append( ew );

  return ew;
}

void KAgenda::addNewEvent(int col, QTime t)
{
  KDPEvent *newEvent = new KDPEvent;
  EventWidget *ew;
  int endhour, endminute;

  QDate newDate = AgendaStartDate.addDays(col);
  endhour = t.hour()+1;
  endminute = t.minute();
  newEvent->setDtStart(QDateTime(newDate, t));
  newEvent->setDtEnd(QDateTime(newDate,
			       QTime(endhour, endminute, 0)));
  newEvent->setFloats(FALSE);
  AgendaCal->addEvent(newEvent);
  // do we want to reverse these?
  ew = newEventWidget(newEvent, newDate);
  doLayout();
  setSelected(ew);
  ew->startEdit();
}

void KAgenda::slotScrollBar( int value )
{
	agendaSheet->move( 0, -value );
}

void KAgenda::RMBSlot()
{
	agendaPopup->popup( QCursor::pos() );
}

void KAgenda::agendaPopupSlot( int index )
{
  switch( index ) {
  case 0:
    emit editEventSignal( selectedWidget->getEvent() );
    break;
  case 1:
    emit deleteEventSignal( selectedWidget->getEvent() );
    break;
  case 2: { // toggle alarm
    KDPEvent *anEvent(selectedWidget->getEvent());
    anEvent->toggleAlarm();
    selectedWidget->repaint();
    break;
  }
  default:
    break;
  }
}

void KAgenda::sheetPopupSlot( int index )
{
	switch( index )
	{
	case 0:
		emit newEventSignal( QDateTime(AgendaStartDate) );
		return;
	case 1:
		emit optionsSignal( 3 );
		PropDlg->show();
		return;
	default:
		return;
	}
}

void KAgenda::resizeEvent( QResizeEvent * )
{
	// printf("begin resize event\n");

	// position the prev/next/view buttons
	prevdatbt->setGeometry( 0,0, (agendaSheet->gridOffset() - DATELABELSIZE) /2, 
				dayFrame->height() );
	nextdatbt->setGeometry( (agendaSheet->gridOffset() - DATELABELSIZE) /2 + DATELABELSIZE, 0, 
				(agendaSheet->gridOffset() - DATELABELSIZE) /2, dayFrame->height() );
 	viewbt->setGeometry( prevdatbt->width(),0, DATELABELSIZE,DATELABELSIZE );

	// sync scrollbar dimensions with view
	scrollBar->setSteps( agendaSheet->rowSize(), sheetFrame->height() );
	scrollBar->setRange( 0, agendaSheet->height() - sheetFrame->height() );

	agendaSheet->resize( sheetFrame->width(), 100 );

	// and realign event widgets
	doLayout();
}

void KAgenda::doLayout()
{
	EventWidget *ew;
	QString time;
	QLabel *lb;
	int i = 0;

	// hide the widgets for faster update
	for( WidgetList.first(); (ew = WidgetList.current()); WidgetList.next() )
	  ew->hide();

	// floatCount is the number of floating events.
	allDayFrame->setFixedHeight(1+agendaSheet->rowSize()*floatCount);
	if (Layout) {
	  delete subLayout;
	  delete Layout;
	}
	// the layout managers for the view
	Layout = new QHBoxLayout( this, 2, 1 );
	subLayout = new QVBoxLayout( 1 );
	dayLayout = new QHBoxLayout( dayFrame, 1 );
	Layout->addLayout( subLayout );
	Layout->addWidget( scrollBar );
	subLayout->addWidget( dayFrame );
	subLayout->addWidget( allDayFrame );
	subLayout->addWidget( sheetFrame );
	// don't activate unless we have some stuff (kind of hacked)
	if (floatCount > 0)
	  Layout->activate();

	// layout the date headers
	for( dateLabels.first();(lb = dateLabels.current());dateLabels.next() )
	{
		int lw = agendaSheet->colSize();
		int lx = agendaSheet->gridOffset() + dateLabels.at() * agendaSheet->colSize();
		lb->setGeometry( lx,0,lw, DATELABELSIZE );
		lb->show();
	}
	
	// walk through all event widgets
	WidgetList.first();
	while((ew = WidgetList.current()) != 0 )
	{
		int x,y,w,h, daycount = 0;
		KDPEvent *event;

		// skip empty dates
		selectedDates.first();
		while( selectedDates.current() && ew->date() != *selectedDates.current() )
		{
			selectedDates.next();
			daycount++;
		}
		if ( ! selectedDates.current() )
		{
			printf("PARSE ERROR in Layout Event widgets\n" );
			break;
		}

		// get event + data from widget
		event = ew->getEvent();
		QTime starttime = event->getDtStart().time();
		QTime endtime = event->getDtEnd().time();

		// calculate proper geometry
		if (event->doesFloat() || event->isMultiDay()) {
		  // for multi-day events, make width span whole range of displayed dates
		  uint days = event->getDtStart().daysTo(event->getDtEnd().date());
		  if (days > selectedDates.count())
		    days = selectedDates.count();
		  w = ((1 + days) * agendaSheet->colSize()) - 2;
		  h = agendaSheet->rowSize();
		  x = agendaSheet->gridOffset() + daycount *
		    agendaSheet->colSize() + 1;
		  y = i * h;
		  i++;
		  agendaSheet->setWidgetGeometry( ew, x, y, w, h);
		} else {
		  /*x = agendaSheet->gridOffset() + daycount * 
		    agendaSheet->colSize() + 1;
		  y = ( starttime.hour() * 60 + starttime.minute() )
		    * agendaSheet->height() / (24*60) + 1;
		  w = agendaSheet->colSize() - 2;
		  h = ( ( endtime.hour() - starttime.hour() ) * 60 + 
			( endtime.minute() - starttime.minute() ) ) *
		    agendaSheet->height() / (24*60) - 2;*/
		  agendaSheet->setWidgetGeometry( ew, starttime, endtime, daycount );
		}

		
		//ew->setGeometry( x + DAYMARGIN , y, w - DAYMARGIN*2, h );
		//ew->show();
		WidgetList.next();
	}
//	printf("dolayout complete\n");
	for( WidgetList.first(); (ew = WidgetList.current()); WidgetList.next() )
		ew->show();
}

KAgenda::~KAgenda()
{
	WidgetList.clear();
}

int KAgenda::getSelectedId()
{
	if( selectedWidget )
		return selectedWidget->getEvent()->getEventId();
	else
		return 0;
}

KDPEvent *KAgenda::getSelected()
{
	// get event from selected event widget
	if( selectedWidget )
		return selectedWidget->getEvent();
	else
		return 0;
}

void KAgenda::setSelected( EventWidget *selew )
{
	// set selected widget
	//	printf("starting setSelected:%x\n",(unsigned int)selew );
	// first deselect the old widget if not the same
	
	if( selectedWidget && (selew != selectedWidget) )
		selectedWidget->setSelected(FALSE);

	// select new widget if any
	
	selectedWidget = selew;
	if( selectedWidget )
		selectedWidget->setSelected(TRUE);
	
	//	printf("finished setSelected:%x\n",(unsigned int)selew );
}
// old move stuff removed

// new stuff
void KAgenda::updateEventWidget( EventWidget *ew, int action )
{
  QDate newdate;
  QTime newTime;
  QDateTime dtstart = ew->getEvent()->getDtStart(); 
  QDateTime dtend = ew->getEvent()->getDtEnd();
  
  int dtsize = dtstart.secsTo( ew->getEvent()->getDtEnd() );
  int col = (ew->dragPosX() - agendaSheet->gridOffset())/ agendaSheet->colSize();
  if( col >= agendaSheet->columns() || col < -1 )
    col =-1;
  
 switch( action )
    {
    case EventWidget::MOVE:
      if (col != -1)
	newdate = *selectedDates.at( col);
      else
	newdate = dtstart.date();
      
      dtstart.setDate( newdate ); 

      newTime = yposToTime(ew->dragPosY());
      if (newTime.isValid()) {
	dtstart.setTime(newTime);
	dtend = dtstart.addSecs( dtsize ) ;
      }
      ew->updateHeader();
      break;
      
    case EventWidget::RESIZETOP:
      newTime = yposToTime(ew->y() + ew->dragPosY());
      if (newTime.isValid()) {
	dtstart.setTime(newTime);
	col =-1; // <- i dont understand why this is necessary but it is.
	
	ew->updateHeader();
      }
      break;
      
    case EventWidget::RESIZEBOTTOM:
      newTime = yposToTime(ew->y() + ew->dragPosY());
      if (newTime.isValid()) {
	dtend.setTime(newTime);
	col =-1;
	
	ew->updateHeader();
      }
      break;
    case EventWidget::RESIZELEFT:
    case EventWidget::RESIZERIGHT:
    case EventWidget::DTCHANGE:
    case EventWidget::CONTENTS:
      break;
    }
  // granularity moved to yposTotime()
  
  // set date/time
  ew->getEvent()->setDtStart( dtstart );
  ew->getEvent()->setDtEnd( dtend );
  
  // generic update -> good idea !
  agendaSheet->setWidgetGeometry( ew, ew->getEvent()->getDtStart().time(),
				  ew->getEvent()->getDtEnd().time(),
				  col );
  ew->updateHeader();
}

QTime KAgenda::yposToTime(int y)
{
  int hour;
  int minutes;
  int rs;
  
  rs = agendaSheet->rowSize();
  hour = y / rs;

  minutes = (y - hour*rs ) *60/rs;
  minutes = minutes - minutes%timeGridSize;
  return QTime(hour, minutes, 0);
}


void KAgenda::updateConfig()
{
  int fmt;
  config->setGroup("Time & Date");
  fmt = config->readNumEntry("Format", 12);
  agendaSheet->setTimeStyle((fmt == 12 ? KAgendaSheet::Time_12 : 
			     KAgendaSheet::Time_24));
  update();
}

void KAgenda::changeEventDisplay(KDPEvent *, int)
{
  // this should be re-written to be MUCH smarter.  Right now we
  // are just playing dumb.
  setView(ViewType);

  fillAgenda();
}

void KAgenda::slotPrevDates()
{
  int datenum, count;

  switch( ViewType )
    {
    case DAY:
      AgendaStartDate = AgendaStartDate.addDays(-1);
      selectedDates.clear();
      selectedDates.append(new QDate(AgendaStartDate));
      break;
    case WORKWEEK:
      AgendaStartDate = AgendaStartDate.addDays(-7);
      selectedDates.clear();
      for (count = 0; count < 5; count++)
	selectedDates.append(new QDate(AgendaStartDate.addDays(count)));
      break;
    case WEEK:
      AgendaStartDate = AgendaStartDate.addDays(-7);
      selectedDates.clear();
      for( count = 0; count < 7; count++ )
	selectedDates.append( new QDate( AgendaStartDate.addDays( count ) ) );
      break;
    case LIST:
      datenum = selectedDates.count();
      AgendaStartDate = AgendaStartDate.addDays( -datenum );
      selectedDates.clear();
      for (count = 0; count < datenum; count++)
	selectedDates.append( new QDate ( AgendaStartDate.addDays( count ) ) );
      break;
    }
  emit datesSelected(selectedDates);
  setView( ViewType );
  fillAgenda();
}

void KAgenda::slotNextDates()
{
  int datenum, count;

  switch( ViewType )
    {
    case DAY:
      AgendaStartDate = AgendaStartDate.addDays(1);
      selectedDates.clear();
      selectedDates.append(new QDate(AgendaStartDate));
      break;
    case WORKWEEK:
      AgendaStartDate = AgendaStartDate.addDays(7);
      selectedDates.clear();
      for (count = 0; count < 5; count++)
	selectedDates.append(new QDate(AgendaStartDate.addDays(count)));
      break;
    case WEEK:
      AgendaStartDate = AgendaStartDate.addDays( 7 );
      selectedDates.clear();
      for( count = 0; count < 7; count++ )
	selectedDates.append( new QDate( AgendaStartDate.addDays( count ) ) );
      break;
    case LIST:
      datenum = selectedDates.count();
      AgendaStartDate = selectedDates.last()->addDays( 1 );
      selectedDates.clear();
      for (count = 0; count < datenum; count++)
	selectedDates.append( new QDate ( AgendaStartDate.addDays( count ) ) );
      break;
    }
  emit datesSelected(selectedDates);
  setView( ViewType );
  fillAgenda();
}

void KAgenda::slotViewChange()
{
  const char *viewbuttips[] = { i18n("Work week view"),
				i18n("Week view"), 
				i18n("List of dates") };
  
  // kind of a hack.  Add the number of days currently in view, then call
  // slotPrevDates() to go to the next group of dates with the view type incremented.
  setView(ViewType + 1);
  switch (ViewType) {
  case DAY:
    AgendaStartDate = AgendaStartDate.addDays(1);
    break;
  case WORKWEEK:
  case WEEK:
    AgendaStartDate = AgendaStartDate.addDays(7-(AgendaStartDate.dayOfWeek()-1));
    break;
  case LIST:
    AgendaStartDate = AgendaStartDate.addDays(selectedDates.count());
    break;
  }

  QToolTip::add( viewbt, viewbuttips[ ViewType ] );
  slotPrevDates();
}

void KAgenda::setView( int view )
{
  // set the view characteristics
  
  QPixmap px;
  
  // change view type if valid
  if( selectedDates.first() ) {
    if ((view >= DAY) && (view <= LIST))
      ViewType = view;
    else 
      ViewType = DAY;
  } else
    ViewType = DAY;
  
  switch( ViewType ) {
  case DAY:
    px = loader.loadIcon("dayicon.xpm" );
    viewbt->setPixmap( px);
    break;
    // if its a workweek view, calculate the dates and emit
  case WORKWEEK:
    px = loader.loadIcon("5dayicon.xpm" );
    viewbt->setPixmap( px);
    break;
    // if its a week view, calculate the dates and emit
  case WEEK:
    px = loader.loadIcon("weekicon.xpm" );
    viewbt->setPixmap( px);
    break;
    // if its a list view, update the list properties.
  case LIST:
    // we want to make sure that this is up to date.
    px = loader.loadIcon("agenda.xpm" );
    viewbt->setPixmap( px);
    break;
  }
}

void KAgenda::selectDates( const QDateList list )
{
  selectedDates.clear();
  selectedDates = list;
  AgendaStartDate = *selectedDates.first();

  // if there are 5 dates and the first is a monday, we have a workweek.
  if ((selectedDates.count() == 5) && (selectedDates.first()->dayOfWeek() == 1) &&
      (selectedDates.first()->daysTo(*selectedDates.last()) == 4))
    setView(WORKWEEK);
  // if there are 7 dates and the first date is a monday, we have a regular week.
  else if ((selectedDates.count() == 7) && (selectedDates.first()->dayOfWeek() == 1) &&
	   (selectedDates.first()->daysTo(*selectedDates.last()) == 6))
    setView(WEEK);
  else if (selectedDates.count() == 1)
    setView(DAY);
  else
    // for sanity, set viewtype to LIST for now...
    setView(LIST);
  
  // and update the view
  fillAgenda();
}		

KAgendaSheet::KAgendaSheet( int style, int size, int cols, QWidget *parent, char *name )
    : QWidget( parent, name )
{
	TimeZone = new QFrame( this );
	setTimeStyle( style );
	setRowSize( size );
	setColumns( cols );
	setFocusPolicy( ClickFocus );
	QFont f = font();
	f.setBold(TRUE);
	f.setPointSize( TIMESIZE );
	TimeZone->setFont( f );
	TimeZone->installEventFilter( this );
	TimeZone->setMouseTracking( TRUE );

	// create menu and connect to handler
	sheetPopup = new QPopupMenu();
	QPixmap pixmap;
	pixmap = Icon("newevent.xpm");
	sheetPopup->insertItem(pixmap,i18n("New appointment") );
	//sheetPopup->insertItem( "Options" );
	connect( sheetPopup, SIGNAL( activated(int) ), this, SIGNAL( popupActivated( int ) ) );
}

KAgendaSheet::~KAgendaSheet()
{
}

void KAgendaSheet::setWidgetGeometry( QWidget *widget, int x, int y, int w, int h )
{
//	printf("set ew geom time\n");
	int xpix = x ;//+ gridOffset();
	widget->setGeometry( xpix,y,w/*colSize()-1*/,h );
}

void KAgendaSheet::setWidgetGeometry( QWidget *widget, QTime start, QTime end, int col )
{
	int x;
	int y = (start.hour()*60+start.minute() )*40/60;
	//	printf("set ew xywh %d\n",y);
	if( col >-1 )
		x = col * colSize() + gridOffset() + 1;
	else	x = widget->x();
	int w = colSize() - 1;
	int h = (end.hour()*60+end.minute() )*40/60 - y;
	widget->setGeometry( x,y,w,h );
}

void KAgendaSheet::paintEvent( QPaintEvent *pev )
{
	int t;
	int p_start, p_end, p_wid;
//	int p_hi, p_count;
	QString time;
	QPainter paint;
	
//	printf( "sheet paintEvent\n");

	paint.begin( this );

	p_start = pev->rect().top();
	p_end = pev->rect().bottom();
	p_wid = colSize();
//	p_hi = height();
	
	// draw vertical lines
	for( t=0 ; t <= Columns; t++ )
	{
		int x;
		x = t * p_wid + gridOffset();
//		printf("vert:%d,%d\n",t,x );
		paint.drawLine( x, p_start, x, p_end );
	}

	p_start = pev->rect().top()/rowSize();
	p_end = pev->rect().bottom()/rowSize();
	p_wid = width();
	for( t= p_start; t <= p_end; t++ )
	{
		int y;
		y = t * rowSize() ;
		paint.drawLine( gridOffset(), y, p_wid, y );
		//y = t * (rowSize() / 2);
		//paint.drawLine(gridOffset(), y, p_wid, y);
	}
	paint.end(); 
}

void KAgendaSheet::timePaintEvent( QPaintEvent *pev )
{
	QString time;
	QString timeExt = i18n("am");
	register int t, hour, y;
	QPainter paint;

	paint.begin(TimeZone);
	int p_start = pev->rect().top()/rowSize() - 1;
	int p_end = pev->rect().bottom()/rowSize() + 1;
	int p_wid = pev->rect().width() - 10;

//	printf("p_start: %d, p_end: %d\n",p_start, p_end);
	// draw time bar
	for( t = p_start; t < p_end; t++ )
	{
		hour = t;
		if( TimeStyle == Time_12 )
		{
			  if (hour == 0)
			  {
			  	hour = 12;
			  	timeExt = i18n("am");
			  } 
			  else if (hour > 11)
			  {
				timeExt = i18n("pm");
				if (hour != 12)
				hour -= 12;
			  }
			  else 
			  	timeExt = i18n("am");
		}
		else timeExt = "00";
		time.sprintf("%2d",hour );
		//	timeExt.data());
		y = t * rowSize();
		paint.setPen(PaletteDark);
		paint.drawLine( 5, y, p_wid, y );
		y += rowSize() / 2;
		paint.drawLine( 5+p_wid/2, y, p_wid, y );
		paint.setPen(black);

		QFont f = font();
		f.setBold(FALSE);
		f.setPointSize( TIMESIZE );
		paint.setFont( f );
		paint.drawText(2, t * RowSize + RowSize/2,
				      time.data() );

		f.setBold(FALSE);
		f.setPointSize( TIMESIZE - TIMESCALE );
		paint.setFont( f );
		paint.drawText(TimeZone->width() - 2 * (TIMESIZE-TIMESCALE), t * RowSize + RowSize/2 - 4,
				      timeExt.data() );
	}
	paint.end();
}

bool KAgendaSheet::eventFilter( QObject */*obj*/, QEvent *ev )
{
  if( ev->type() == Event_Paint ) {
      timePaintEvent( (QPaintEvent *)ev );
  } 
  return FALSE;
}

void KAgendaSheet::resizeEvent( QResizeEvent *)
{
  TimeZone->setGeometry( 0,0, font().pointSize() * TIMEWIDTH, height() - 2 );
}

void KAgendaSheet::mousePressEvent( QMouseEvent *mev )
{
  emit clearSelection();
  if( mev->button() == RightButton )
    sheetPopup->popup( QCursor::pos() );
  
}

void KAgendaSheet::mouseDoubleClickEvent(QMouseEvent *mev)
{
  if (mev->button() == LeftButton) {
    int clickedHour = (int) mev->y() / RowSize;
    int halfHour = (int) ((mev->y() % RowSize) / ((int) (RowSize / 2)));
    int col = (mev->x() - gridOffset()) / colSize();
    // this is wrong when we have multiple dates displayed at once
    // at least as far as the date goes (the time should be correct)
    emit addNewEventSignal(col, QTime(clickedHour,(halfHour > 0 ? 30 : 0), 0));
						       
  }  
}

void KAgendaSheet::setTimeStyle( int style )
{
  TimeStyle = style;
  TimeZone->update();
}

int KAgendaSheet::timeStyle()
{	return TimeStyle; }

void KAgendaSheet::setColumns( int col )
{
  if( col > 0 ) Columns = col;
  else Columns = 1;
  update();
}

int KAgendaSheet::columns()
{	return Columns;}

void KAgendaSheet::setRowSize( int rowsize )
{
  RowSize = rowsize;
  setFixedHeight( RowSize * 24 );
  update();
}

int KAgendaSheet::rowSize()
{	return RowSize; }

int KAgendaSheet::colSize()
{	return ( width() - gridOffset() ) / Columns; }

int KAgendaSheet::gridOffset()
{
  //printf("TimeZone:%x\n",TimeZone );
  return TimeZone->width();
}
