// 	$Id: kdated.cpp,v 1.15 1998/08/20 04:21:31 pbrown Exp $	

#include <kapp.h>
#include <qpixmap.h>
#include <kiconloader.h>

#include "kdated.h"
#include "kdated.moc"

KDateEdit::KDateEdit(QWidget *parent, const char *name, QDate initialDate)
  : QFrame(parent, name)
{
  KIconLoader *loader = kapp->getIconLoader();
  QPixmap pixmap;

  //kfEdit = new KFormatEdit(this, name, "aaa ## ####");
  kfEdit = new QLineEdit(this);
  kfEdit->setFixedHeight(20);
  kfEdit->setFixedWidth(90);
  kfEdit->adjustSize();

  pixmap = loader->loadIcon("smallcal.xpm");
  dateButton = new QPushButton(this);
  dateButton->setPixmap(pixmap);
  dateButton->move(kfEdit->geometry().topRight().x()+1,
		   kfEdit->geometry().topRight().y());
  dateButton->setFixedHeight(20);
  dateButton->setFixedWidth(30);
  dateButton->adjustSize();

  datePicker = new KODatePicker(0, initialDate);
  datePicker->setFixedSize(200,200);

  adjustSize();
  //datePicker->setDate(initialDate);
  connect(dateButton, SIGNAL(clicked()), 
	  this, SLOT(toggleDatePicker()));
  connect(datePicker, SIGNAL(dateSelected(QDate)),
	  this, SLOT(setDate(QDate)));
  connect(datePicker, SIGNAL(dateSelected(QDate)),
	  datePicker, SLOT(hide()));
  setDate(initialDate);
}

KDateEdit::~KDateEdit()
{
  delete datePicker;
}

void KDateEdit::setDate(QDate date)
{
  QString tmpStr;
  int month, day, year;

  if (!date.isValid())
    return;

  tmpStr = kfEdit->text();
  month = numFromMonthName(tmpStr.left(3));
  day = tmpStr.mid(4,2).toInt();
  year = tmpStr.right(4).toInt();

  if(QDate::isValid(year, month, day) && 
     date == QDate(year, month, day)) {
    // if we have a valid date and the date is the same as what
    // we are trying set, return.
    return;
  }

  tmpStr = date.toString().data();
  tmpStr.remove(0, 4);
  kfEdit->setText(tmpStr.data());
  emit dateChanged(date);
}

void KDateEdit::setEnabled(bool on)
{
  kfEdit->setEnabled(on);
  dateButton->setEnabled(on);
}

QDate KDateEdit::getDate()
{
  QString tmpStr;
  int month, day, year;

  tmpStr = kfEdit->text();
  month = numFromMonthName(tmpStr.left(3));
  day = tmpStr.mid(4,2).toInt();
  year = tmpStr.right(4).toInt();

  return QDate(year, month, day);
}

void KDateEdit::toggleDatePicker()
{
  static bool visible = FALSE;
  QPoint tmpPoint;

  tmpPoint = mapToGlobal(dateButton->geometry().bottomRight());

  datePicker->setGeometry(tmpPoint.x()-207, tmpPoint.y(), 200, 200);
  if (!visible)
    datePicker->show();
  else
    datePicker->hide();
}

int KDateEdit::numFromMonthName(QString name)
{
  name = name.upper();

  if (name == "JAN")
    return 1;
  else if (name == "FEB")
    return 2;
  else if (name == "MAR")
    return 3;
  else if (name == "APR")
    return 4;
  else if (name == "MAY")
    return 5;
  else if (name == "JUN")
    return 6;
  else if (name == "JUL")
    return 7;
  else if (name == "AUG")
    return 8;
  else if (name == "SEP")
    return 9;
  else if (name == "OCT")
    return 10;
  else if (name == "NOV")
    return 11;
  else if (name == "DEC")
    return 12;
  else
    // should never get here!
    return 0;  
}

