// 	$Id: kdatenav.cpp,v 1.32 1998/07/31 20:45:58 pbrown Exp $	

#include <qstring.h>
#include <qkeycode.h>
#include <kiconloader.h>
#include <kapp.h>

#include "kdatenav.h"
#include "kdatenav.moc"

KDateNavigator::KDateNavigator(QWidget *parent, 
			       CalObject *calendar, 
			       bool show_week_nums,
			       QDate startDate, 
			       const char *name) 
  : QFrame(parent, name)
{
  QPixmap pixmap;
  KDateNavigator::calendar = calendar;
  int i;
  
  setFrameStyle(QFrame::NoFrame);
  setBackgroundMode(PaletteBase);

  // initialize date

  selectedDates.setAutoDelete(TRUE);
  selectedDates.append(new QDate(startDate));
  m_MthYr = startDate;
  m_bShowWeekNums = show_week_nums;

  // Set up the control buttons and date label
  ctrlFrame = new QFrame(this);
  ctrlFrame->setFrameStyle(QFrame::Panel|QFrame::Raised);
  ctrlFrame->setLineWidth(1);
  
  QFont tfont = font();
  tfont.setPointSize(10);
  tfont.setBold(FALSE);

  pixmap = Icon("2leftarrow.xpm");
  prevYear  = new KPButton(pixmap, ctrlFrame);
  prevYear->setFont(tfont);
  prevYear->resize(16,16);

  pixmap = Icon("1leftarrow.xpm");
  prevMonth = new KPButton(pixmap, ctrlFrame);
  prevMonth->setFont(tfont);
  prevMonth->setFixedSize(prevYear->size());

  pixmap = Icon("1rightarrow.xpm");
  nextMonth = new KPButton(pixmap, ctrlFrame);
  nextMonth->setFont(tfont);
  nextMonth->setFixedSize(prevYear->size());

  pixmap = Icon("2rightarrow.xpm");
  nextYear  = new KPButton(pixmap, ctrlFrame);
  nextYear->setFont(tfont);
  nextYear->setFixedSize(prevYear->size());

  dateLabel = new QLabel(ctrlFrame);
  dateLabel->setFont(tfont);
  dateLabel->setAlignment(AlignCenter);

  const char *months[] = {i18n("January"), i18n("February"), i18n("March"),
			  i18n("April"), i18n("May"), i18n("June"), i18n("July"),
			  i18n("August"), i18n("September"), i18n("October"),
			  i18n("November"), i18n("December")};
  QString dtstr = months[startDate.month()-1];
  QString yrstr;
  yrstr.setNum(startDate.year());
  dtstr += " " + yrstr;

  dateLabel->setText(dtstr);
  dateLabel->adjustSize();
  dateLabel->setFixedHeight(prevYear->height()-2);

  connect(prevYear, SIGNAL(clicked()), this, SLOT(goPrevYear()));
  connect(prevMonth, SIGNAL(clicked()), this, SLOT(goPrevMonth()));
  connect(nextMonth, SIGNAL(clicked()), this, SLOT(goNextMonth()));
  connect(nextYear, SIGNAL(clicked()), this, SLOT(goNextYear()));

  viewFrame = new QFrame(this);
  viewFrame->setFrameStyle(QFrame::NoFrame);
  viewFrame->setLineWidth(0);
  //viewFrame->setBackgroundColor(PaletteBase);

  // get the day of the week on the first day
  QDate dayone(m_MthYr.year(), 
	       m_MthYr.month(), 1);
  m_fstDayOfWk = dayone.dayOfWeek();

  // Set up the heading fields.
  QPalette heading_Palette = palette();
  QColor hiliteColor = kapp->winStyleHighlightColor();
  QColorGroup my_Group = QColorGroup(palette().normal().base(),
				     palette().normal().base(),
				     palette().normal().base(),
				     palette().normal().base(),
				     palette().normal().base(),
				     hiliteColor,
				     palette().normal().base());

  heading_Palette.setNormal(my_Group);
  
  const char *day[] = { i18n("S"), i18n("M"), i18n("T"), i18n("W"),
			i18n("T"), i18n("F"), i18n("S") };
  for(i=0; i<7; i++) {
    headings[i] = new QLabel(day[i], viewFrame);
    headings[i]->setPalette(heading_Palette);
    headings[i]->setFont(QFont("Arial", 10, QFont::Bold));
    headings[i]->setAlignment(AlignCenter);
  }
  headingSep = new QFrame(viewFrame);
  headingSep->setFrameStyle(QFrame::HLine|QFrame::Plain);
  headingSep->setLineWidth(2);
  headingSep->setFixedHeight(2);

  // Create the weeknumber labels

  QPalette wkno_Palette = palette();
  my_Group = QColorGroup(palette().normal().base(),
			 palette().normal().base(),
			 palette().normal().base(),
			 palette().normal().base(),
			 palette().normal().base(),
			 palette().normal().mid(),
			 palette().normal().base());

  wkno_Palette.setNormal(my_Group);
  
  for(i=0; i<7; i++) {
    weeknos[i] = new QLabel(viewFrame);
    weeknos[i]->setPalette(wkno_Palette);
    weeknos[i]->setAlignment(AlignRight);
    weeknos[i]->setFont(QFont("Arial", 10));
    if(!show_week_nums) {
      weeknos[i]->hide();
    }
  }
  weeknumSep = new QFrame(viewFrame);
  weeknumSep->setFrameStyle(QFrame::VLine|QFrame::Plain);
  weeknumSep->setLineWidth(1);
  weeknumSep->setFixedWidth(1);
  weeknumSep->setBackgroundColor(PaletteMid);
  if(!show_week_nums) {
    weeknumSep->hide();
  }

  // Create the date buttons.
  for(i=1; i <= m_fstDayOfWk; i++) {
    buttons[m_fstDayOfWk-i] = 
      new KDateButton(dayone.addDays(-i), m_fstDayOfWk-i, viewFrame);
    buttons[m_fstDayOfWk-i]->setEnabled(FALSE);
  }
  
  for(i=m_fstDayOfWk; i<42; i++) {
    buttons[i] = 
      new KDateButton(dayone.addDays(i-m_fstDayOfWk), i, viewFrame);
  }

  for(i=m_MthYr.daysInMonth()+m_fstDayOfWk; i<42; i++) {
    buttons[i]->setEnabled(FALSE);
  }

  // Connect the date buttons to the highlighting scheme.
  for(i=0; i<42; i++) {
    connect(buttons[i], SIGNAL(selected(QDate, int, bool)), 
	    this, SLOT(addSelection(QDate, int, bool)));
  }

  updateView();
}


KDateNavigator::~KDateNavigator()
{
  // clean up widgets
  delete prevMonth;
  delete prevYear;
  delete nextYear;
  delete nextMonth;
  delete dateLabel;
  selectedDates.clear();
}

void KDateNavigator::resizeEvent(QResizeEvent *e) 
{
  int column_width, row_height, i, start_x;
  
  setUpdatesEnabled(FALSE);
  // these shouldn't be hardcoded, and they don't look "quite right..."
  ctrlFrame->setFixedSize(e->size().width(), 18);
  viewFrame->setFixedSize(e->size().width()+3, e->size().height() - 
			  ctrlFrame->size().height() + 4);

  ctrlFrame->move(0,0);

  prevYear->move(5,1);
  prevMonth->move(prevYear->geometry().topRight().x()+2, 1);
  nextYear->move(ctrlFrame->width()-nextYear->width()-5, 1);
  nextMonth->move(nextYear->geometry().topLeft().x()-nextMonth->width()-2, 1);
  dateLabel->setFixedSize(ctrlFrame->width()-prevYear->width()*4-14,
			  dateLabel->height());
  dateLabel->move(ctrlFrame->width()/2-dateLabel->width()/2, 1);

  viewFrame->move(0, ctrlFrame->size().height());

  row_height = (viewFrame->height()-2)/7;
  if(m_bShowWeekNums) {
    column_width = (viewFrame->width()-10)/7;
    start_x = 11;
    weeknos[0]->setFixedSize(11, row_height+2);
    weeknos[0]->move(0,0);
    for(i=1; i<7; i++) {
      weeknos[i]->setFixedSize(10, row_height);
      weeknos[i]->move(0, i*row_height+2);
    }
    weeknumSep->move(start_x-1, row_height+2);
  } else {
    column_width = (viewFrame->width())/7;
    start_x = 0;
  }
  for(i=0; i<7; i++) {
    headings[i]->setFixedSize(column_width, row_height);
    headings[i]->move(start_x+i*column_width, 0);
  }
  for(i=0; i<42; i++) {
    buttons[i]->setFixedSize(column_width, row_height);
    buttons[i]->move(start_x+(i%7)*column_width,
		     (i/7+1)*row_height+2);
  }
  headingSep->setFixedWidth(7*column_width);
  headingSep->move(start_x, row_height);
  weeknumSep->setFixedHeight(6*row_height);

  setUpdatesEnabled(TRUE);
  repaint();
}

void KDateNavigator::updateView()
{
  register int i;
  const char *months[] = {i18n("January"), i18n("February"), i18n("March"),
			  i18n("April"), i18n("May"), i18n("June"), i18n("July"),
			  i18n("August"), i18n("September"), i18n("October"),
			  i18n("November"), i18n("December")};

  setUpdatesEnabled(FALSE);

  // compute the label at the top of the navigator
  QString dtstr = months[m_MthYr.month()-1];
  QString yrstr;
  yrstr.setNum(m_MthYr.year());
  dtstr += " " + yrstr;
  dateLabel->setText(dtstr);
  
  //find the first day of the week of the current month.
  QDate dayone(m_MthYr.year(), m_MthYr.month(), 1);

  // set the date of each of the day buttons which make up the part of
  // the calendar prior to the first day of this month (they will be
  // grayed out).
  for(i=1; i <= m_fstDayOfWk; i++) {
    buttons[m_fstDayOfWk-i]->setDate(dayone.addDays(-i));
    buttons[m_fstDayOfWk-i]->setEnabled(FALSE);
    updateButton(i);
  }
  
  // set the date for all the rest of the buttons.
  for(i=m_fstDayOfWk; i<42; i++) {
    buttons[i]->setDate(dayone.addDays(i-m_fstDayOfWk));
    buttons[i]->setEnabled(TRUE);
    updateButton(i);
  }

  // disable the day buttons which make up the beginning of the next
  // month.
  for(i=m_MthYr.daysInMonth()+m_fstDayOfWk; i<42; i++)
    buttons[i]->setEnabled(FALSE);

  // set the week numbers.
  for(i=0; i<6; i++) {
    QString weeknum;
    int dayOfYear = buttons[(i+1)*7-1]->date().dayOfYear();
    if(dayOfYear%7 != 0)
      weeknum.setNum(dayOfYear/7+1);
    else
      weeknum.setNum(dayOfYear/7);
    weeknos[i+1]->setText(weeknum);
  }

  setUpdatesEnabled(TRUE);
  repaint();

}


inline void KDateNavigator::updateButton(int i)
{
  int index;
  int extraDays;

  // check right away that a valid calendar is available.
  ASSERT(calendar != 0);

  // check to see if this button is currently selected, because then
  // it's coloring is simply the selected coloring.
  for (QDate *tmpDate = selectedDates.first(); tmpDate;
       tmpDate = selectedDates.next()) {

    // find out if the selected date is in the previous or the next
    // month, and compute an offset accordingly
    if (tmpDate->month() > m_MthYr.month()) {
      extraDays = tmpDate->daysInMonth()-1;
    } else if (tmpDate->month() < m_MthYr.month()) {
      extraDays = 0 - tmpDate->daysInMonth();
    } else {
      extraDays = 0;
    }

    // compute the "number" of the date in the table. Check that 
    // it is in the valid range.
    index = dayToIndex((tmpDate->day() + extraDays));
    ASSERT(index >= 0);
    ASSERT(index < 42);

    // if today is in the currently displayed month, hilight today.
    if ((m_MthYr.year() == QDate::currentDate().year()) &&
	(m_MthYr.month() == QDate::currentDate().month()) &&
	(buttons[i]->date().day() == QDate::currentDate().day())) {
      buttons[i]->setHiliteStyle(KDateButton::TodayHilite);
      return;
    }

    // if the current button's index is the same as that of one of the selected
    // dates, hilight this button and return.
    if (index == i) {
      buttons[i]->setHiliteStyle(KDateButton::SelectHilite);
      return;
    }
  }
    
  // if not, check calendar for events on this day, so that we can
  // color it properly. Some care is taken to only change the
  // coloring if necessary, to reduce flicker.
  if(calendar->numEvents(buttons[i]->date())) {
    if(buttons[i]->hiliteStyle() != KDateButton::EventHilite) {
      buttons[i]->setHiliteStyle(KDateButton::EventHilite);
    }
  } else {
    if(buttons[i]->hiliteStyle() != KDateButton::NoHilite) {
      buttons[i]->setHiliteStyle(KDateButton::NoHilite);
    }
  } 
}

void KDateNavigator::setShowWeekNums(bool enabled)
{
  m_bShowWeekNums = enabled;
  for(int i=0; i<7; i++) {
    if(enabled)
      weeknos[i]->show();
    else
      weeknos[i]->hide();
  }
  resize(size());
}

const QDateList KDateNavigator::getSelected()
{
  for (QDate *tmpDate = selectedDates.first(); tmpDate;
       tmpDate = selectedDates.next()) {
    if (!tmpDate->isValid()) {
      debug("Null or invalid date selected!!!");
      selectedDates.clear();
      selectedDates.append(new QDate(QDate::currentDate()));
      emit datesSelected(selectedDates);
      break;
    }
  }
  return selectedDates;
}


void KDateNavigator::goNextMonth()
{
  int yr, mth, day;

  // calculate yr, mth and day for the next month, wrapping if
  // necessary.
  yr  = m_MthYr.month() < 12 ? m_MthYr.year() : m_MthYr.year()+1;
  mth = m_MthYr.month() < 12 ? m_MthYr.month()+1 : 1;
  day = m_MthYr.day();

  // make sure that we move to a valid day in the next month.
  for(;;) {
    if(QDate::isValid(yr, mth, day))
      break;
    day--;
  }

  // set our record of the month, year and day that this datetbl is
  // displaying.
  m_MthYr.setYMD(yr, mth, day);

  fixupSelectedDates(yr, mth);

  updateView();
}

void KDateNavigator::goPrevMonth()
{
  int yr, mth, day;

  // calculate yr, mth and day for the next month, wrapping if
  // necessary.
  yr  = m_MthYr.month() > 1 ? m_MthYr.year() : m_MthYr.year()-1;
  mth = m_MthYr.month() > 1 ? m_MthYr.month()-1 : 12;
  day = m_MthYr.day();

  // make sure that we move to a valid day in the next month.
  for(;;) {
    if(QDate::isValid(yr, mth, day))
      break;
    day--;
  }

  // set our record of the month and year that this datetbl is
  // displaying.
  m_MthYr.setYMD(yr, mth, day);

  fixupSelectedDates(yr, mth);

  updateView();
}

void KDateNavigator::goNextYear()
{
  int yr, mth, day;

  // calculate yr, mth and day for the next month, wrapping if
  // necessary.
  yr  = m_MthYr.year()+1;
  mth = m_MthYr.month();
  day = m_MthYr.day();

  // make sure that we move to a valid day in the next month.
  for(;;) {
    if(QDate::isValid(yr, mth, day))
      break;
    day--;
  }

  // set our record of the month, year and day that this datetbl is
  // displaying.
  m_MthYr.setYMD(yr, mth, day);

  fixupSelectedDates(yr, mth);

  updateView();
}

void KDateNavigator::goPrevYear()
{
  int yr, mth, day;

  // calculate yr, mth and day for the next month, wrapping if
  // necessary.
  yr  = m_MthYr.year()-1;
  mth = m_MthYr.month();
  day = m_MthYr.day();

  // make sure that we move to a valid day in the next month.
  for(;;) {
    if(QDate::isValid(yr, mth, day))
      break;
    day--;
  }

  // set our record of the month, year and day that this datetbl is
  // displaying.
  m_MthYr.setYMD(yr, mth, day);

  // remove dates from the list of selected dates that
  // aren't visible in the new month that is currently displayed
  fixupSelectedDates(yr, mth);
  
  updateView();
}


// remove dates from the selected dates list that fall outside the
// current month.
void KDateNavigator::fixupSelectedDates(int yr, int mth)
{
  QDateList tmpDates(FALSE); // don't do deep copies on this list.
  bool changeFlag = FALSE;
  int day;

  tmpDates = selectedDates;
  for (QDate *tmpDate = tmpDates.first(); tmpDate;
       tmpDate = tmpDates.next()) {
    if ((tmpDate->year() != yr) ||
	(tmpDate->month() != mth)) {
      changeFlag = TRUE;
      day = tmpDate->day();
      
      // remove the bad date
      selectedDates.remove(tmpDate);
      // add the corrected date to selectedDates
      while (!(QDate::isValid(yr,mth,day))) day--;
      selectedDates.inSort(new QDate(yr,mth,day));
    }
  }

  // set our record of the first day of the week of the current
  // month. This needs to be done before calling dayToIndex, since it
  // relies on this information being up to date.
  QDate dayone(m_MthYr.year(), m_MthYr.month(), 1);
  m_fstDayOfWk = dayone.dayOfWeek();

  if (changeFlag)
    emit datesSelected(selectedDates);
}

void KDateNavigator::selectDates(const QDateList dateList)
{
  selectedDates.clear();
  selectedDates = dateList;
  
  // check to see if these dates are valid.
  QListIterator<QDate> it(dateList);
  for (; it.current(); ++it) {
    if (!it.current()->isValid()) {
      selectedDates.clear();
      selectedDates.append(new QDate(QDate::currentDate()));
      debug("KDateNavigator::selectDates: an invalid date was passed as a parameter!");
      emit datesSelected(selectedDates);
    }
  }
  
  // set our record of the month and year that this datetbl is
  // displaying.
  m_MthYr = *selectedDates.first();

  // set our record of the first day of the week of the current
  // month. This needs to be done before calling dayToIndex, since it
  // relies on this information being up to date.
  QDate dayone(m_MthYr.year(), m_MthYr.month(), 1);
  m_fstDayOfWk = dayone.dayOfWeek();

  updateView();
}

void KDateNavigator::selectDates(QDate qd)
{
  selectedDates.clear();
  selectedDates.append(new QDate(qd));
  m_MthYr = qd;
  
  QDate dayone(m_MthYr.year(), m_MthYr.month(), 1);
  m_fstDayOfWk = dayone.dayOfWeek();

  updateView();
}

void KDateNavigator::addSelection(QDate selDate, int index, bool ctrlPressed)
{
  QDate *tmpDate;
  bool found = FALSE;

  // if control wasn't held down, remove all selected dates.  Update those
  // dates appearances.
  if (!ctrlPressed) {
    // make a deep copy.
    QDateList tmpDates;
    tmpDates.setAutoDelete(TRUE);
    tmpDates = selectedDates;

    selectedDates.clear();
    for (tmpDate = tmpDates.first(); tmpDate;
	 tmpDate = tmpDates.next()) {
      updateButton(dayToIndex(tmpDate->day()));
    }
    // clear it out, we're done.
    tmpDates.clear();
  }

  // now, if there are any dates left (i.e. ctrl was held down), check
  // to see if they clicked an already selected date.  If so, we need to unselect it.
  // if not, we need to add it to the list of selected dates.
  for (tmpDate = selectedDates.first(); tmpDate;
       tmpDate = selectedDates.next()) {
    if (*tmpDate == selDate) {
      found = TRUE;
      break;
    }
  }
  if (found)
    selectedDates.remove();
  else
    selectedDates.inSort(new QDate(selDate));

  // update the appearance of the clicked date
  updateButton(index);

  emit datesSelected(selectedDates);
}

inline int KDateNavigator::dayNum(int row, int col)
{
  return 7*(row-1)+(col+1)-m_fstDayOfWk;
}

inline int KDateNavigator::dayToIndex(int dayNum)
{
  int row, col;
  row = (dayNum+m_fstDayOfWk-1) / 7;
  col = (dayNum+m_fstDayOfWk-1) % 7;
  return row*7+col;
}
