// $Id: kdpdatebutton.cpp,v 1.15 1998/08/20 07:11:17 garbanzo Exp $	

#include <qstring.h>
#include <kapp.h>

#include "kdpdatebutton.h"
#include "kdpdatebutton.moc"

KDateButton::KDateButton(QDate date, int index, 
			 QWidget *parent, const char *name)
  : QLabel(parent, name), currHiliteStyle(NoHilite)
{

  // drop zone
  KDNDDropZone *dropZone = new KDNDDropZone(this, DndText);
  connect(dropZone, SIGNAL(dropAction(KDNDDropZone *)),
	  this, SLOT(doDrop(KDNDDropZone *)));
  connect(dropZone, SIGNAL(dropEnter(KDNDDropZone *)),
	  this, SLOT(doDropEnter(KDNDDropZone *)));

  myFont = font();
  setFrameStyle(QFrame::Box|QFrame::Plain);
  setAlignment(AlignCenter);
  my_index = index;
  bt_Date = date;
  QString tstr;
  tstr.setNum(date.day());
  setText(tstr);
  adjustSize();

  QColor hiliteColor = kapp->winStyleHighlightColor();

  my_NormalPalette = palette();
  my_NormalGroup = QColorGroup(palette().normal().base(),
			       palette().normal().base(),
			       palette().normal().base(),
			       palette().normal().base(),
			       palette().normal().base(),
			       palette().normal().dark(),
			       palette().normal().base());

  my_DisabledGroup = QColorGroup(palette().normal().base(),
			         palette().normal().base(),
				 palette().normal().base(),
				 palette().normal().base(),
				 palette().normal().base(),
				 palette().normal().mid(),
				 palette().normal().base());

  my_HiDisabledGroup = QColorGroup(hiliteColor,
				   hiliteColor,
				   palette().normal().base(),
				   palette().normal().base(),
				   palette().normal().base(),
				   palette().normal().mid(),
				   palette().normal().base());

  my_HiliteGroup = QColorGroup(hiliteColor,
			       hiliteColor,
			       hiliteColor,
			       hiliteColor,
			       hiliteColor,
 			       palette().normal().light(),
			       hiliteColor);

  my_TodayGroup = QColorGroup(palette().normal().dark(),
			      palette().normal().base(),
			      palette().normal().dark(),
			      palette().normal().dark(),
			      palette().normal().dark(),
			      palette().normal().dark(),
			      palette().normal().base());

  my_EventGroup =  QColorGroup(palette().normal().base(),
			       palette().normal().base(),
			       palette().normal().base(),
			       palette().normal().base(),
			       palette().normal().base(),
			       hiliteColor,
			       palette().normal().base());

  my_NormalPalette.setNormal(my_NormalGroup);
  my_NormalPalette.setDisabled(my_DisabledGroup);
  my_HilitePalette = my_NormalPalette;
  my_HilitePalette.setNormal(my_HiliteGroup);
  my_HilitePalette.setDisabled(my_HiDisabledGroup);

  my_TodayPalette = my_NormalPalette;
  my_TodayPalette.setNormal(my_TodayGroup);

  my_EventPalette = my_NormalPalette;
  my_EventPalette.setNormal(my_EventGroup);

  setPalette(my_NormalPalette);
}

KDateButton::~KDateButton()
{
}

inline QDate KDateButton::date()
{
  return bt_Date;
}

void KDateButton::setHiliteStyle(int HiliteStyle)
{
  myFont.setBold(FALSE);
  setFont(myFont);
  switch(HiliteStyle) {
  case NoHilite:
    setPalette(my_NormalPalette);
    break;
  case SelectHilite:
    setPalette(my_HilitePalette);
    break;
  case EventHilite:    
    myFont.setBold(TRUE);
    setFont(myFont);
    setPalette(my_EventPalette);
    break;
  case TodayHilite:
    setPalette(my_TodayPalette);
    break;
  default:
    setPalette(my_NormalPalette);
    break;
  }
  currHiliteStyle = HiliteStyle;
  update();
}

inline int KDateButton::hiliteStyle()
{
  return currHiliteStyle;
}

void KDateButton::setDate(QDate date)
{
  bt_Date = date;
  QString tstr;
  tstr.setNum(date.day());
  setText(tstr);
}

void KDateButton::mousePressEvent(QMouseEvent *e)
{
  bool c;

  c = (e->state() & ControlButton);
  // do the actual work.... :)
  emit selected(bt_Date, my_index, c);
}

void KDateButton::doDrop(KDNDDropZone *dz)
{
  printf("The summary '%s' was dropped on date %s\n",
	 dz->getData(), bt_Date.toString().data());
}

void KDateButton::doDropEnter(KDNDDropZone *)
{
  printf("Entered date %s with drop thingie\n",bt_Date.toString().data());
}
