/* 	$Id: kmonthview.h,v 1.16 1998/08/05 19:48:30 pbrown Exp $	 */

#ifndef _KMONTHVIEW_H 
#define _KMONTHVIEW_H

#include <qlabel.h>
#include <qframe.h>
#include <qdatetm.h>
#include <qlistbox.h>
#include <qlayout.h>
#include <qintdict.h>

#include "qdatelist.h"
#include "calobject.h"
#include "kdpevent.h"
#include "kpbutton.h"
#include "ksellabel.h" 

class EventListBoxItem: public QListBoxItem
{
 public:
  EventListBoxItem(const char *s);
  void setText(const char *s)
    { QListBoxItem::setText(s); }
  void setRecur(bool on) 
    { recur = on; }
  void setAlarm(bool on)
    { alarm = on; }

 protected:
  virtual void paint(QPainter *);
  virtual int height(const QListBox *) const;
  virtual int width(const QListBox *) const;
 private:
  bool    recur;
  bool    alarm;
  QPixmap alarmPxmp;
  QPixmap recurPxmp;
};

class KNoScrollListBox: public QListBox {
  Q_OBJECT
 public:
  KNoScrollListBox(QWidget *parent=0, const char *name=0)
    :QListBox(parent, name)
    {
      clearTableFlags();
      setTableFlags(Tbl_clipCellPainting | Tbl_cutCellsV | Tbl_snapToVGrid |
		    Tbl_scrollLastHCell| Tbl_smoothHScrolling);
    }
  ~KNoScrollListBox() {}

 signals:
  void shiftDown();
  void shiftUp();

 protected slots:
  void keyPressEvent(QKeyEvent *);
  void keyReleaseEvent(QKeyEvent *);
  void mousePressEvent(QMouseEvent *);
};

class KSummaries: public KNoScrollListBox {
  Q_OBJECT
 public:
  KSummaries(QWidget    *parent=0, 
	     CalObject  *calendar = 0, 
	     QDate       qd       = QDate::currentDate(),
	     int         index    = 0,
	     const char *name=0);
  ~KSummaries() {}
  QDate getDate() { return(myDate); }
  void setDate(QDate);
  void calUpdated();

 signals:
  void daySelected(int index);
  void editEventSignal(KDPEvent *);

 protected slots:
  void itemHighlighted(int);
  void itemSelected(int);

 private:
   QDate               myDate;
   int                 idx;
   CalObject          *myCal;
   QList<KDPEvent>    events;
   QIntDict<KDPEvent> *currIdxs; 
};

class KDPMonthView: public QFrame {
   Q_OBJECT
 public:
   KDPMonthView(CalObject *calendar,
		QWidget    *parent   = 0, 
		QDate       qd       = QDate::currentDate(),
		const char *name     = 0);
   ~KDPMonthView();

   enum { EVENTADDED, EVENTEDITED, EVENTDELETED };

 public slots:
   void selectDates(const QDateList);
   void changeEventDisplay(KDPEvent *, int);

 signals:
   void newEventSignal(QDateTime);
   void editEventSignal(KDPEvent *);
   void deleteEventSignal(KDPEvent *);
   void datesSelected(const QDateList);

 protected slots:
   void updateView();
   void resizeEvent(QResizeEvent *);
   void goBackYear();
   void goForwardYear();
   void goBackMonth();
   void goForwardMonth();
   void goBackWeek();
   void goForwardWeek();
   void daySelected(int index);

 protected:
   void viewChanged();
   
 private:
   // date range label.
   QLabel           *dispLabel;
   // day display vars
   QFrame           *vFrame;
   QGridLayout      *dayLayout;
   QLabel           *dayNames[7];
   KSelLabel        *dayHeaders[42];
   KSummaries       *daySummaries[42];
   bool              shortdaynames;
   
   // display control vars
   QFrame           *cFrame;
   QVBoxLayout      *cLayout;
   KPButton         *upYear;
   KPButton         *upMonth;
   KPButton         *upWeek;
   KPButton         *downWeek;
   KPButton         *downMonth;
   KPButton         *downYear;

   // state data.
   QDate             myDate;
   QDateList         selDates;
   CalObject        *myCal;
   QList<int>        selDateIdxs;
};

#endif
 
