// Context-sensitive help system for KDE/Qt
// (c) 1998 Preston Brown
// part of the KOrganizer project

#include <kapp.h>
#include <qwidget.h>
#include <qfileinf.h>
#include <qmsgbox.h>

#include "kontexthelp.h"


KontextHelp::KontextHelp(const char *baseName, QWidget *parent) 
  : QObject()
{
  helpFlag = FALSE;

  myParent = parent;

  // install application-global event filter
  kapp->installEventFilter(this);

  baseFileName = baseName;
}

KontextHelp::~KontextHelp()
{

}

void KontextHelp::setHelpMode(bool mode)
{
  if (mode) {
    helpFlag = TRUE;
    KApplication::setOverrideCursor(crossCursor);
  } else {
    helpFlag = FALSE;
    KApplication::restoreOverrideCursor();
  }
}

void KontextHelp::doHelp(const char *wClass, const char *wName)
{  
  QString filename;
  QFileInfo qfi;

  filename = baseFileName;
  filename.detach();
  if (wName != NULL) {
    filename += wName;
    filename += ".html";
  } else {
    filename += wClass;
    filename += ".html";
  }
  qfi.setFile(filename.data());
  if (!qfi.exists()) {
    QString infoStr = "There is no context-sensitive help available\n"
      "for the item you selected.\n\n"
      "if it was available, it would have been in:\n";
    infoStr += filename.data();
    
    QMessageBox::information(myParent, "Help Not Available",
			     infoStr.data(),
			     QMessageBox::Ok);
  } else
    kapp->invokeHTMLHelp(filename.data(), "");
}

bool KontextHelp::eventFilter( QObject *obj, QEvent *event ) 
{
  if (helpFlag && (event->type() == Event_MouseButtonPress)) {
      QMouseEvent *e = (QMouseEvent*) event; 
      if ( e->button() == LeftButton ) {
	setHelpMode(FALSE);
	if (obj->name())
	  doHelp(obj->className(), obj->name());
	else
	  doHelp(obj->className(), 0L);
	return TRUE;
      }
  } 
  return FALSE;                               // don't eat event
}

