/* 
   KOrganizer (c) 1997,1998 Preston Brown
   All code is covered by the GNU Public License

   $Id: main.cpp,v 1.29 1998/08/05 19:48:35 pbrown Exp $	
*/

#include <signal.h>
#include <sys/types.h>
#include <qdir.h>

#include <kapp.h>
#include <kconfig.h>

#include "calobject.h"
#include "topwidget.h"
#include "calendarlatexprinter.h"
#include "version.h"

KApplication *korganizer;
TopWidget *mainwin;
CalObject *cal;


// display items that are imminent over the next several days as listed
void display_imminent(int numdays, QString fileName)
{
  QDate currDate(QDate::currentDate());
  KDPEvent *currEvent;
  
  if (fileName.isEmpty()) {
    printf(i18n("No default calendar, and none was specified on the command line.\n"));
    exit(0);
  }
  if (!cal->load(fileName)) {
    printf(i18n("Error reading from default calendar.\n"));
    exit(0);
  }

  for (int i = 1; i <= numdays; i++) {
    QList<KDPEvent> tmpList(cal->getEventsForDate(currDate, TRUE));
    printf("%s\n",currDate.toString().data());
    printf("---------------------------------\n");
    if (tmpList.first())
      for (currEvent = tmpList.first(); currEvent; currEvent = tmpList.next())
	currEvent->print(KDPEvent::ASCII);
    else
      printf("(none)\n");
    printf("\n");
    currDate = currDate.addDays(1);
  }
  exit(0);
}

void print_help()
{
  printf("KOrganizer %s\n\n",korgVersion);
  printf(i18n("X11/Qt Options:\n"));
  printf("-display <display>\n");
  printf("-geometry <geometry>\n"); 
  printf("-name <name>\n");
  printf("-title <title>\n");
  printf("-cmap\n\n");
  printf(i18n("KOrganizer Options:\n"));
  printf("-c or --calendar <calendar>\n");
  printf("-l or --list\n");
  printf("-s or --show <numdays>\n");
  printf("-h or --help\n");
  exit(0);
}

int main (int argc, char **argv)
{

  TopWidget *mainwin;
  KConfig *config;
  QString fn;
  QString cfgPath;
  QDir qd;
  QFile pidFile;
  QString pidFileName;
  cal = new CalObject(); // here so we can display stuff (PGB 07/27/98)

  pidFileName.sprintf("%s/share/apps/korganizer",
		      KApplication::localkdedir().data());
  qd.setPath(pidFileName.data());
  if (!qd.exists())
    qd.mkdir(pidFileName.data(), TRUE);


  cfgPath = KApplication::localconfigdir();
  cfgPath += "/korganizerrc";

  config = new KConfig(cfgPath.data());
  config->setGroup("General");

  fn = config->readEntry("Current Calendar");
  delete config;
 
  // process command line arguments
  if (argc > 1) {
    for (int i = 0; i < argc; i++) {
      if (!strcasecmp(argv[i], "-h") || !strcasecmp(argv[i], "--help"))
	print_help();
      else if (!strcasecmp(argv[i], "-c") || 
	       !strcasecmp(argv[i], "--calendar")) {
	i++;
	fn = argv[i];
      } else if (!strcasecmp(argv[i], "-l") ||
		 !strcasecmp(argv[i], "--list")) {

	display_imminent(1, fn);
      } else if (!strcasecmp(argv[i], "-s") ||
		 !strcasecmp(argv[i], "--show")) {
	i++;
	display_imminent(atoi(argv[i]), fn);
      } 
    } 
  }

  // only do this if we have a calendar to pass as an argument
  if(!fn.isEmpty()) {
    bool startDaemon = FALSE;
    // we need to see if alarmd is already running, and if not, start it up.
    char pidStr[25];
    pidFileName += "/alarmd.pid";
    pidFile.setName(pidFileName.data());

    if (pidFile.exists() && !fn.isEmpty()) {
      if(pidFile.open(IO_ReadOnly)) {
	pid_t pid;
	pidFile.readLine(pidStr, 24);
	pidFile.close();
	pid = atoi(pidStr);
	if (kill(pid, SIGUSR1) < 0) {
	  // stale lockfile
	  unlink(pidFileName.data()); // get rid of old lockfile
	  startDaemon = TRUE;
	}
      }
    }

    if (!pidFile.exists() && !fn.isEmpty())
      startDaemon = TRUE;
    
    // start daemon if lockfile doesn't exist or file exists and no proc
    if (startDaemon) {
      QString execStr;
      execStr.sprintf("%s/alarmd %s",
		      KApplication::kde_bindir().data(),
		      fn.data());
      system(execStr.data());
    }
  }
  
  // those options which should exit are gone by now.
  korganizer = new KApplication(argc, argv, "korganizer");
  mainwin = new TopWidget(cal, fn);
  if (korganizer->isRestored()) {
    if (KTopLevelWidget::canBeRestored(1))
      //      mainwin->restoreSession(1);
      ;
  }

  //korganizer->enableSessionManagement(); // should I be using this?!?
  korganizer->setMainWidget(mainwin);
  mainwin->show();
  korganizer->exec();
  
  // cleanup time
  delete mainwin;
  CalendarLatexPrinter::getInstance(NULL); // deletes
  delete cal;
  delete korganizer;

  return 0;
}

