// $Id: optionsdlg.cpp,v 1.22 1998/08/20 04:21:34 pbrown Exp $

#include <qlayout.h>
#include <qlabel.h>
#include <qgrpbox.h>
#include <qbttngrp.h>
#include <qlined.h>
#include <kapp.h>
#include <qbttngrp.h>

#include "optionsdlg.h"
#include "optionsdlg.moc"

#include "printdialog.h"

///////////////////////////////////////////////////////////////////////////////
char* OptionsDialog::labeltext[4][5] = {	
  { "User name","Email","","","" },
  { "Select your Time Zone","Time Format","Date format","","" },
  {"","","","",""},
  {"Latex executable","Dvipr executable", "DVIPS executable",
   "DVI viewer executable","PS viewer executable" }
};

char* OptionsDialog::confignames[4][5] = {
  {"user_name", "user_email","","",""},
  {"","","","",""},
  {"","","","",""},
  {"latex_executable", "dvipr_executable",
   "dvips_executable", "dvi_viewer_executable",
   "ps_viewer_executable"}
};


OptionsDialog::OptionsDialog( CalObject *calPtr, const char *title, 
			      QWidget *parent, char *name )
  : KPropDlg( KPropDlg::TREE, KPropDlg::OK | 
	      KPropDlg::CANCEL | KPropDlg::APPLY | 
	      KPropDlg::NEXT | KPropDlg::PREV, title, 0, name )
{
  // Set calendar and get configuration  
  calendar = calPtr;
  config = kapp->getConfig();

  // First page
  personalFrame = addPage( -1, i18n("Personal Settings"));
  setupMainTab();

  // Second page
  timeFrame = addPage( -1, i18n("Time & Date"));
  setupTimeTab();

  fontsFrame = addPage( -1, "Fonts");
  setupFontsTab();

  colorsFrame = addPage( -1, "Colors");
  setupColorsTab();

  viewsFrame = addPage( -1, "Views");
  setupViewsTab();

  displayFrame = addPage( -1, "Display");
  setupDisplayTab();

  // Third page
  printerFrame = addPage( -1, i18n("Printing"));
  setupPrinterTab();

  // Layout
  setMinimumSize(600, 300);

  // Set up calling chain.
  connect(this, SIGNAL(configChanged()), parent, SLOT(configDialogChanged()));

  // read settings
  getConfig();

  // Setup primary page for display
  showPage(personalFrame);
}

///////////////////////////////////////////////////////////////////////////////

OptionsDialog::~OptionsDialog()
{
}

///////////////////////////////////////////////////////////////////////////////

void OptionsDialog::setupMainTab()
{
  config->setGroup("Personal Settings");
  
  QGroupBox *gb;
  
  gb = new QGroupBox(personalFrame);
  gb->move(10,10);

  QLabel *nameLabel;
  nameLabel = new QLabel(gb);
  nameLabel->setText("Name:");
  nameLabel->adjustSize();
  nameLabel->move(10, 20);

  QLabel *emailLabel;
  emailLabel = new QLabel(gb);
  emailLabel->setText("Email Address:");
  emailLabel->adjustSize();
  emailLabel->move(10, nameLabel->geometry().bottomLeft().y()+20);

  nameEdit = new QLineEdit(gb);
  nameEdit->setFixedWidth(150);
  nameEdit->adjustSize();
  nameEdit->move(emailLabel->geometry().topRight().x()+10, 10);

  emailEdit = new QLineEdit(gb);
  emailEdit->setFixedWidth(150);
  emailEdit->adjustSize();
  emailEdit->move(emailLabel->geometry().topRight().x()+10, 
		  emailLabel->geometry().topRight().y());
  
  gb->adjustSize();
  gb->setMinimumWidth(300);
}

///////////////////////////////////////////////////////////////////////////////

void OptionsDialog::setupTimeTab()
{
  // Modifications by Fester
  
  QGroupBox *gb;
  gb = new QGroupBox(timeFrame);
  gb->setTitle("Time and Date");
  gb->move(10,10);

  QLabel *timeLabel;
  timeLabel = new QLabel(gb);
  timeLabel->setText("Time Format:");
  timeLabel->adjustSize();
  timeLabel->move(10, 20);
  
  QLabel *tzLabel;
  tzLabel = new QLabel(gb);
  tzLabel->setText("TimeZone:");
  tzLabel->adjustSize();
  tzLabel->move(10, timeLabel->geometry().bottomLeft().y()+20);

  QLabel *dateLabel;
  dateLabel = new QLabel(gb);
  dateLabel->setText("Date Format:");
  dateLabel->adjustSize();
  dateLabel->move(10, tzLabel->geometry().bottomLeft().y()+20);

  timeCombo = new QComboBox(gb);
  timeCombo->insertItem("12 Hour (AM/PM)");
  timeCombo->insertItem("24 Hour");
  timeCombo->adjustSize();
  timeCombo->move(timeLabel->geometry().topRight().x()+10,
		  timeLabel->geometry().topRight().y());

  tzCombo = new QComboBox(FALSE, gb);
  const char *tzList[] = { "-1200", "-1130", "-1100", "-1030", "-1000",
			   "-0930", "-0900", "-0830", "-0800", "-0730",
			   "-0700", "-0630", "-0600", "-0530", "-0500",
			   "-0430", "-0400", "-0330", "-0300", "-0230",
			   "-0200", "-0130", "-0100", "-0030", "+0000",
			   "+0030", "+0100", "+0130", "+0200", "+0230",
			   "+0300", "+0330", "+0400", "+0430", "+0500",
			   "+0530", "+0600", "+0630", "+0700", "+0730",
			   "+0800", "+0830", "+0900", "+0930", "+1000",
			   "+1030", "+1100", "+1130", "+1200", 0L };

  tzCombo->insertStrList(tzList);
  tzCombo->adjustSize();
  tzCombo->move(timeCombo->geometry().topLeft().x(),
		tzLabel->geometry().topRight().y());
  
  gb->adjustSize();
  gb->setMinimumWidth(300);
}

///////////////////////////////////////////////////////////////////////////////

void OptionsDialog::setupViewsTab()
{
  // I'm working on this section. Expect major revision shortly. (Fester)

//	QLabel *dummy = new QLabel("Under Construction", viewFrame);
//	dummy->adjustSize();




}
///////////////////////////////////////////////////////////////////////////////
void OptionsDialog::setupFontsTab()
{
  // I'm working on this section. Expect major revision shortly. (Fester)

//	QLabel *dummy = new QLabel("Under Construction", viewFrame);
//	dummy->adjustSize();




}
///////////////////////////////////////////////////////////////////////////////
void OptionsDialog::setupColorsTab()
{
  // I'm working on this section. Expect major revision shortly. (Fester)
  
//	QLabel *dummy = new QLabel("Under Construction", viewFrame);
//	dummy->adjustSize();




}
///////////////////////////////////////////////////////////////////////////////
void OptionsDialog::setupDisplayTab()
{
  // I'm working on this section. Expect major revision shortly. (Fester)

//	QLabel *dummy = new QLabel("Under Construction", viewFrame);
//	dummy->adjustSize();




}
///////////////////////////////////////////////////////////////////////////////

void OptionsDialog::setupPrinterTab()
{
  int ypos = 10;
  config->setGroup("Printer");
  
  for (int i=0; i!=5; i++) {
    ypos = 10 + i * 30;
    labelList[Printer][i] = new QLabel( printerFrame );
    labelList[Printer][i]->setGeometry( 10, ypos, 200, 20 );
    labelList[Printer][i]->setMinimumSize( 10, 10 );
    labelList[Printer][i]->setMaximumSize( 32767, 32767 );
    labelList[Printer][i]->setText( i18n(labeltext[Printer][i]) );
    
    lineEditList[Printer][i] = new QLineEdit( printerFrame, "Latex executable" );
    lineEditList[Printer][i]->setGeometry( 160, ypos, 260, 20 );
    lineEditList[Printer][i]->setMinimumSize( 10, 10 );
    lineEditList[Printer][i]->setMaximumSize( 32767, 32767 );
    lineEditList[Printer][i]->setText(config->readEntry(confignames[Printer][i]));
  }
}

///////////////////////////////////////////////////////////////////////////////

void OptionsDialog::getConfig()
{
  QString tmpStr;
  int i1, i2;

  // Personal Frame
  config->setGroup("Personal Settings");

  tmpStr = config->readEntry("user_name");
  nameEdit->setText(tmpStr.data());

  tmpStr = config->readEntry("user_email");
  emailEdit->setText(tmpStr.data());

  // Time and Date Frame
  config->setGroup("Time & Date");
  
  tmpStr = config->readEntry("Format");
  if (tmpStr == "12")
    timeCombo->setCurrentItem(0);
  else
    timeCombo->setCurrentItem(1);

  i1 = config->readNumEntry("Time Zone");
  i2 = 25;

  if (i1 % 30) {
    // it has minutes
    i1 -= 30;
    i2 += 1;
  }
  i1 = i1 / 100; // get just hours

  i2 += (i1 * 2);
  
  tzCombo->setCurrentItem(i2);
}

void OptionsDialog::setConfig() 
{
  // Personal Frame
  config->setGroup("Personal Settings");

  config->writeEntry("user_name", nameEdit->text());
  config->writeEntry("user_email", emailEdit->text());

  
  // Time and Date Frame
  config->setGroup("Time & Date");

  config->writeEntry("Format", 
		     (timeCombo->currentItem() == 0 ? 12 : 24));
  config->writeEntry("Time Zone", tzCombo->currentText());
  
  config->setGroup("Printer");
  for (int i=0; i!=5; i++) {
    config->writeEntry(confignames[Printer][i], 
		       lineEditList[Printer][i]->text());
  }	    
  
  // Sync the configuration cache to disk. Is this really
  // necessary right here?
  config->sync();
  
  // Signal applications on notification list to update their state
  // information from the new cache.
  emit configChanged();
}

void OptionsDialog::closeEvent(QCloseEvent *)
{
  emit closed(this);
}

///////////////////////////////////////////////////////////////////////////////

