/*

   	 $Id: printcontrol.cpp,v 1.1 1998/07/05 22:41:48 boloni Exp $	

	 Requires the Qt and KDE widget libraries, available at no cost at
	 http://www.troll.no and http://www.kde.org respectively
	 
	 Copyright (C) 1997, 1998 
	 Preston Brown (preston.brown@yale.edu)
	 Fester Zigterman (F.J.F.ZigtermanRustenburg@student.utwente.nl)
	 Ian Dawes (iadawes@globalserve.net)
     Neil Hart (nhart@magma.ca)
	 
	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.
	 
	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
	 GNU General Public License for more details.
	 
	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "printcontrol.h"
#include "printcontrol.moc"

KDPPrintController::KDPPrintController( ) 
{
	printDialog = NULL;
}

KDPPrintController::~KDPPrintController( )
{
	if ( printDialog != NULL )
		delete printDialog;
}

void KDPPrintController::Print_Setup( QWidget* parent )
{
	prt.setup( parent );
}

void KDPPrintController::Print( QWidget* parent, CalObject* cal )
{
	parentWidget = parent;
	calendar = cal;
	if ( printDialog == NULL )
	{
		printDialog = new KDPPrintDialog( );
		printDialog->show();
		connect( printDialog, SIGNAL(buttonPressed(bool)), this,
			SLOT(dialogClosed(bool)) );
	}
}

void KDPPrintController::dialogClosed( bool Ok )
{
	// make the dialog disappear so the user knows it's working
	// FIXME - we should test that the pointer isn't null
	printDialog->hide();

	if ( Ok )
	{
		// user pressed 'OK', so print the job
		QDate startDate = printDialog->getStartDate();
		QDate endDate = printDialog->getEndDate();
		int format = printDialog->getFormat();
		int layout = printDialog->getLayout();

		// need to override printer orientation to suit layout
		switch ( layout )
		{
			case 0:				// full page
				prt.setOrientation( QPrinter::Portrait );
				break;
			case 1:				// half page
				prt.setOrientation( QPrinter::Landscape );
				break;
		}

		if ( format == 0 )
		{
			switch ( layout )
			{
				case 0:				// full page
				{
					KDPPrintJobFullList printjob( &prt, calendar,
						startDate, endDate );
					if ( printjob.init() )
						printjob.print( parentWidget );
					break;
				}
				case 1: 			// half page
				{
					KDPPrintJobHalfList printjob( &prt, calendar,
						startDate, endDate );
					if ( printjob.init() )
						printjob.print( parentWidget );
					break;
				}
			}
		}

	}
	// now get rid of the dialog
	delete printDialog;
	printDialog = NULL;
}

