/*

   	 $Id: printjob.cpp,v 1.1 1998/07/05 22:41:48 boloni Exp $	

	 Requires the Qt and KDE widget libraries, available at no cost at
	 http://www.troll.no and http://www.kde.org respectively
	 
	 Copyright (C) 1997, 1998 
	 Preston Brown (preston.brown@yale.edu)
	 Fester Zigterman (F.J.F.ZigtermanRustenburg@student.utwente.nl)
	 Ian Dawes (iadawes@globalserve.net)
     Neil Hart (nhart@magma.ca)
	 
	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.
	 
	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
	 GNU General Public License for more details.
	 
	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "printjob.h"

#define MARGIN 72
#define HALFMARGIN ( MARGIN / 2 )
#define A4HEIGHT		842
#define A4WIDTH			595
#define B5HEIGHT		729
#define B5WIDTH			516
#define LETTERHEIGHT	792
#define LETTERWIDTH		612
#define LEGALHEIGHT		1008
#define LEGALWIDTH		612
#define EXECHEIGHT		720
#define EXECWIDTH		540

KDPPrintJobFull::KDPPrintJobFull( QPrinter* printer, CalObject* calendar,
	QDate , QDate ) : KDPPrintJob( printer )
{
}

int KDPPrintJobFull::getPrintableWidth( QPrinter::PageSize pagesize )
{
	switch ( pagesize )
	{
		case QPrinter::A4:
			return A4WIDTH - (2 * MARGIN); break;
		case QPrinter::B5:
			return B5WIDTH - (2 * MARGIN); break;
		case QPrinter::Letter:
			return LETTERWIDTH - (2 * MARGIN); break;
		case QPrinter::Legal:
			return LEGALWIDTH - (2 * MARGIN); break;
		case QPrinter::Executive:
			return EXECWIDTH - (2 * MARGIN); break;
		default:
			return 0; break;
	}
}

int KDPPrintJobFull::getPrintableHeight( QPrinter::PageSize pagesize )
{
	switch ( pagesize )
	{
		case QPrinter::A4:
			return A4HEIGHT - (2 * MARGIN); break;
		case QPrinter::B5:
			return B5HEIGHT - (2 * MARGIN); break;
		case QPrinter::Letter:
			return LETTERHEIGHT - (2 * MARGIN); break;
		case QPrinter::Legal:
			return LEGALHEIGHT - (2 * MARGIN); break;
		case QPrinter::Executive:
			return EXECHEIGHT - (2 * MARGIN); break;
		default:
			return 0; break;
	}
}


bool KDPPrintJobFullList::init( )
{
	if ( prtPainter == NULL )
		return false;

	KDPPrintObject* ppol;
	
	int printableWidth = getPrintableWidth( prt->pageSize() );
	int printableHeight = getPrintableHeight( prt->pageSize() );

	// set the printer viewport to one inch margins
	prtPainter->setWindow( 0, 0, printableWidth, printableHeight );
	prtPainter->setViewport( MARGIN, MARGIN,
		printableWidth, printableHeight );

	// set lastPage and lastOffset to be the beginning of the first page
	lastPage = 1;
	lastOffset = 0;

	for ( ppol = poList->first(); ppol != 0; ppol = poList->next() )
		if ( !( ppol->init( printableWidth, printableHeight, &lastPage,
				&lastOffset) ) )
			return false;

	return true;
}

bool KDPPrintJobFullList::print( QWidget* )
{
	KDPPrintObject* ppol;
	int currentPage;

	// Cycle through the list of pages, from 1 to lastPage. For each
	//  page, ask each print object whether it can print on that
	//  page.
	for ( currentPage = 1; currentPage <= lastPage; currentPage++ ) 
	{
		for ( ppol = poList->first(); ppol != 0; ppol = poList->next() )
			if ( !( ppol->print( currentPage ) ) )
				return false;
		if ( currentPage != lastPage )
			prt->newPage();
	}

	return true;
}

KDPPrintJobFullList::KDPPrintJobFullList( QPrinter* printer,
	CalObject* calendar, QDate startDate, QDate endDate )
	: KDPPrintJobFull( printer, calendar, startDate, endDate )
{
	poList = new QList<KDPPrintObjectList>;
	poList->setAutoDelete( TRUE );
	
	QDate tempDate = startDate;
	while ( tempDate <= endDate )
	{
		poList->append( new KDPPrintObjectList( prtPainter, calendar,
			tempDate ) );
		tempDate = tempDate.addDays( 1 );
	}

	lastPage = 0;
	lastOffset = 0;
}

KDPPrintJobFullList::~KDPPrintJobFullList( )
{
	delete poList;
}

KDPPrintJobHalf::KDPPrintJobHalf( QPrinter* printer, CalObject* ,
	QDate , QDate ) : KDPPrintJob( printer )
{
	if ( prtPainter != NULL )
	{
		// KDPPrintJob constructor created a QPainter. Let's change the 
		// font size to suit the fact that we're printing two pages on one
		QFont ourFont = prtPainter->font();
		int currentSize = ourFont.pointSize();
		currentSize /= 1.414;					// divide by root 2
		ourFont.setPointSize( currentSize );
		prtPainter->setFont( ourFont );

		// now, let's rotate the painter 
		//prtPainter->rotate( 90 );
	}
}

int KDPPrintJobHalf::getPrintableWidth( QPrinter::PageSize pagesize )
{
	switch ( pagesize )
	{
		case QPrinter::A4:
			return (A4HEIGHT / 2) - (2 * HALFMARGIN); break;
		case QPrinter::B5:
			return (B5HEIGHT / 2) - (2 * HALFMARGIN); break;
		case QPrinter::Letter:
			return (LETTERHEIGHT / 2) - (2 * HALFMARGIN); break;
		case QPrinter::Legal:
			return (LEGALHEIGHT / 2) - (2 * HALFMARGIN); break;
		case QPrinter::Executive:
			return (EXECHEIGHT / 2) - (2 * HALFMARGIN); break;
		default:
			return 0; break;
	}
}

int KDPPrintJobHalf::getPrintableHeight( QPrinter::PageSize pagesize )
{
	switch ( pagesize )
	{
		case QPrinter::A4:
			return A4WIDTH - (2 * HALFMARGIN); break;
		case QPrinter::B5:
			return B5WIDTH - (2 * HALFMARGIN); break;
		case QPrinter::Letter:
			return LETTERWIDTH - (2 * HALFMARGIN); break;
		case QPrinter::Legal:
			return LEGALWIDTH - (2 * HALFMARGIN); break;
		case QPrinter::Executive:
			return EXECWIDTH - (2 * HALFMARGIN); break;
		default:
			return 0; break; 
	}
}


bool KDPPrintJobHalfList::init( )
{
	if ( prtPainter == NULL )
		return false;

	KDPPrintObject* ppol;
	
	int printableWidth = getPrintableWidth( prt->pageSize() );
	int printableHeight = getPrintableHeight( prt->pageSize() );

	// set the printer viewport to half inch margins
	prtPainter->setWindow( 0, 0, printableWidth, printableHeight );
	prtPainter->setViewport( HALFMARGIN, HALFMARGIN,
		printableWidth, printableHeight );

	// set lastPage and lastOffset to be the beginning of the first page
	lastPage = 1;
	lastOffset = 0;

	for ( ppol = poList->first(); ppol != 0; ppol = poList->next() )
		if ( !( ppol->init( printableWidth, printableHeight, &lastPage,
				&lastOffset) ) )
			return false;

	return true;
}

bool KDPPrintJobHalfList::print( QWidget* parent )
{
	KDPPrintObject* ppol;
	int currentPage;

	int printableWidth = getPrintableWidth( prt->pageSize() );
	int printableHeight = getPrintableHeight( prt->pageSize() );

	// Cycle through the list of pages, from 1 to lastPage. For each
	//  page, ask each print object whether it can print on that
	//  page. Because we're printing two logical pages on every printer
	//  page, we want to set the viewport differently for odd-numbered
	//  logical pages, and to issue a newPage after every even-numbered
	//  logical page.
	for ( currentPage = 1; currentPage <= lastPage; currentPage++ ) 
	{
		prtPainter->setWindow( 0, 0, printableWidth, printableHeight );
		if ( (currentPage % 2) == 0 )
			// it's an even-numbered logical page
			prtPainter->setViewport( ( printableWidth + (3 * HALFMARGIN)),
				HALFMARGIN, printableWidth, printableHeight );
		else
			prtPainter->setViewport( HALFMARGIN, HALFMARGIN,
				printableWidth, printableHeight );

		for ( ppol = poList->first(); ppol != 0; ppol = poList->next() )
			if ( !( ppol->print( currentPage ) ) )
				return false;
		if ( ((currentPage % 2) == 0) && (currentPage != lastPage) )
			// issue a new page after printing an even-numbered logical 
			//  page, unless this is the last page
			prt->newPage();
	}

	return true;
}

KDPPrintJobHalfList::KDPPrintJobHalfList( QPrinter* printer,
	CalObject* calendar, QDate startDate, QDate endDate )
	: KDPPrintJobHalf( printer, calendar, startDate, endDate )
{
	poList = new QList<KDPPrintObjectList>;
	poList->setAutoDelete( TRUE );
	
	QDate tempDate = startDate;
	while ( tempDate <= endDate )
	{
		poList->append( new KDPPrintObjectList( prtPainter, calendar,
			tempDate ) );
		tempDate = tempDate.addDays( 1 );
	}

	lastPage = 0;
	lastOffset = 0;
}

KDPPrintJobHalfList::~KDPPrintJobHalfList( )
{
	delete poList;
}
