/*

   	 $Id: printobject.h,v 1.1 1998/07/05 22:41:49 boloni Exp $	

	 Requires the Qt and KDE widget libraries, available at no cost at
	 http://www.troll.no and http://www.kde.org respectively
	 
	 Copyright (C) 1997, 1998 
	 Preston Brown (preston.brown@yale.edu)
	 Fester Zigterman (F.J.F.ZigtermanRustenburg@student.utwente.nl)
	 Ian Dawes (iadawes@globalserve.net)
     Neil Hart (nhart@magma.ca)
	 
	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.
	 
	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
	 GNU General Public License for more details.
	 
	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _PRINTOBJECT_H
#define _PRINTOBJECT_H

#include <qpainter.h>
#include <qprinter.h>
#include <qstrlist.h>
#include <qdatetm.h>
#include <qlist.h>
#include <qstring.h>
#include <qfontmet.h>

#include "calobject.h"
#include "kdpevent.h"
#include "kdpeventlist.h"

/**
 *
 * This is the top-level abstract class of print objects. The subclasses 
 *  represent printing day, list or month views on logical pages.
 *
 * @short top-level abstract class of print objects
 * @author Neil Hart
 * @version $Revision: 1.1 $
 */
class KDPPrintObject
{
public:
	/**
	 *
	 * Constructs a new print object
	 *
	 * @param painter is the QPainter we're printing on
	 *
	 */
  KDPPrintObject( QPainter* painter, CalObject* calendar, QDate date );
  virtual ~KDPPrintObject( );

	/**
	 *
	 * Initialises the print object, by determining how many pages it will take
	 *  to print the desired view.
	 *
	 * Note that the last two parameters, pageNum and pageOffset, are passed
	 *  by reference. These will be updated with the new page number and 
	 *  offset after we've printed 
	 * 
	 * @return FALSE if we run into problems, TRUE otherwise
	 * @param width is the width of the logical page on which to print
	 * @param height is the height of the logical page on which to print
	 * @param pageNum is the logical page on which we're currently printing
	 * @param pageOffset is the offset on that logical page
	 *
	 */
	virtual bool init( int width, int height, int* pageNum,
		int* pageOffset ) = 0;

	/**
	 *
	 * Prints the requested page of the print object.
	 *
	 * @return FALSE if we run into problems, TRUE otherwise
	 * @param page is the page to print
	 */
	virtual bool print( int page ) = 0;

protected:

	// paint object for printer
	QPainter* prtPainter;
	// list of events for this date
	KDPEventList *currEvents;
	// and the date is...
	QDate thisDate;
	// list of lines to be printed
	QStrList* lineList;
	// first logical page on which this object appears
	int firstPage;
	// last logical page on which this object appears
	int lastPage;
	// offset from start of first page at which this object starts
	int startOffset;
	// height of the page on which we can print
	int pageHeight;
	// width of the page on which we can print
	int pageWidth;

};

/**
 *
 * This class prints the events of a single day in list view format.
 *
 * @short prints events of one day in list format
 * @author Neil Hart
 * @version $Revision: 1.1 $
 */
class KDPPrintObjectList : public KDPPrintObject
{
public:
  /**
   *
   * Constructs a new print object
   *
	*/
  KDPPrintObjectList( QPainter* painter, CalObject* calendar, QDate date );
  virtual ~KDPPrintObjectList( );

	/**
	 *
	 * Initialises the print object, by determining how many pages it will take
	 *  to print the desired view.
	 *
	 * Note that the last two parameters, pageNum and pageOffset, are passed
	 *  by reference. These will be updated with the new page number and 
	 *  offset after we've printed 
	 * 
	 * @return FALSE if we run into problems, TRUE otherwise
	 * @param width is the width of the logical page on which to print
	 * @param height is the height of the logical page on which to print
	 * @param pageNum is the logical page on which we're currently printing
	 * @param pageOffset is the offset on that logical page
	 *
	 */
	bool init( int width, int height, int* pageNum, int* pageOffset );

	/**
	 *
	 * Prints the requested page of the print object.
	 *
	 * @return FALSE if we run into problems, TRUE otherwise
	 * @param page is the page to print
	 */
	bool print( int page );

protected:

};

#endif // _PRINTOBJECT_H

