/*

   	 $Id: printdialog.cpp,v 1.1 1998/07/05 21:36:43 boloni Exp $	

	 Requires the Qt and KDE widget libraries, available at no cost at
	 http://www.troll.no and http://www.kde.org respectively
	 
	 Copyright (C) 1997, 1998 
	 Preston Brown (preston.brown@yale.edu)
	 Fester Zigterman (F.J.F.ZigtermanRustenburg@student.utwente.nl)
	 Ian Dawes (iadawes@globalserve.net)
	 Neil Hart (nhart@magma.ca)
	 Laszlo Boloni (boloni@cs.purdue.edu)
	 
	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.
	 
	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
	 GNU General Public License for more details.
	 
	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <kapp.h>

#include "printdialog.h"
#include "printdialog.moc"
#include "calendarlatexprinter.h"

KDPPrintDialog* KDPPrintDialog::_instance = 0;

KDPPrintDialog* KDPPrintDialog::getInstance(CalObject *cal) {
    if (_instance == 0) {
	_instance = new KDPPrintDialog("KOrganizer - Print",cal);
    } else {
	_instance->calendar = cal;
    }
    return _instance;    
}

KDPPrintDialog::KDPPrintDialog( const char* name, CalObject *cal)
	// note that this has to be a modeless dialog, otherwise the
	// KDatePicker will never get keyboard focus
	: QDialog( 0, name, FALSE )
{
    int ypos = 30;
    calendar = cal;

    startDateEditLabel = new QLabel( this, "Label_1" );
    startDateEditLabel->setGeometry( 10, ypos, 140, 20 );
    startDateEditLabel->setMinimumSize( 10, 10 );
    startDateEditLabel->setMaximumSize( 32767, 32767 );
    startDateEditLabel->setText( "Start Date:" );
    // startDateEditLabel->setAlignment( );
    // startDateEditLabel->setMargin( -1 );

    startDateEdit = new KDateEdit( this );
    startDateEdit->setGeometry( 160, ypos, 140, 20 );
    startDateEdit->setEnabled( TRUE );

    ypos = 60; 

    endDateEditLabel = new QLabel( this, "Label_2" );
    endDateEditLabel->setGeometry( 10, ypos, 140, 20 );
    endDateEditLabel->setMinimumSize( 10, 10 );
    endDateEditLabel->setMaximumSize( 32767, 32767 );
    endDateEditLabel->setText( "End Date:" );  

    // endDateEditLabel->setAlignment( );
    // endDateEditLabel->setMargin( -1 );

    endDateEdit = new KDateEdit( this );
    endDateEdit->setGeometry( 160, ypos, 140, 20 );
    endDateEdit->setEnabled( TRUE );
    
    ypos = 90;

    formatComboLabel = new QLabel( this, "Label_3" );
    formatComboLabel->setGeometry( 10, ypos, 140, 20 );
    formatComboLabel->setMinimumSize( 10, 10 );
    formatComboLabel->setMaximumSize( 32767, 32767 );
    formatComboLabel->setText( "Format:" );
    // formatComboLabel->setAlignment( );
    // formatComboLabel->setMargin( -1 );

    formatCombo = new QComboBox( FALSE, this, "Combo_1" );
    formatCombo->setGeometry( 150, 90, 140, 20 );
    formatCombo->setMinimumSize( 10, 10 );
    formatCombo->setMaximumSize( 32767, 32767 );
    formatCombo->insertItem( "Upcoming events & todos", 0 );
    formatCombo->insertItem( "Weekly view", 1 );
    formatCombo->insertItem( "Monthly view", 2 );
    formatCombo->insertItem( "Custon view", 3 );

    ypos = 120;

    layoutComboLabel = new QLabel( this, i18n("Layout") );
    layoutComboLabel->setGeometry( 10, ypos, 140, 20 );
    layoutComboLabel->setMinimumSize( 10, 10 );
    layoutComboLabel->setMaximumSize( 32767, 32767 );
    //layoutComboLabel->setText( "Layout:" );
    // layoutComboLabel->setAlignment( );
    // layoutComboLabel->setMargin( -1 );

    layoutCombo = new QComboBox( FALSE, this, "Combo_2" );
    layoutCombo->setGeometry( 150, ypos, 140, 20 );
    layoutCombo->setMinimumSize( 10, 10 );
    layoutCombo->setMaximumSize( 32767, 32767 );
    layoutCombo->insertItem( "Full Page", 0 );
    layoutCombo->insertItem( "Half Page", 1 );

    ypos = 150;

    printButton = new QPushButton( klocale->translate("Print"), this );
    printButton->setGeometry( 10, ypos, 70, 30 );
    connect( printButton, SIGNAL(clicked()), SLOT( printButtonPressed()) );

    previewButton = new QPushButton( klocale->translate("Preview"), this );
    previewButton->setGeometry( 90, ypos, 70, 30 );
    connect( previewButton, SIGNAL(clicked()), SLOT( previewButtonPressed()) );

    cancelButton = new QPushButton( klocale->translate("Cancel"), this );
    cancelButton->setGeometry( 170, 150, 70, 30 );
    connect( cancelButton, SIGNAL(clicked()), SLOT( cancelButtonPressed()) );

    ypos = 180;
    /*
    QCheckBox *printmode = new QCheckBox( klocale->translate("LaTeX mode printing"), this);    
    printmode->setGeometry( 10, ypos, 120, 30 );
    */
}

KDPPrintDialog::~KDPPrintDialog( )
{
	delete startDateEditLabel;
	delete startDateEdit;
	delete endDateEditLabel;
	delete endDateEdit;
	delete formatComboLabel;
	delete formatCombo;
	delete layoutComboLabel;
	delete layoutCombo;
}

void KDPPrintDialog::printButtonPressed( )
{
    CalendarLatexPrinter *cpl = CalendarLatexPrinter::getInstance(calendar);
    cpl->setDates(getStartDate(), getEndDate());
    cpl->generate();
    cpl->print();
    hide();
}

void KDPPrintDialog::previewButtonPressed( )
{
    // for printing
    CalendarLatexPrinter *cpl = CalendarLatexPrinter::getInstance(calendar);
    cpl->setDates(getStartDate(), getEndDate());
    cpl->generate();
    cpl->preview();
}

void KDPPrintDialog::cancelButtonPressed( )
{
    //	emit buttonPressed( false );
	hide();
}

QDate KDPPrintDialog::getStartDate()
{
	return startDateEdit->getDate();
}

QDate KDPPrintDialog::getEndDate()
{
	return endDateEdit->getDate();
}

int KDPPrintDialog::getFormat()
{
	return formatCombo->currentItem();
}

int KDPPrintDialog::getLayout()
{
	return layoutCombo->currentItem();
}

