/*
         $Id: topwidget.cpp,v 1.145 1998/08/24 13:10:24 pbrown Exp $

	 Requires the Qt and KDE widget libraries, available at no cost at
	 http://www.troll.no and http://www.kde.org respectively
	 
	 Copyright (C) 1997, 1998 
	 Preston Brown (preston.brown@yale.edu)
	 Fester Zigterman (F.J.F.ZigtermanRustenburg@student.utwente.nl)
	 Ian Dawes (iadawes@globalserve.net)
	 Christopher Beard
	 Laszlo Boloni (boloni@cs.purdue.edu)

	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.
	 
	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
	 GNU General Public License for more details.
	 
	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <sys/types.h>
#include <signal.h>

#include <qfiledlg.h>
#include <qcursor.h>
#include <qmlined.h>
#include <qmsgbox.h>
#include <kiconloader.h>
#include <qtimer.h>

#include "misc.h"
#include "version.h"
#include "printdialog.h"
#include "koarchivedlg.h"
#include "calendarlatexprinter.h"

#include "topwidget.h"
#include "topwidget.moc"	

QList<TopWidget> TopWidget::windowList;

TopWidget::TopWidget(CalObject *cal, QString fn, QWidget *, const char *name) 
  : KTopLevelWidget( name )
{
  QString titleStr = "KOrganizer ";
  titleStr += korgVersion;
  setCaption(titleStr.data());
  searchDlg = 0L;
  viewMode = AGENDAVIEW;
  setMinimumSize(600,400);	// make sure we don't get resized too small...
  // context-sensitive help
  kHelp = new KontextHelp("korganizer/korg_", this);

  // get calendar from passed argument to the constructor.
  calendar = cal;

  // create the main layout frames.
  mainFrame    = new QFrame(this);
  dayWeekFrame = new QFrame(mainFrame);

  // create and setup the datenav for the day/week views.
  dateNavFrame = new QFrame(dayWeekFrame);
  dateNavFrame->setFrameStyle(QFrame::Panel|QFrame::Sunken);
  dateNavigator = new KDateNavigator(dateNavFrame, calendar, TRUE,
				     QDate::currentDate(),"DateNavigator");
  dateNavigator->resize(160, 150);
  dateNavigator->move(2, 2);
  dateNavFrame->resize(dateNavigator->width()+4, dateNavigator->height()+4);

  // add this instance of the window to the static list.
  windowList.append(this);
  initCalendar(fn);

  // create the main data display views.
  agendaView = new KAgenda(calendar, dayWeekFrame);
  listView   = new ListView(calendar, dayWeekFrame);
  monthView  = new KDPMonthView(calendar, mainFrame);
  todoList   = new TodoView(calendar, dayWeekFrame);

  // setup toolbar, menubar and status bar, NOTE: this must be done
  // after the widget creation, because setting the menubar, toolbar
  // or statusbar will generate a call to updateRects, which assumes
  // that all of them are around.
  initMenus();
  initToolBar();

  //sb = new KStatusBar(this, "status_0");
  //setStatusBar(sb);

  // hook up the signals/slots of all widgets together so communication
  // can happen when things get clicked.
  hookupSignals();

  // this is required for KTopLevelWidget for geometry management
  setView(mainFrame, TRUE);

  changeView(viewMode);
  goToday();
}

TopWidget::~TopWidget(void)
{
  // Free memory allocated for widgets (not children)
  // Take this window out of the window list.
  windowList.removeRef( this );

  //KTLW doesn't delete it's main view
  delete tb;
  delete menubar;
  delete mainFrame;
}


void TopWidget::initCalendar(QString fn) 
{
  // read the settings from the config file
  readSettings();
  // cmd line argument for filename overrides config files 
  if(!fn.isEmpty())
    fileName = fn;

  if ((!fileName.isEmpty()) && (windowList.count() == 1)) {
    if (!(calendar->load(fileName))) {
      // while failing to load, the calendar object could 
      // have become partially populated.  Clear it out.
      calendar->close();
      fileName = "";
    }
  }
  unsetModified();
}

// hey, it works. but some KPaneds would be nice...
void TopWidget::updateRects()
{
  KTopLevelWidget::updateRects();
  dayWeekFrame->move(5,5);
  dayWeekFrame->resize(mainFrame->width()-10, mainFrame->height()-10);
  if (viewMode == TODOVIEW) {
      todoList->move(5,5);
      todoList->resize(dayWeekFrame->width()-10, dayWeekFrame->height()-10);
  } else {
  dateNavFrame->move(0,0);
  agendaView->resize(dayWeekFrame->width()-
		     dateNavFrame->geometry().topRight().x()-5,
		     dayWeekFrame->height());
  agendaView->move(dateNavFrame->geometry().topRight().x()+5, 
		   dateNavFrame->geometry().topRight().y());
  
  listView->resize(dayWeekFrame->width()-
		   dateNavFrame->geometry().topRight().x()-5,
		   dayWeekFrame->height());
  listView->move(dateNavFrame->geometry().topRight().x()+5, 
		 dateNavFrame->geometry().topRight().y());
  
  todoList->move(dateNavFrame->geometry().topLeft().x(),
		 dateNavFrame->geometry().bottomLeft().y()+5);
  todoList->resize(dateNavFrame->width(), 
		   dayWeekFrame->height()-dateNavFrame->height());

  monthView->move(5,5);
  monthView->resize(mainFrame->width()-10, mainFrame->height()-10);
  }
}

void TopWidget::readSettings()
{
  QString str;

  // read settings from the KConfig, supplying reasonable
  // defaults where none are to be found

  config = korganizer->getConfig();

  int windowWidth = 600;
  int windowHeight = 400;
  viewMode = AGENDAVIEW;

  config->setGroup("Recently Opened Files");
  str = config->readEntry("1", "");
  if (!str.isEmpty())
    recentFileList.append(str.data());
  str = config->readEntry("2", "");
  if (!str.isEmpty())
    recentFileList.append(str.data());
  str = config->readEntry("3", "");
  if (!str.isEmpty())
    recentFileList.append(str.data());
  str = config->readEntry("4", "");
  if (!str.isEmpty())
    recentFileList.append(str.data());
  str = config->readEntry("5", "");
  if (!str.isEmpty())
    recentFileList.append(str.data());
	
  config->setGroup("General");
  str = config->readEntry("Width");
  if (!str.isEmpty())
    windowWidth = str.toInt();
  str = config->readEntry("Height");
  if (!str.isEmpty())
    windowHeight = str.toInt();
  this->resize(windowWidth, windowHeight);
  str = config->readEntry("Current View");
  if (!str.isEmpty())
    viewMode = str.toInt();
  str = config->readEntry("Current Calendar");
  if (!str.isEmpty())
    fileName = str;

  // Setting defaults for the printing programs
  // we assume /usr/bin for the latex stuff, as this is where
  // it exists on most Linux distributions.  However, we could
  // make this smarter...

  // preston aug 19: this is evil.  We should really do readEntry's
  // with defaults, and not write anything here. I'll fix it later
  config->setGroup("Printer");

  str = config->readEntry("latex_executable");
  if (str.isEmpty()) {
    config->writeEntry("latex_executable", "/usr/bin/latex");
  }
  
  str = config->readEntry("dvipr_executable");
  if (str.isEmpty()) {
    config->writeEntry("dvipr_executable", 
		       "/usr/bin/dvipr");
  }
  
  str = config->readEntry("dvips_executable");
  if (str.isEmpty()) {
    config->writeEntry("dvips_executable", 
		       "/usr/bin/dvips");
  }
  
  str = config->readEntry("dvi_viewer_executable");
  if (str.isEmpty()) {
    config->writeEntry("dvi_viewer_executable", 
		       korganizer->kde_bindir()+"/kdvi");
  }
  
  str = config->readEntry("ps_viewer_executable");
  if (str.isEmpty()) {
    config->writeEntry("ps_viewer_executable", 
		       korganizer->kde_bindir()+"/kghostview");
  }
  
  // user information...
  config->setGroup("Personal Settings");
  str = config->readEntry("user_name");
  if (str.isEmpty()) {
    config->writeEntry("user_name",
		       calendar->getOwner().data());
  } else {
    // override what was gotten from the password file
    calendar->setOwner(str);
  }
  str = config->readEntry("user_email");
  if (str.isEmpty()) {
    config->writeEntry("user_email",
		       calendar->getEmail().data());
  } else {
    // override
    calendar->setEmail(str);
  }
  // time info
  config->setGroup("Time & Date");
  str = config->readEntry("Time Zone");
  if (!str.isEmpty())
    calendar->setTimeZone(str.toInt());

  config->sync();
}

void TopWidget::writeSettings()
{
	
  config = korganizer->getConfig();

  config->setGroup("Recently Opened Files");
	
  QString recent_number;
  for(int i = 0; i <(int) recentFileList.count();i++){
    recent_number.setNum(i+1);
    config->writeEntry(recent_number.data(),recentFileList.at(i));
  }

  QString tmpStr;
  config->setGroup("General");

  tmpStr.sprintf("%d", this->width() );
  config->writeEntry("Width",	tmpStr);
	
  tmpStr.sprintf("%d", this->height() );
  config->writeEntry("Height",	tmpStr);

  tmpStr.sprintf("%d", viewMode);
  config->writeEntry("Current View", tmpStr);
  config->writeEntry("Current Calendar", fileName);

  config->sync();

}

void TopWidget::goToday()
{
  dateNavigator->selectDates(QDate::currentDate());
  updateView(dateNavigator->getSelected());
}

void TopWidget::initMenus()
{
  QPixmap pixmap;
	
  fileMenu = new QPopupMenu;
  pixmap = Icon("mini/korganizer.xpm");
  fileMenu->insertItem(pixmap,i18n("&New Window"), this,
		       SLOT(newTopWidget()), CTRL+Key_N);
  fileMenu->insertSeparator();
  pixmap = Icon("filenew.xpm");
  fileMenu->insertItem(pixmap, i18n("&Open..."), this,
		       SLOT(file_open()), CTRL+Key_O);
  recentPop = new QPopupMenu;
  fileMenu->insertItem(i18n("Open &Recent"), recentPop);
  connect( recentPop, SIGNAL(activated(int)), SLOT(file_openRecent(int)));

  // setup the list of recently used files
  recentPop->clear();
  for (int i = 0; i < (int)recentFileList.count(); i++)
    recentPop->insertItem(recentFileList.at(i));
  if(!fileName.isEmpty()) 
    add_recent_file();  

  fileMenu->insertItem(i18n("&Close"), this,
		       SLOT(file_close()), CTRL+Key_W);
  fileMenu->insertSeparator();

  pixmap = Icon("filefloppy.xpm");
  fileMenu->insertItem(pixmap, i18n("&Save"), this,
		       SLOT(file_save()), CTRL+Key_S);
  fileMenu->insertItem(i18n("Save &As..."), this,
		       SLOT(file_saveas()));

  fileMenu->insertSeparator();

  fileMenu->insertItem(i18n("&Import From Ical..."), this,
  		       SLOT(file_import()));
  fileMenu->insertItem(i18n("&Merge Calendar..."), this,
		       SLOT(file_merge()));
  
  fileMenu->setItemEnabled(fileMenu->insertItem(i18n("Archive Old Entries..."),
						this,
						SLOT(file_archive())),
			   FALSE);
  

  fileMenu->insertSeparator();
  pixmap = Icon("fileprint.xpm");
  fileMenu->insertItem(pixmap, i18n("&Print..."), this,
		       SLOT(print()), CTRL+Key_P);  
  
  fileMenu->insertSeparator();
  fileMenu->insertItem(i18n("&Quit"), this,
		       SLOT(file_quit()), CTRL+Key_Q);

  editMenu = new QPopupMenu;
  //put stuff for editing here 
  editMenu->insertItem(i18n("C&ut"), this,
		       SLOT(edit_cut()), CTRL+Key_X);
  editMenu->insertItem(i18n("&Copy"), this,
		       SLOT(edit_copy()), CTRL+Key_C);
  editMenu->insertItem(i18n("&Paste"), this,
		       SLOT(edit_paste()), CTRL+Key_V);

  viewMenu = new QPopupMenu;

  pixmap = Icon("dayicon.xpm");
  viewMenu->insertItem(pixmap, i18n("&Day"), this,
			SLOT( view_list() ) );
  pixmap = Icon("weekicon.xpm");
  viewMenu->insertItem(pixmap, i18n("&Week"), this,
			SLOT( view_week() ) );
  pixmap = Icon("monthicon.xpm");
  viewMenu->insertItem(pixmap, i18n("&Month"), this,
			SLOT( view_month() ) );
  pixmap = Icon("checkedbox.xpm");
  viewMenu->insertItem(pixmap,i18n("&Todo list"), this,
			SLOT( view_todolist() ) );
  viewMenu->insertSeparator();
  viewMenu->insertItem( i18n("&Update"), this,
			SLOT( update() ) );
	
  actionMenu = new QPopupMenu;
  pixmap = Icon("newevent.xpm");
  actionMenu->insertItem(pixmap, i18n("New &Appointment"), 
			 this, SLOT(apptmnt_new()));
  actionMenu->insertItem(i18n("New E&vent"),
			 this, SLOT(allday_new()));
  actionMenu->insertItem(i18n("&Edit Appointment"), this,
			  SLOT(apptmnt_edit()));
  pixmap = Icon("delete.xpm");
  actionMenu->insertItem(pixmap, i18n("&Delete Appointment"),
			 this, SLOT(apptmnt_delete()));
  actionMenu->insertSeparator();
  pixmap = Icon("search.xpm");
  actionMenu->insertItem(pixmap,i18n("&Search..."), this,
			 SLOT(action_search()));

  pixmap = Icon("send.xpm");
  actionMenu->insertItem(pixmap, i18n("&Mail Appointment..."), this,
			 SLOT(action_mail()));

  optionsMenu = new QPopupMenu;
  optionsMenu->insertItem(i18n("Show &Tool Bar"));
  optionsMenu->insertItem(i18n("Show St&atus Bar"));
  optionsMenu->insertSeparator();
			  
  optionsMenu->insertItem(i18n("&Edit Options"), 
			  this, SLOT(edit_options()));

  helpMenu = new QPopupMenu;
  helpMenu->insertItem(i18n("&Contents"), this,
		       SLOT(help_contents()));
  helpMenu->insertItem(i18n("Help &On"), this,
		       SLOT(help_on()));
  helpMenu->insertSeparator();
  helpMenu->insertItem(i18n("&About"), this,
		       SLOT(help_about())); 
  helpMenu->insertItem(i18n("About &KDE"), this,
		       SLOT(help_aboutKDE()));
  // construct a basic menu
  menubar = new KMenuBar(this, "menubar_0");
  menubar->insertItem(i18n("&File"), fileMenu);
  menubar->insertItem(i18n("&Edit"), editMenu);
  menubar->insertItem(i18n("&View"), viewMenu);
  menubar->insertItem(i18n("&Actions"), actionMenu);
  menubar->insertItem(i18n("&Options"), optionsMenu);
  menubar->insertSeparator();
  menubar->insertItem(i18n("&Help"), helpMenu);

  setMenu(menubar);
}

void TopWidget::initToolBar()
{
  QPixmap pixmap;
  QString dirName;

  tb = new KToolBar(this);
  pixmap = Icon("fileopen.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(file_open()), TRUE, 
		   i18n("Open A Calendar"));
	
  pixmap = Icon("filefloppy.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(file_save()), TRUE, 
		   i18n("Save This Calendar"));

  pixmap = Icon("fileprint.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(print()), TRUE, 
		   i18n("Print"));

  QFrame *sepFrame = new QFrame(tb);
  sepFrame->setFrameStyle(QFrame::VLine|QFrame::Raised);
  tb->insertWidget(0, 10, sepFrame);

  pixmap = Icon("newevent.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(apptmnt_new()), TRUE, 
		   i18n("New Appointment"));
  pixmap = Icon("delete.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(apptmnt_delete()), TRUE,
		   i18n("Delete Appointment"));

  pixmap = Icon("search.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(action_search()), TRUE,
		   i18n("Search For an Appointment"));

  pixmap = Icon("send.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(action_mail()), TRUE,
		   i18n("Mail Appointment"));

  sepFrame = new QFrame(tb);
  sepFrame->setFrameStyle(QFrame::VLine|QFrame::Raised);
  tb->insertWidget(0, 10, sepFrame);

  pixmap = Icon("todayicon.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(goToday()), TRUE, i18n("Go to Today"));

  sepFrame = new QFrame(tb);
  sepFrame->setFrameStyle(QFrame::VLine|QFrame::Raised);
  tb->insertWidget(0, 10, sepFrame);

  pixmap = Icon("listicon.xpm");
  tb->insertButton(pixmap, 0, SIGNAL(clicked()), this,
		   SLOT(view_list()), TRUE,
		   i18n("List View"));
  pixmap = Icon("agenda.xpm");
  tb->insertButton(pixmap, 0, SIGNAL(clicked()), this,
		   SLOT(view_day()), TRUE,
		   i18n("Schedule View"));

  pixmap = Icon("monthicon.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(view_month()), TRUE,
		   i18n("Month View"));

  pixmap = Icon("checkedclipboard.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(view_todolist()), TRUE,
		   i18n("Todo list view"));

  addToolBar(tb);
}

void TopWidget::hookupSignals()
{
  // SIGNAL/SLOTS FOR DATE SYNCHRO
    
  connect(listView, SIGNAL(datesSelected(const QDateList)),
	  dateNavigator, SLOT(selectDates(const QDateList)));
  connect(agendaView, SIGNAL(datesSelected(const QDateList)),
	  dateNavigator, SLOT(selectDates(const QDateList)));
  connect(monthView, SIGNAL(datesSelected(const QDateList)),
	  dateNavigator, SLOT(selectDates(const QDateList)));    
  connect(dateNavigator, SIGNAL(datesSelected(const QDateList)),
	  this, SLOT(updateView(const QDateList)));
    
  // SIGNALS/SLOTS FOR LIST VIEW
  connect(listView, SIGNAL(editEventSignal(KDPEvent *)),
	  this, SLOT(editEvent(KDPEvent *)));
  connect(listView, SIGNAL(deleteEventSignal(KDPEvent *)), 
	  this, SLOT(deleteEvent(KDPEvent *)));

  // SIGNALS/SLOTS FOR DAY/WEEK VIEW
  connect(agendaView,SIGNAL(newEventSignal(QDateTime)),
		this, SLOT(newEvent(QDateTime)));
  connect(agendaView, SIGNAL(editEventSignal(KDPEvent *)),
	  this, SLOT(editEvent(KDPEvent *)));
  connect(agendaView, SIGNAL(deleteEventSignal(KDPEvent *)), 
	  this, SLOT(deleteEvent(KDPEvent *)));

  // SIGNALS/SLOTS FOR MONTH VIEW
  connect(monthView, SIGNAL(editEventSignal(KDPEvent *)),
	  this, SLOT(editEvent(KDPEvent *)));
  connect(monthView, SIGNAL(deleteEventSignal(KDPEvent *)),
	  this, SLOT(deleteEvent(KDPEvent *)));


  // SIGNALS/SLOTS FOR TODO LIST
  connect(todoList, SIGNAL(editEventSignal(KDPEvent *)),
	  this, SLOT(editEvent(KDPEvent *)));

  // CONFIGURATION SIGNALS/SLOTS

  // Notification list. Add a connection to widgets when they
  // need to know about changed in configuration.
  connect(this, SIGNAL(configChanged()), agendaView, SLOT(updateConfig()));
  connect(this, SIGNAL(configChanged()), listView, SLOT(updateConfig()));
 
  // MISC. SIGNALS/SLOTS
  connect(calendar, SIGNAL(calUpdated(KDPEvent *)), this, SLOT(setModified()));
}

// most of the changeEventDisplays() right now just call the view's
// total update mode, but they SHOULD be recoded to be more refresh-efficient.
void TopWidget::changeEventDisplay(KDPEvent *which, int action)
{
  dateNavigator->updateView();
  switch(viewMode) {
  case LISTVIEW:
    listView->changeEventDisplay(which, action);
    break;
  case AGENDAVIEW:
    agendaView->changeEventDisplay(which, action);
    break;
  case MONTHVIEW:
    monthView->changeEventDisplay(which, action);
    break;
  default:
    debug(i18n("we don't handle updates for that view, yet.\n"));
    qApp->beep();
    return;
  }
}

void TopWidget::changeView(int newViewMode)
{
  viewMode = newViewMode;

  updateView(dateNavigator->getSelected());

  switch(viewMode) {
  case AGENDAVIEW:
    dayWeekFrame->show();
    dateNavFrame->show();
    monthView->hide();
    listView->hide();
    agendaView->show();
    break;
  case LISTVIEW:
    dayWeekFrame->show();
    dateNavFrame->show();
    monthView->hide();
    agendaView->hide();
    listView->show();
    break;
  case MONTHVIEW:
      dateNavFrame->show();
      monthView->show();
      dayWeekFrame->hide();
      break; 
  case TODOVIEW:
    dateNavFrame->hide();
    dayWeekFrame->show();
    monthView->hide();
    agendaView->hide();
    listView->hide();
    todoList->show();
    break;
  default:
    debug(i18n("we don't handle that view yet.\n"));
    break;
  }
  updateRects();
}

void TopWidget::updateView(const QDateList selectedDates)
{ 
  switch (viewMode) {
  case AGENDAVIEW:
      agendaView->selectDates(selectedDates);
    break;
  case LISTVIEW:
      listView->selectDates(selectedDates);
    break;
  case MONTHVIEW:
      monthView->selectDates(selectedDates);
    break;
  default:
    break;
  }
}

void TopWidget::updateView()
{
  // update the current view with the current dates from the date navigator
  QDateList tmpList(FALSE); // we want a shallow copy
  tmpList = dateNavigator->getSelected();

  // if no dates are supplied, we should refresh the dateNavigator too...
  dateNavigator->updateView();
  updateView(tmpList);
}

void TopWidget::file_open()
{
  QString defaultPath;
  
  // be nice and tidy and have all the files in a nice easy to find place
  defaultPath = KApplication::localkdedir().data();
  defaultPath += "/share/apps/korganizer/";
  
  if (isModified()) {
    switch(QMessageBox::warning(this, 
				i18n("Modified Calendar Warning"),
				i18n("You are opening a new calendar, but the current"
				     "one has been modified.\nAre you sure you want to"
				     "do this? All changes will be lost!\n\n"),
				i18n("&Ok"), 
				i18n("&Save"),
				i18n("&Cancel"),
				2)) {
    case 0: // OK
      
      fileName = QFileDialog::getOpenFileName(defaultPath, "*.vcs", 
					      this);
            
      // We sanity check the returned filename...
      if (!fileName.isEmpty()) {
	QFileInfo tf("",fileName);
	if(!tf.isDir()) {
	  calendar->close();
	  if(calendar->load(fileName)) {
	    unsetModified();
	    add_recent_file();
	    updateView();
	    todoList->updateView();
	    return;
	  } 
	}
      }
      
      // Spit out an indication that the file was not opened,
      // either a bad file name was specified or the operation
      // was cancelled.
      QMessageBox::warning(this, i18n("KOrganizer Error"),
			   i18n("You did not specify a valid filename."),
			   i18n("&Ok"));

      break;

    case 1: // save

      file_save();
      file_open();
      break;

    case 2: // Cancel
      // exit
      break;
			
    } // switch

  } else {

    fileName = QFileDialog::getOpenFileName(defaultPath, "*.vcs",
					    this);

    // If file dialog box was cancelled (trap for (null) 
    if(fileName.isNull()) {
      return;
    }
    
    // We sanity check the returned filename
    if (!fileName.isEmpty()) {
      QFileInfo tf("",fileName);
      if(!tf.isDir()) {
	calendar->close();
	if(calendar->load(fileName)) {
	  unsetModified();
	  add_recent_file();
	  updateView();
	  todoList->updateView();
	  return;
	} 
      }
    }
    
    // Spit out an indication that the file was not opened,
    // either a bad file name was specified or the operation
    // was cancelled.
    QMessageBox::warning(this, i18n("KOrganizer Error"),
			 i18n("You did not specify a valid filename."),
			 i18n("&Ok"));
  } 
}

void TopWidget::file_openRecent(int i)
{
  if (isModified()) {
		
    switch(QMessageBox::warning(this, 
				i18n("KOrganizer Modified Calendar Warning"),
				i18n("You are opening a new calendar, but the current one "
				     "has been modified.\nAre you sure you want to do this?"
				     " All changes will be lost!\n\n"),
				i18n("&Ok"),
				i18n("&Save"),
				i18n("&Cancel"),
				2)) {
    case 0: // OK

      fileName = recentFileList.at(i);
      
      if (!fileName.isEmpty()) {
	QFileInfo tf("",fileName);
	// sanity check
	if(!tf.isDir()) {
	  calendar->close();
	  if(calendar->load(fileName)) {
	    unsetModified();
	    add_recent_file();
	    updateView();
	    todoList->updateView();
	    return;
	  } 
	}
      }
      
      // Spit out an indication that the file was not opened,
      // either a bad file name was specified or the operation
      // was cancelled.
      QMessageBox::warning(this, i18n("KOrganizer Error"),
			   i18n("You did not specify a valid filename."),
			   i18n("&Ok"));
      
      break;

    case 1: // save

      file_save();
      calendar->close();
      fileName = "";
      file_openRecent(i);
      break;

    case 2: // Cancel
      // exit
      break;
			
    } // switch
  } else {

    fileName = recentFileList.at(i);
    
    if (!fileName.isEmpty()) {
      QFileInfo tf("",fileName);
      // sanity check
      if(!tf.isDir()) {
	calendar->close();
	if(calendar->load(fileName)) {
	  unsetModified();
	  add_recent_file();
	  updateView();
	  todoList->updateView();
	  return;
	} 
      }
    }
    
    // Spit out an indication that the file was not opened,
    // either a bad file name was specified or the operation
    // was cancelled.
    QMessageBox::warning(this, i18n("KOrganizer Error"),
			 i18n("You did not specify a valid filename."),
			 i18n("&Ok"));
  }
}					


void TopWidget::file_import()
{
  // eventually, we will need a dialog box to select import type, etc.
  // for now, hard-coded to ical file, $HOME/.calendar.
  int retVal;
  QString progPath;
  char *tmpFn;

  tmpFn = tmpnam(0);
  progPath.sprintf("%s/korganizer/ical2vcal %s",
		   KApplication::kde_datadir().data(), tmpFn);
  retVal = system(progPath.data());

  if (retVal >= 0 && retVal <= 2) {
    // now we need to MERGE what is in the iCal to the current calendar.
    calendar->load(tmpFn);
    if (!retVal)
      QMessageBox::information(this, i18n("KOrganizer Info"),
			       i18n("KOrganizer succesfully imported and merged your\n"
			       ".calendar file from ical into the currently\n"
			       "opened calendar.\n"),
			       QMessageBox::Ok);
    else
          QMessageBox::warning(this, i18n("ICal Import Successful With Warning"),
			       i18n("KOrganizer encountered some unknown fields while\n"
			       "parsing your .calendar ical file, and had to\n"
			       "discard them.  Please check to see that all\n"
			       "your relevant data was correctly imported.\n"));
  } else if (retVal == -1) {
    QMessageBox::warning(this, i18n("KOrganizer Error"),
			 i18n("KOrganizer encountered some error parsing your\n"
			 ".calendar file from ical.  Import has failed.\n"));
  } else if (retVal == -2) {
    QMessageBox::warning(this, i18n("KOrganizer Error"),
			 i18n("KOrganizer doesn't think that your .calendar\n"
			 "file is a valid ical calendar. Import has failed.\n"));
  }
  updateView();
  todoList->updateView();
}

void TopWidget::file_merge()
{
  QString defaultPath;
  
  // be nice and tidy and have all the files in a nice easy to find place
  defaultPath = KApplication::localkdedir().data();
  defaultPath += "/share/apps/korganizer/";
  
  fileName = QFileDialog::getOpenFileName(defaultPath, "*.vcs",
					  this);
  
  // If file dialog box was cancelled (trap for (null) 
  if(fileName.isNull()) {
    return;
  }
    
  // We sanity check the returned filename
  if (!fileName.isEmpty()) {
    QFileInfo tf("",fileName);
    if(!tf.isDir()) {
      if(calendar->load(fileName)) {
	setModified();
	updateView();
	todoList->updateView();
	return;
      } 
    }
  }
  
  // Spit out an indication that the file was not opened,
  // either a bad file name was specified or the operation
  // was cancelled.
  QMessageBox::warning(this, i18n("KOrganizer Error"),
		       i18n("You did not specify a valid filename."),
		       i18n("&Ok"));
}

void TopWidget::file_archive()
{
  ArchiveDialog ad;

  if (ad.exec()) {
    // ok was pressed.
  }
}

void TopWidget::file_saveas()
{
  QString defaultPath;
  QString saveFilename = fileName;

  saveFilename.detach();
  // Be nice and tidy and have all the files in a nice easy to find place.
  defaultPath = KApplication::localkdedir();

  defaultPath += "/share/apps/korganizer/";

  // bugs in KFileDialog are screwing us up.

  fileName = QFileDialog::getSaveFileName(defaultPath, "*.vcs", this);

  // If file dialog box was cancelled (trap for (null))
  if(fileName.isNull() || fileName.isEmpty()) {
    fileName = saveFilename;
    return;
  }
  
  QFileInfo tf("",fileName);
  if(!tf.isDir()) {
    
    // Force the extension for consistency.
    if(fileName.size() > 3) {
      QString e = fileName.right(4);
      // Extension ending in '.vcs' or anything else '.???' is cool.
      if(e != ".vcs" && e.right(1) != ".")
	// Otherwise, force the default extension.
	fileName += ".vcs";
    }
    
    calendar->save(fileName);
    unsetModified();
    return;
  }
  
  // Spit out an indication that the file was not saved
  // either a bad file name was specified or the operation
  // was cancelled.
  QMessageBox::warning(this, i18n("KOrganizer Error"),
		       i18n("You did not specify a valid filename."),
		       i18n("&Ok"));

  fileName = saveFilename;
  // Prompt for a proper filename.
  file_saveas();
}



void TopWidget::file_close()
{
  if (isModified()) {
    
    switch(QMessageBox::warning(this, i18n("KOrganizer Warning"),
				i18n("The calendar you are closing has been modified.\n"
				     "Do you want to save your changes?"),
				i18n("&Save"),
				i18n("&Don't Save"),
				i18n("&Cancel"),
				0,2)) {
      // Save changes...
    case 0:
      file_save();
      calendar->close();
      fileName = "";
      unsetModified();
      break;
      // Don't save changes...
    case 1:
      calendar->close();
      fileName = "";
      unsetModified();
      break;
      // Cancel operation...
    case 2: 
      break;

    } // switch

  } else {
    
    // Calendar not modified, close without confirmation...
    calendar->close();
    fileName = "";
    unsetModified();
  
  }
  
  updateView();
  todoList->updateView();
  
  // Should we really remove the current window when we close something?
  // right now we don't, but that isn't exactly consistent.
  if( windowList.count() > 1 )
    close();
}

void TopWidget::file_save()
{
  // has this calendar been saved before? 
  if (!fileName.isEmpty()) {
    add_recent_file();
    calendar->save(fileName);
    unsetModified();
  }
  // force it to 'save as'
  else
    file_saveas();
}

void TopWidget::file_quit()
{
  QString tmpfn;

  TopWidget *ptopwidget;
  for (ptopwidget=windowList.first(); ptopwidget != 0; ptopwidget=windowList.next()) {
    if (ptopwidget->isModified()) 
      // if any other windows have modified calendars, we need to
      // set the dirty bit.
      setModified();	
  }
  
  if (!isModified()) {
    writeSettings();
    emit closingDown();
  } else { // modified
    // work around the fact that file_close loses the file name
    tmpfn = fileName;
    file_close(); // see if they want to save the file they are losing
    if (fileName != "")
      // they hit cancel
      return;
    fileName = tmpfn;
    writeSettings();
    emit closingDown();
  }
  if (windowList.count() == 1)
    kapp->quit();
  else
    emit closed(this); // suicide
}

void TopWidget::edit_cut()
{
  KDPEvent *anEvent;
  switch(viewMode) {
  case LISTVIEW:
    anEvent = listView->getSelected();
    break;
  case AGENDAVIEW:
    anEvent = agendaView->getSelected();
    break;
  default:
    QMessageBox::warning(this,i18n("KOrganizer error"),
			 i18n("Unfortunately, we don't handle cut/paste for\n"
			      "that view yet.\n"));
    return;
  }
  if (!anEvent) {
    qApp->beep();
    return;
  }
  calendar->cutEvent(anEvent);
  changeEventDisplay(anEvent, EVENTDELETED);
}

void TopWidget::edit_copy()
{
  KDPEvent *anEvent;
  switch(viewMode) {
  case LISTVIEW:
    anEvent = listView->getSelected();
    break;
  case AGENDAVIEW:
    anEvent = agendaView->getSelected();
    break;
  default:
    QMessageBox::warning(this,i18n("KOrganizer error"),
			 i18n("Unfortunately, we don't handle cut/paste for\n"
			      "that view yet.\n"));
    return;
  }
  if (!anEvent) {
    qApp->beep();
    return;
  }
  calendar->copyEvent(anEvent);
}

void TopWidget::edit_paste()
{
  KDPEvent *pastedEvent;
  QDateList tmpList(FALSE);

  tmpList = dateNavigator->getSelected();
  pastedEvent = calendar->pasteEvent(tmpList.first());
  changeEventDisplay(pastedEvent, EVENTADDED);
}

void TopWidget::edit_options()
{
  OptionsDialog *optionsDlg = new OptionsDialog(calendar, 
						"KOrganizer Configuration Options",
						this);
  connect(optionsDlg, SIGNAL(closed(QWidget *)), 
	  this, SLOT(cleanWindow(QWidget *)));
  optionsDlg->show();
}

// Configuration changed as a result of the options dialog.
// I wanted to break this up, in order to avoid ineffeciency 
// introduced as we were ALWAYS updating configuration
// in multiple widgets regardless of changed in information.
void TopWidget::configDialogChanged()
{
  emit configChanged();
  readSettings(); // this is the best way to assure that we have them back
}

/****************************************************************************/

void TopWidget::newEvent(QDateTime fromHint, QDateTime toHint)
{
  // create empty event win
  EditEventWin *eventWin = new EditEventWin( calendar );

  // put in date hint
  eventWin->newEvent(fromHint, toHint);

  // connect the win for changed events
  connect(eventWin, SIGNAL(eventChanged(KDPEvent *, int)), 
  	  this, SLOT(changeEventDisplay(KDPEvent *, int)));
  connect(eventWin, SIGNAL(closed(QWidget *)),
	  this, SLOT(cleanWindow(QWidget *)));
  connect(this, SIGNAL(closingDown()),
	  eventWin, SLOT(cancel()));
  // show win
  eventWin->show();
}

void TopWidget::newEvent(QDateTime fromHint, QDateTime toHint, bool allDay)
{
  // create empty event win
  EditEventWin *eventWin = new EditEventWin( calendar );

  // put in date hint
  eventWin->newEvent(fromHint, toHint, allDay);

  // connect the win for changed events
  connect(eventWin, SIGNAL(eventChanged(KDPEvent *, int)), 
  	  this, SLOT(changeEventDisplay(KDPEvent *, int)));
  connect(eventWin, SIGNAL(closed(QWidget *)),
	  this, SLOT(cleanWindow(QWidget *)));
  connect(this, SIGNAL(closingDown()),
	  eventWin, SLOT(cancel()));

  // show win
  eventWin->show();
}

void TopWidget::apptmnt_new()
{
  QDate from, to;

  QDateList tmpList(FALSE);
  tmpList = dateNavigator->getSelected();
  from = *tmpList.first();
  to = *tmpList.last();
  
  ASSERT(from.isValid());
  if (!from.isValid()) { // dateNavigator sometimes returns GARBAGE!
    from = QDate::currentDate();
    to = from;
  }

  newEvent(QDateTime(from, QTime(12,0,0)),
	   QDateTime(to, QTime(13,0,0)));
}

void TopWidget::allday_new()
{

  QDate from, to;
  QDateList tmpList(FALSE);
  tmpList = dateNavigator->getSelected();

  from = *tmpList.first();
  to = *tmpList.last();

  ASSERT(from.isValid());
  if (!from.isValid()) {
    from = QDate::currentDate();
    to = from;
  }
  
  newEvent(QDateTime(from, QTime(12,0,0)),
	   QDateTime(to, QTime(12,0,0)), TRUE);
}

void TopWidget::editEvent(KDPEvent *anEvent)
{
  QDateList tmpList(FALSE);
  QDate qd;

  tmpList = dateNavigator->getSelected();
  if(anEvent!=0)
    {
      qd = *tmpList.first();
      EditEventWin *eventWin = new EditEventWin(calendar, 
						anEvent->getTodoStatus());
      eventWin->editEvent(anEvent, qd);

      // connect for changed events
      connect(eventWin, SIGNAL(eventChanged(KDPEvent *, int)), 
      	      this, SLOT(changeEventDisplay(KDPEvent *, int)));
      connect(eventWin, SIGNAL(closed(QWidget *)),
	      this, SLOT(cleanWindow(QWidget *)));
      connect(this, SIGNAL(closingDown()),
	      eventWin, SLOT(cancel()));
      eventWin->show();
    } else {
      qApp->beep();
    }
}

void TopWidget::apptmnt_edit()
{
  KDPEvent *anEvent;
  
  switch(viewMode) {
  case LISTVIEW:
    anEvent = listView->getSelected();
    break;
  case AGENDAVIEW:
    anEvent = agendaView->getSelected();
    break;
  default:
    debug("we don't handle that view yet.");
    return;
  }
  if (!anEvent) {
    qApp->beep();
    return;
  }

  editEvent(anEvent);
}

void TopWidget::apptmnt_delete()
{
  KDPEvent *anEvent;

  switch(viewMode) {
  case LISTVIEW:
    anEvent = listView->getSelected();
    break;
  case AGENDAVIEW:
    anEvent = agendaView->getSelected();
    break;
  default:
    debug(i18n("don't handle deletes from that view, yet.\n"));
    return;
  }
  if (!anEvent) {
    qApp->beep();
    return;
  }

  deleteEvent(anEvent);
}

void TopWidget::deleteEvent(KDPEvent *anEvent)
{
  if (!anEvent) {
    qApp->beep();
    return;
  }

  if (anEvent->doesRecur()) {
    switch(QMessageBox::warning(this, i18n("KOrganizer Confirmation"),
				i18n("This event recurs over multiple dates.\n"
				"Are you sure you want to delete the\n"
				"selected event, or just this instance?\n\n"),
				i18n("&All"),
				i18n("&This"),
				i18n("&Cancel"),
				0,2)) {
    case 0: // all
      calendar->deleteEvent(anEvent);
      changeEventDisplay(anEvent, EVENTDELETED);
      break;
    case 1: // add an exception
	{
	    QDate qd;
	    QDateList tmpList(FALSE);
	    tmpList = dateNavigator->getSelected();
	    qd = *(tmpList.first());
	    if (!qd.isValid()) {
		debug(i18n("no date selected, or invalid date\n"));
		qApp->beep();
		return;
	    }
	    anEvent->addExDate(qd);
	    changeEventDisplay(anEvent, EVENTDELETED);
	    break;
	}
    case -1: // escape pressed
      // ^^^ Mirko --> maybe an default would be better
    case 2: // cancel
      break;
    } // switch
  } else {
    switch(QMessageBox::warning(this, i18n("KOrganizer Confirmation"),
				i18n("Are you sure you want to delete\n"
				     "the selected event?\n\n"),
				i18n("&Ok"),
				i18n("&Cancel"), 0, 
				0, 1)) {
    case 0: // OK
      calendar->deleteEvent(anEvent);
      changeEventDisplay(anEvent, EVENTDELETED);
      break;
    case 1: // Cancel
      break;
    } // switch
  } // if-else
}

/*****************************************************************************/

void TopWidget::action_search()
{
  if (!searchDlg) {
    searchDlg = new SearchDialog(calendar);
    connect(searchDlg, SIGNAL(closed(QWidget *)),
	    this, SLOT(cleanWindow(QWidget *)));
    connect(searchDlg, SIGNAL(editEventSignal(KDPEvent *)),
	    this, SLOT(editEvent(KDPEvent *)));
    connect(searchDlg, SIGNAL(deleteEventSignal(KDPEvent *)), 
	    this, SLOT(deleteEvent(KDPEvent *)));
  }
  searchDlg->show();
  
}

void TopWidget::action_mail()
{
}

void TopWidget::add_recent_file()
{
  const char *rf;

  // check to see if it is already in the list.
  for (rf = recentFileList.first(); rf;
       rf = recentFileList.next()) {
    if (strcmp(rf, fileName.data()) == 0)
      return;  
  }

  if( recentFileList.count() < 5)
    recentFileList.insert(0,fileName.data());
  else {
    recentFileList.remove(4);
    recentFileList.insert(0,fileName.data());
  }
  recentPop->clear();
  for ( int i =0 ; i < (int)recentFileList.count(); i++)
    {
      recentPop->insertItem(recentFileList.at(i));
    }
  if (recentFileList.count() == 0) {
    // disable the "Open Recent" option on the file menu if necessary
    fileMenu->setItemEnabled(1, FALSE);
  }
}

void TopWidget::help_contents()
{
  korganizer->invokeHTMLHelp("korganizer/korganizer.html","");
}

void TopWidget::help_on()
{
  // eventually, display context sensitive online help.
  // until then, just display the widget name that is clicked on.
  kHelp->setHelpMode(TRUE);
}

void TopWidget::help_about()
{
  
  QMessageBox aboutmsg;
  QString tmpStr;
  tmpStr = i18n("KOrganizer v");
  tmpStr += korgVersion;
  tmpStr += i18n("\n(c) 1997, 1998 Preston Brown, Ian Dawes, Fester Zigterman,\n"
		 "Christopher Beard, Neil Hart, Laszlo Boloni, Dan Pilone,\n"
		 "Herwin Jan Steehouwer, Christian Kirsch, and others.\n\n"
		 "This program is free software; you can redistribute it and/or modify\n"
		 "it under the terms of the GNU General Public License as published by\n"
		 "the Free Software Foundation; either version 2 of the License, or\n"
		 "(at your option) any later version.\n"
		 "\n"
		 "This program is distributed in the hope that it will be useful,\n"
		 "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
		 "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
		 "GNU General Public License for more details.\n"
		 "\n"
		 "You should have received a copy of the GNU General Public License\n"
		 "along with this program; if not, write to the Free Software\n"
		 "Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n");

  aboutmsg.information( this, i18n("About KOrganizer"), 
			tmpStr.data(),
			i18n("&Ok") );
}

void TopWidget::help_aboutKDE()
{
  // why is this private???
  // KApplication::aboutKDE();
}

void TopWidget::newTopWidget()
{
  TopWidget *t = new TopWidget(calendar, fileName.data());
  connect(t, SIGNAL(closed(QWidget *)),
	  this, SLOT(cleanWindow(QWidget *)));
  t->show();
}

void TopWidget::view_list()
{
  changeView(LISTVIEW);
}

void TopWidget::view_day()
{
  agendaView->setView( KAgenda::LIST );
  changeView(AGENDAVIEW);
}

void TopWidget::view_workweek()
{
  agendaView->setView(KAgenda::WORKWEEK);
  changeView(AGENDAVIEW);
}

void TopWidget::view_week()
{
  agendaView->setView(KAgenda::WEEK );
  changeView(AGENDAVIEW);
}

void TopWidget::view_month()
{
  changeView(MONTHVIEW);
}

void TopWidget::view_todolist()
{
  changeView(TODOVIEW);
}

void TopWidget::setModified()
{
  QString tmpStr;

  // quick exit if already set
  if (modifiedFlag)
    return;

  if (! fileName.isEmpty() )
    tmpStr.sprintf(i18n("%s (modified) - KOrganizer"), 
		   (fileName.mid(fileName.findRev('/')+1, 
				 fileName.length())).data());
  else
    tmpStr.sprintf(i18n("New Calendar (modified) - KOrganizer"));
  modifiedFlag = TRUE;
  setCaption(tmpStr);
}

void TopWidget::unsetModified()
{
  QString tmpStr;

  if (!fileName.isEmpty())
    tmpStr.sprintf(i18n("%s - KOrganizer"), 
		   (fileName.mid(fileName.findRev('/')+1, 
				 fileName.length())).data());
  else
    tmpStr.sprintf(i18n("New Calendar - KOrganizer"));
  modifiedFlag = FALSE;
  setCaption(tmpStr);

  signalAlarmDaemon();
}

bool TopWidget::isModified()
{
  return modifiedFlag;
}

void TopWidget::closeEvent( QCloseEvent * )
{
  file_quit();
  emit closed(this);
}

void TopWidget::signalAlarmDaemon()
{
  QFile pidFile;
  QString tmpStr;
  pid_t pid;
  char pidStr[25];

  tmpStr.sprintf("%s/share/apps/korganizer/alarmd.pid",
		 KApplication::localkdedir().data());
  pidFile.setName(tmpStr.data());

  // only necessary if the file actually is opened
  if(pidFile.open(IO_ReadOnly)) {
    pidFile.readLine(pidStr, 24);
    pidFile.close();
    pid = atoi(pidStr);
    kill(pid, SIGHUP);
  }
}

inline void TopWidget::cleanWindow(QWidget *window)
{
  //window->hide(); // to make sure it's not visible.
  delete window;
  window = 0L;
}

/**
 *
 *  Latex print engine - --- BL.
 *
 ***/
void TopWidget::print() {
  // and here it goes...
  KDPPrintDialog *printdialog= KDPPrintDialog::getInstance(calendar);
  printdialog->show();
}

